/***********************************************************************/
/*                                                                     */
/*   MODULE:  BDTest/hdlc.c                                            */
/*   DATE:    98/12/10                                                 */
/*   PURPOSE: hdlc(high level data link control) function define.      */
/*                                                                     */
/*---------------------------------------------------------------------*/
/*      Copyright (C) 1998 Samsung Electronics.                        */
/*                                                                     */
/*      programmed by jwCHOI & such                                    */
/*                                                                     */
/***********************************************************************/

/* 			*/
/* Modified by 		*/
/* Dmitriy Cherkashin 	*/
/* dch@ucrouter.ru	*/
/* 2002			*/
/*			*/

#include "ks32c50.h"
#include "evm50100.h"

//#define HDLC_RESET			

U32 gHdlcBaudRate = 1000000;

////////////////////////////////////////////////////////////////////////////////
// set HDLC baud rate //////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

static void HdlcSetBrg(void)
{
 U32 val;
 Disable_Int(nGLOBAL_INT); 			// enable global interrupt
 Print("\nInput Baud Rate (%d)",gHdlcBaudRate);	// print current baud rate
 val = get_number(10, 16);			// read length from console 
 if(val > 0) { gHdlcBaudRate = val; }
 Print("\nHDLC Baud Rate   %d", gHdlcBaudRate);	// print HDLC baud rate
 HDLC_Init();					// init HDLC controller	
 Enable_Int(nGLOBAL_INT); 			// enable global interrupt
}

////////////////////////////////////////////////////////////////////////////////
// Print HDLC Mode Register (HMODE) ////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

static void PrintHMODE(int ch)
{
 U32 reg[2];				
 reg[0] = HMODEA;			// HDLCA mode register
 reg[1] = HMODEB;			// HDLCB mode register

 if(ch >= 0)
  {
   Print("\nHMODE%c HDLC%c Mode register 0x%08x", 'A'+ch, 'A'+ch, reg[ch]);
  }
 else 
  {
   Print("\nHMODE A=0x%08x B=0x%08x", reg[0], reg[1] );
  }

// [0] 	   Multi-Frame in HTxFIFO //////////////////////////////////////////////
 PrintSBIT(ch, "0     Multi-Frame in HTxFIFO", reg, HMODE_MFinFIFO, 
  "1(Multiple frame in FIFO)", "0(Single frame in FIFO)"
 );

#ifdef _S3C4530_
 if(is_s3c4530(syscfg_pd_id)) 
  {
   PrintUBIT(ch, "2     Rx clock inversion(RXCINV)", reg, HMODE_RXCINV);
   PrintUBIT(ch, "3     Tx clock inversion(TXCINV)", reg, HMODE_TXCINV);
  }
#endif 								/* _S3C4530_ */

// [4] Rx Little-Endian mode (RxLittle) ////////////////////////////////////////
 PrintSBIT(ch, "4     Rx Little-Endian mode (RxLittle)", reg, HMODE_RxLittle,
   "1(Little)", "0(Big-endian)"
 );

// [5] Tx Little-Endian mode (TxLittle) ////////////////////////////////////////
 PrintSBIT(ch, "5     Tx Little-Endian mode (TxLittle)",    reg, HMODE_TxLittle, 
   "1(Little)", "0(Big-endian)"
 );

#ifdef _S3C4530_ 
 if(is_s3c4530(syscfg_pd_id))
  {
   PrintUBIT(ch, "6     Rx Transparent mode (RxTRANS)", reg, HMODE_RxTRANS);
   PrintUBIT(ch, "7     Tx Transparent mode (TxTRANS)", reg, HMODE_TxTRANS);
  }
#endif 								/* _S3C4530_ */

 {
// [10:8]  Tx preamble length (TxPL) ///////////////////////////////////////////
  ABIT tpl_bt[] = {
    {"000(1 byte)" , HMODE_TxPL1},	//
    {"001(2 bytes)", HMODE_TxPL2},	//
    {"010(3 bytes)", HMODE_TxPL3},	//
    {"011(4 bytes)", HMODE_TxPL4},	//
    {"100(5 bytes)", HMODE_TxPL5},	//
    {"101(6 bytes)", HMODE_TxPL6},	//
    {"110(7 bytes)", HMODE_TxPL7},	//
    {"111(8 bytes)", HMODE_TxPL8},	//
    {0, 0}
  };  
  PrintABIT(ch, "10:8  Tx preamble length (TxPL)", reg, HMODE_TxPL8, tpl_bt);
 }

 {
// [14:12] Data formats (DF) ///////////////////////////////////////////////////
  ABIT df_bt[] = {
    {"000(NRZ)"       , HMODE_DF_NRZ	  },  
    {"001(NRZI)"      , HMODE_DF_NRZI	  },  
    {"010(FM0)"       , HMODE_DF_FM0	  },  
    {"011(FM1)"       , HMODE_DF_FM1	  },  
    {"100(Manchester)", HMODE_DF_Manchester}, 
    { 0, 0} 
  };

  PrintABIT(ch, "14:12 Data formats (DF)", reg, HMODE_DF, df_bt);
 }

// [18:16] DPLL clock select (DPLLCLK) /////////////////////////////////////////
 {
   ABIT dpll_bt[] = {
     {"000(TXC)"    , HMODE_DPLLCLK_TXC    },
     {"001(RXC)"    , HMODE_DPLLCLK_RXC    },
     {"010(MCLK)"   , HMODE_DPLLCLK_MCLK   },
     {"011(BRGOUT1)", HMODE_DPLLCLK_BRGOUT1},
     {"100(BRGOUT2)", HMODE_DPLLCLK_BRGOUT2},
     { 0, 0}
   };  
  PrintABIT(ch, "18:16 DPLL clock select (DPLLCLK)", reg, HMODE_DPLLCLK, dpll_bt );
 }

// [19]    BRG clock select (BRGCLK) ///////////////////////////////////////////
 PrintSBIT(ch, "19    BRG clock select (BRGCLK)", reg, HMODE_BRGCLK_MCLK2, 
   "1(MCLK2)", "0(RXC)"
 );
  
 {
// [22:20] Tx clock select (TxCLK) /////////////////////////////////////////////
   ABIT txcs_bt[] = {
     {"000(TXC)"    , HMODE_TxCLK_TXC}, 
     {"001(RXC)"    , HMODE_TxCLK_RXC},
     {"010(DPLLOUTT)",HMODE_TxCLK_DPLLOUTT},
     {"011(BRGOUT1)", HMODE_TxCLK_BRGOUT1},
     {"100(BRGOUT2)", HMODE_TxCLK_BRGOUT2},
     {0, 0}
   };  
  PrintABIT(ch, "22:20 Tx clock select (TxCLK)", reg,HMODE_TxCLK, txcs_bt );
 }

// [26:24] Rx clock select (RxCLK) /////////////////////////////////////////////
 {
   ABIT rxcs_bt[] = {
     {"000(TXC)"    , HMODE_RxCLK_TXC},
     {"001(RXC)"    , HMODE_RxCLK_RXC},
     {"010(DPLLOUTR)",HMODE_RxCLK_DPLLOUTR},
     {"011(BRGOUT1)", HMODE_RxCLK_BRGOUT1},
     {"100(BRGOUT2)", HMODE_RxCLK_BRGOUT2},
     { 0, 0}
   };  
   PrintABIT(ch, "26:24 Rx clock select (RxCLK)", reg, HMODE_RxCLK, rxcs_bt);
 }

// [30:28] TXC output pin select (TXCOPS) //////////////////////////////////////
 {
  ABIT txco_bt[] =  {
    {"000(TxCLK)"   , HMODE_TXCOPS_TxCLK   },
    {"001(RxCLK)"   , HMODE_TXCOPS_RxCLK   },
    {"010(BRGOUT1)" , HMODE_TXCOPS_BRGOUT1 },
    {"011(BRGOUT2)" , HMODE_TXCOPS_BRGOUT2 },
    {"100(DPLLOUTT)", HMODE_TXCOPS_DPLLOUTT},
    {"101(DPLLOUTR)", HMODE_TXCOPS_DPLLOUTR},
    { 0, 0}
  };
  PrintABIT(ch, "30:28 TXC output pin select (TXCOPS)", reg, HMODE_TXCOPS, txco_bt);
 }
}

////////////////////////////////////////////////////////////////////////////////
// Print HDLC control register /////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

static void PrintHCON(int ch)
{
 U32 reg[2];				// HCONA or HCONB
 reg[0] = HCONA;			// HCONA 
 reg[1] = HCONB;			// HCONB 

 if(ch >= 0) 
  {
   Print("\nHCON%c HDLC%c Control register.", 'A'+ch, 'A'+ch);
  }
 else
  {
   Print("\nHCON A&B HDLC Control regs A=0x%08x B=0x%08x", reg[0], reg[1]);
  }  

////////////////////////////////////////////////////////////////////////////////	
// HDLC control register (HCON) ////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////	

// [0] Tx reset (TxRS) /////////////////////////////////////////////////////////
 PrintSBIT(ch, "0     Tx reset (TxRS)",	reg, HCON_TxRS, 
   "1(TxFIFO & Tx block reset)", "0(Normal operation)"
 );

// [1] Rx reset (RxRS) /////////////////////////////////////////////////////////
 PrintSBIT(ch, "1     Rx reset (RxRS)",	reg, HCON_RxRS, 
   "1(RxFIFO and Rx block reset)", "0(Normal operation)"
 );

// [2] DMA Tx reset (DTxRS) ////////////////////////////////////////////////////
 PrintSBIT(ch, "2     DMA Tx reset (DTxRS)", reg, HCON_DTxRS, 
   "1(DMA Tx reset)", "0(Normal operation)"
 );

// [3] DMA Rx reset (DRxRS) ////////////////////////////////////////////////////
 PrintSBIT(ch, "3     DMA Rx reset (DRxRS)", reg, HCON_DRxRS, 
   "1(DMA Rx block is reset)", "0(Normal operation)"
 );

// [4] Tx enable (TxEN) ////////////////////////////////////////////////////////
 PrintSBIT(ch, "4     Tx enable (TxEN)", reg, HCON_TxEN, 
   "1(Tx enabled)", "0(Tx disabled)"
 );

// [5] Rx enable (RxEN) ////////////////////////////////////////////////////////
 PrintSBIT(ch, "5     Rx enable (RxEN)", reg, HCON_RxEN, 
   "1(Rx enabled)", "0(Rx disabled)"
 );

// [6] DMA Tx enable (DTxEN) ///////////////////////////////////////////////////
 PrintSBIT(ch, "6     DMA Tx enable (DTxEN)", reg, HCON_DTxEN, 
   "1(DMA Tx enabled)", "0(DMA Tx disabled"
 );

// [7] DMA Rx enable (DRxEN) ///////////////////////////////////////////////////
 PrintSBIT(ch, "7     DMA Rx enable (DRxEN)", reg, HCON_DRxEN, 
   "1(DMA Rx enabled)", "0(DMA Rx disabled)"
 );

// [8] DPLL enable (DPLLEN) ////////////////////////////////////////////////////
 PrintSBIT(ch, "8     DPLL enable (DPLLEN)", reg, HCON_DPLLEN,
   "1(Enable)", "0(Disable)"
 );

// [9] BRG enable (BRGEN) //////////////////////////////////////////////////////

 PrintSBIT(ch, "9     BRG enable (BRGEN)", reg, HCON_BRGEN,
   "1(BRG counter is enabled)", "0(BRG counter is inhibited)"
 );

// [10]    Tx 4 word mode (Tx4WD) //////////////////////////////////////////////
 PrintSBIT(ch, "10    Tx 4 word mode (Tx4WD)", reg, HCON_Tx4WD,
   "1(4-word mode selected)", "0(1-word mode selected)"
 );

// [11]    Rx 4 word mode (Rx4WD) //////////////////////////////////////////////
 PrintSBIT(ch, "11    Rx 4 word mode (Rx4WD)", reg, HCON_Rx4WD, 
   "1(4-word mode selected)", "0(1-word mode selected)"
 );

// [13:12] Rx widget alignment (RxWA) //////////////////////////////////////////
 {
   ABIT wa_bt[] = {
     {"00(No Invalid bytes)", HCON_RxWA0},
     {"01(1 invalid byte)"  , HCON_RxWA1},
     {"10(2 invalid bytes)" , HCON_RxWA2},
     {"11(3 invalid bytes)" , HCON_RxWA3},
     { 0, 0} 
   };
  PrintABIT(ch, "13:12 Rx widget alignment (RxWA)", reg, HCON_WIDGETMASK, wa_bt);
 }

// [14] DMA Tx stop or skip (DTxSTSK) //////////////////////////////////////////
 PrintSBIT(ch, "14    DMA Tx stop or skip (DTxSTSK)", reg, HCON_DTxSTSK,
   "1(DMA Tx stops)", "0(DMA Tx skips)"
 );

// [15] DMA Rx stop or skip (DRxSTSK) //////////////////////////////////////////
 PrintSBIT(ch, "15    DMA Rx stop or skip (DRxSTSK)", reg, HCON_DRxSTSK,
   "1(DMA Rx stops)", "0(DMA Rx skips)"
 );

// [16] DMA Rx memory address decrement (DRxMADEC) /////////////////////////////
 PrintSBIT(ch, "16    DMA Rx memory address decrement",reg,HCON_DRxMADEC,
   "1(Address is decremented)", "0(Address is incremented)"
 );

// [17] Tx flag idle (TxFLAG) //////////////////////////////////////////////////
 PrintSBIT(ch, "17    Tx flag idle (TxFLAG)", reg, HCON_TxFLAG, 
   "1(time fill mode)", "0(mark idle mode)"
 );

// [18] Tx single flag (TxSFLAG) ///////////////////////////////////////////////
 PrintSBIT(ch, "18    Tx single flag (TxSFLAG)", reg, HCON_TxSFLAG, 
   "1(Single flag mode)", "0(Double flag mode)"
 );

// [19] Tx loop-back mode (TxLOOP) /////////////////////////////////////////////
 PrintSBIT(ch, "19    Tx loop-back mode (TxLOOP)", reg, HCON_TxLOOP, 
   "1(loop-back)", "0(Normal operation)"
 );

// [20] Rx echo mode (RxECHO) //////////////////////////////////////////////////
 PrintSBIT(ch, "20    Rx echo mode (RxECHO)", reg, HCON_RxECHO, 
   "1(enabled)", "0(disabled)"
 );

 Print("\nPress any key to continue"); 
 get_byte(); 

// [21] Tx abort extension (TxABTEXT) //////////////////////////////////////////

 PrintSBIT(ch, "21    Tx abort extension (TxABTEXT)",  reg, HCON_TxABTEXT,
   "1(At least 16 consecutive 1s are Tx)",
   "0(At least consecutive eight 1s are Tx"
 );

// [22] Tx abort (TxABT) ///////////////////////////////////////////////////////
 PrintSBIT(ch, "22    Tx abort (TxABT)", reg, HCON_TxABT,
   "1(Tx abort)", "0(Normal)"
 );

// [23] Tx preamble (TxPRMB) ///////////////////////////////////////////////////
 PrintSBIT(ch, "23    Tx preamble (TxPRMB)", reg, HCON_TxPRMB,
   "1(Tx the content of HPRMB)", "0(mark idle or time fill pattern"
 );

// [24] Tx data terminal ready (TxDTR) /////////////////////////////////////////
 PrintSBIT(ch, "24    Tx data terminal ready (TxDTR)", reg, HCON_TxDTR,
   "1(nDTR goes low level)", "0(nDTR goes high level)"
 );

// [25] Rx frame discontinue (RxDISCON) ////////////////////////////////////////
 PrintSBIT(ch, "25    Rx frame discontinue (RxDISCON)", reg, HCON_RxDISCON, 
   "1(Ignore currently Rx frame)", "0(Normal)"
 ); 

// [26] Tx no CRC (TxNOCRC) ////////////////////////////////////////////////////
 PrintSBIT(ch, "26    Tx no CRC (TxNOCRC)", reg, HCON_TxNOCRC,
   "1(CRC is not appended by hardware)", "0(Disable)"
 );

// [27] Rx no CRC (RxNOCRC) ////////////////////////////////////////////////////
 PrintSBIT(ch, "27    Rx no CRC (RxNOCRC)", reg, HCON_RxNoCRC, 
   "1(Receiver does not check CRC)", "0(Disable)"
 );

// [28] Auto enable (AutoEN) ///////////////////////////////////////////////////
 PrintSBIT(ch, "28    Auto enable (AutoEN)", reg, HCON_AutoEN, 
   "1(enabled)", "0(disabled)"
 );

#ifdef _S3C4530_
 if(is_s3c4530(syscfg_pd_id))
  { 
   PrintUBIT(ch, "29    Transparent Rx stop",      reg, HCON_TRxSTOP);
   PrintUBIT(ch, "30    Transmit reverse (TxREV)", reg, HCON_TxREV);
   PrintUBIT(ch, "31    Receive reverse (RxREV)",  reg, HCON_RxREV);
  }
#endif							/* _S3C4530_ */
}


////////////////////////////////////////////////////////////////////////////////
// HDLC Status Register (HSTAT) ////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

static void PrintHSTAT(int ch)
{
 int cn, cnt;				// chan number, channels count
 U32 reg[2];				// HSTATA or HSTATB
 reg[0] = HSTATA;			// HSTATA 
 reg[1] = HSTATB;			// HSTATB 

 if(ch >= 0) 
  { 
   cnt = ch + 1;
   Print("HSTAT%c HDLC%c Status register 0x%08x", 'A'+ch, 'A'+ch, reg[ch]);
  }
 else 
  {
   cnt = -ch;
   Print("HSTAT Status regs A=0x%08x B=0x%08x", reg[0], reg[1]);
  } 

 {
// [3:0]   Rx remaining bytes (RxRB) ///////////////////////////////////////////
  PrintMBIT("3:0 Rx remaining bytes (RxRB)");
  for(cn = 0; cn < cnt; ++cn)  
   {
    if(cn == ch || ch < 0) 
     {
      Print("%-15d", (reg[cn] & HSTAT_RxRB));
     } 
   }
 }

// [4] Tx frame complete (TxFC) ////////////////////////////////////////////////
 PrintUBIT(ch, "4   Tx frame complete (TxFC)", reg, HSTAT_TxFC);
// [5] Tx FIFO available ///////////////////////////////////////////////////////
 PrintUBIT(ch, "5   Tx FIFO available  (TxFA)",reg, HSTAT_TxFA);
// [6] Tx clear-to-send (TxCTS) ////////////////////////////////////////////////
 PrintSBIT(ch, "6   Tx clear-to-send (TxCTS)", reg, HSTAT_TxCTS, 
  "1(nCTS input pin is Low)", "0(nCTS input pin is High)"
 );

// [7] Tx stored clear-to-send (TxSCTS) ////////////////////////////////////////
 PrintSBIT(ch, "7   Tx stored clear-to-send(TxSCTS)", reg, HSTAT_TxSCTS,
   "1(transition occurred at the nCTS input)",
   "0(Normal operation)"
 );

// [8] Tx underrun (TxU) ///////////////////////////////////////////////////////
 PrintUBIT(ch, "8   Tx underrun (TxU)", reg, HSTAT_TxU);
// [9] Rx FIFO available (RxFA) ////////////////////////////////////////////////
 PrintSBIT(ch, "9   Rx FIFO available (RxFA)", reg, HSTAT_RxFA, 
   "1(Data is available in the RxFIFO)", "0(Normal operation)"
 );

#ifdef _S3C4530_ 
//  [10] Tx Frame Good (TxFG) //////////////////////////////////////////////////
 if(is_s3c4530(syscfg_pd_id))
  {
   PrintUBIT(ch, "10  Tx Frame Good (TxFG)", reg, HSTAT_TxFG);
  }
#endif 								/* _S3C4530_ */

// [11] Rx flag detected (RxFD) ////////////////////////////////////////////////
 PrintUBIT(ch, "11  Rx flag detected (RxFD)", reg, HSTAT_RxFD);

// [12] Rx data carrier detected (RxDCD) ///////////////////////////////////////
 PrintSBIT(ch, "12  Rx data carrier detected(RxDCD)", reg, HSTAT_RxDCD, 
   "1(nDCD input pin is Low)", "0(nDCD input pin is High)"
 );

// [13] Rx stored data carrier detected (RxSDCD) ///////////////////////////////
 PrintSBIT(ch, "13  Rx stored data carrier detected", reg, HSTAT_RxSDCD, 
   "1(transition of the nDCD input)", "0(Normal operation)"
 );

// [14] Rx frame valid (RxFV) //////////////////////////////////////////////////
 PrintSBIT(ch, "14  Rx frame valid (RxFV)", reg, HSTAT_RxFV, 
   "1(last data byte of a frame recived)",
   "0(Normal operation)"
 );

// [15] Rx idle (RxIDLE) ///////////////////////////////////////////////////////
 PrintSBIT(ch, "15  Rx idle (RxIDLE)", reg, HSTAT_RxIDLE, 
   "1(Rx 15 consecutive 1s)", "0(Normal operation)"
 );

// [16] Rx abort (RxABT) ///////////////////////////////////////////////////////
 PrintSBIT(ch, "16  Rx abort (RxABT)", reg, HSTAT_RxABT, 
   "1(Rx consecutive 1s)", "0(Normal operation)"
 );

// [17] Rx CRC error (RxCRCE) //////////////////////////////////////////////////
 PrintSBIT(ch, "17  Rx CRC error (RxCRCE)", reg, HSTAT_RxCRCE,
    "1(CRC error)", "0(Normal operation)"
 );

// [18] Rx non-octet align (RxNO) //////////////////////////////////////////////
 PrintSBIT(ch, "18  Rx non-octet align (RxNO)", reg, HSTAT_RxNO,
  "1(Received frame is not octet)", "0(Received frame is octet)"
 );

// [19] Rx overrun (RxOV) //////////////////////////////////////////////////////
 PrintUBIT(ch, "19  Rx overrun (RxOV)", reg, HSTAT_RxOV);

// [20] DMA Rx memory overflow (RxMOV) /////////////////////////////////////////
 PrintSBIT(ch, "20  DMA Rx memory overflow (RxMOV)", reg, HSTAT_RxMOV,
    "1(overflow Rx buf desc,next ptr == null)",
    "0(Normal operation)"
 );

 Print("\nPress any key to continue"); 
 get_byte(); 

// [22] DMA Tx abort (DTxABT) //////////////////////////////////////////////////
 PrintSBIT(ch, "22  DMA Tx abort (DTxABT)", reg, HSTAT_DTxABT, 
    "1", "0(Normal operation)"
 );

// [23] Rx internal error (RxIERR) /////////////////////////////////////////////
 PrintUBIT(ch, "23  Rx internal error (RxIERR)", reg, HSTAT_RxIERR);

// [24] DMA Rx frame done every received frame (DRxFD) /////////////////////////
 PrintUBIT(ch, "24  DMA Rx frame done every frame", reg, HSTAT_DRxFD);

// [25] DMA Rx null list (DRxNL) ///////////////////////////////////////////////
 PrintUBIT(ch, "25  DMA Rx null list (DRxNL)", reg, HSTAT_DRxNL);

// [26] DMA Rx not owner (DRxNO) ///////////////////////////////////////////////
 PrintSBIT(ch, "26  DMA Rx not owner (DRxNO)", reg, HSTAT_DRxNO,
    "1(CPU has the ownership)", "0(DMA has the ownership)"
 );

// [27] DMA Tx frame done (DTxFD) //////////////////////////////////////////////
 PrintUBIT(ch, "27  DMA Tx frame done (DTxFD)", reg, HSTAT_DTxFD);

// [28] DMA Tx null list (DTxNL) ///////////////////////////////////////////////
 PrintUBIT(ch, "28  DMA Tx null list (DTxNL)", reg, HSTAT_DTxNL);

// [29] DMA Tx not owner (DTxNO) ///////////////////////////////////////////////
 PrintSBIT(ch, "29  DMA Tx not owner (DTxNO)", reg, HSTAT_DTxNO, 
    "1(CPU has the ownership)", "0(DMA has the ownership)"
 );

// [30] DPLL one clock missing (DPLLOM) ////////////////////////////////////////
 PrintUBIT(ch, "30  DPLL one clock missing (DPLLOM)", reg, HSTAT_DPLLOM);
// [31] DPLL two clock missing (DPLLTM) ////////////////////////////////////////
 PrintUBIT(ch, "31  DPLL two clock missing (DPLLTM)", reg, HSTAT_DPLLTM);
}

////////////////////////////////////////////////////////////////////////////////
// HDLC Interrupt Enable Register (HINTEN) /////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

static void PrintHINTB(char * str, U32 val, U32 mask)
{
 Print("\n%s%s", 
   str ,
   ((val & mask) != 0) ?
    "1(enabled)" :
    "0(disabled)"
  );
}

static void PrintHINTEN(int cn)
{
 U32 hi;				// 
 U8 ch;					// channel letter 
 if(cn == 0)
  {
   ch = 'A';				// channel A 
   hi = HINTENA;			// 
  } 
 else
 if(cn == 1)
  {
   ch = 'B';				// channel B 
   hi = HINTENB;
  }

 Print("\nHINTEN%c HDLC%c Int enable register.", ch, ch);

// [4] Tx frame complete interrupt enable //////////////////////////////////////
 PrintHINTB("4  Tx frame complete Int (TxFCIE).........",hi,HINTEN_TxFCIE);
// [5] Tx FIFO available to write interrupt enable /////////////////////////////
 PrintHINTB("5  Tx FIFO available to write Int(TxFAIE).",hi,HINTEN_TxFAIE);
// [7] CTS transition has occurred interrupt enable ////////////////////////////
 PrintHINTB("7  CTS transition Int (TxSCTSIE)..........",hi,HINTEN_TxSCTSIE);
// [8] Tx underrun has occurred interrupt enable ///////////////////////////////
 PrintHINTB("8  Tx underrun Int (TxUIE)................",hi,HINTEN_TxUIE);
// [9] Rx FIFO available to read interrupt enable //////////////////////////////
 PrintHINTB("9  Rx FIFO available to read Int (RxFAIE).",hi,HINTEN_RxFAIE);

#ifdef _S3C4530_ 
 PrintHINTB("10 Tx Fframe good Int (TxFGIE)............",hi,HINTEN_TxFGIE);
#endif 								/* _S3C4530_ */ 

// [11]Rx flag detected interrupt enable ///////////////////////////////////////
 PrintHINTB("11 Rx flag detected Int (RxFDIE)..........",hi,HINTEN_RxFDIE);
// [13]DCD transition interrupt enable /////////////////////////////////////////
 PrintHINTB("13 DCD transition Int (RxSDCDIE)..........",hi,HINTEN_RxSDCDIE);
// [14]Rx frame valid interrupt enable /////////////////////////////////////////
 PrintHINTB("14 Rx frame valid Int (RxFVIE)............",hi,HINTEN_RxFVIE);
// [15]Idle detected interrupt enable //////////////////////////////////////////
 PrintHINTB("15 Idle detected Int (RxIDLEIE)...........",hi,HINTEN_RxIDLEIE);
// [16]Abort detected interrupt enable /////////////////////////////////////////
 PrintHINTB("16 Abort detected Int (RxABTIE)...........",hi,HINTEN_RxABTIE);
// [17]CRC error frame interrupt enable ////////////////////////////////////////
 PrintHINTB("17 CRC error frame Int (RxCRCEIE).........",hi,HINTEN_RxCRCEIE); 
// [18]Non-octet aligned frame interrupt enable ////////////////////////////////
 PrintHINTB("18 Non-octet aligned frame Int (RxNOIE)...",hi,HINTEN_RxNOIE);
// [19]Rx overrun interrupt enable /////////////////////////////////////////////
 PrintHINTB("19 Rx overrun Int (RxOVIE)................",hi,HINTEN_RxOVIE);
// [20]Rx memory overflow interrupt enable /////////////////////////////////////
 PrintHINTB("20 Rx memory overflow Int (RxMOVIE).......",hi,HINTEN_RxMOVIE);

 Print("\nPress any key to continue"); 
 get_byte(); 


// [22]DMA Tx abort interrupt enable ///////////////////////////////////////////
 PrintHINTB("22 DMA Tx abort Int (DTxABTIE)............",hi,HINTEN_DTxABTIE);
// [23]Rx internal error interrupt enable //////////////////////////////////////
 PrintHINTB("23 Rx internal error Int (RxIERRIE).......",hi,HINTEN_RxIERRIE);
// [24]DMA Rx frame done interrupt enable //////////////////////////////////////
 PrintHINTB("24 DMA Rx frame done Int (DRxFDIE)........",hi,HINTEN_DRxFDIE);
// [25]DMA Rx null list interrupt enable ///////////////////////////////////////
 PrintHINTB("25 DMA Rx null list Int (DRxNLIE).........",hi,HINTEN_DRxNLIE);
// [26]DMA Rx not owner interrupt enable ///////////////////////////////////////
 PrintHINTB("26 DMA Rx not owner Int (DRxNOIE).........",hi,HINTEN_DRxNOIE);
// [27]DMA Tx frame done every transmitted frame interrupt enable //////////////
 PrintHINTB("27 DMA Tx frame done every frame(DTxFDIE).",hi,HINTEN_DTxFDIE);
// [28]DMA Tx null list interrupt enable ///////////////////////////////////////
 PrintHINTB("28 DMA Tx null list Int (DTxNLIE).........",hi,HINTEN_DTxNLIE);
// [29]DMA Tx not owner interrupt enable ///////////////////////////////////////
 PrintHINTB("29 DMA Tx not owner Int (DTxNOIE).........",hi,HINTEN_DTxNOIE);
// [30]DPLL one clock missing interrupt enable /////////////////////////////////
 PrintHINTB("30 DPLL one clock missing Int( DPLLOMIE)..",hi,HINTEN_DPLLOMIE);
// [31]DPLL two clocks missing interrupt enable ////////////////////////////////
 PrintHINTB("31 DPLL two clocks missing Int (DPLLTMIE).",hi,HINTEN_DPLLTMIE);
}


static void PrintHBRGTC(int cn)
{
 U32 hmode;
 U32 hbrgtc;				// 
 U32 cnt2;
 U32 cnt1;
 U32 cnt0;
 U8 ch;					// channel letter 
 if(cn == 0)
  {
   ch = 'A';				// channel A 
   hmode  = HMODEA;			// HDLC A Mode register   
   hbrgtc = HBRGTCA;			// HDLC A BRG register 
  } 
 else
 if(cn == 1)
  {
   ch = 'B';				// channel B 
   hmode  = HMODEB;			// HDLC N Mode register 
   hbrgtc = HBRGTCB;			// HDLC B BRG register 
  }

// 
 Print("\nHBRGTC%c HDLC%c.", ch, ch);

// 
 cnt2 = (hbrgtc & 0x3); 
 Print("\n1:0  Time constant value for CNT2....");
 switch( cnt2 )
  {
   case 0 : Print("00(divide by 1)");  cnt2 =  1; break;	// 
   case 1 : Print("01(divide by 16)"); cnt2 = 16; break;	// 
   case 2 : Print("10(divide by 32)"); cnt2 = 32; break;	// 
   default:
    break; 
  }

 Print("\n3:2  Time constant value for CNT1....");
 cnt1 = (hbrgtc >> 2) & 0x3;
 switch(cnt1)
  {
   case 0 : Print("00(divide by 1)");  cnt1 =  1; break;
   case 1 : Print("01(divide by 16)"); cnt1 = 16; break;
   default:
    break; 
  }

 cnt0 = ((hbrgtc >> 4) & 0xFFF);
 Print("\n15:4 Time constant value for CNT0....%d", cnt0 );

 if( (hmode & HMODE_BRGCLK_MCLK2) != 0 )
  {
   Print("\nBRGOUT1=MCLK2/(CNT0+1)/(16**CNT1)....%d", MCLK2/(cnt0+1)/cnt1);
   Print("\nBRGOUT2=BRGOUT1/CNT2.................%d", MCLK2/(cnt0+1)/cnt1/cnt2);
  }
}

static void PrintHSAR(char * nam, U32 val)
{
 Print("\n%s = 0x%08x", nam, val);
 Print("\n31:24 First address byte.......0x%02x", (val >> 24) & 0xFF);
 Print("\n23:16 Second address byte......0x%02x", (val >> 16) & 0xFF);
 Print("\n15:8  Third address byte.......0x%02x", (val >>  8) & 0xFF);
 Print("\n7:0   Fourth address byte......0x%02x", (val)       & 0xFF);
}


#ifdef _S3C4530_
// Transparent Contgrol Regs
static void PrintTCON(int ch)
{
 int cn, cnt;				// channel number, count
 U32 reg[2];				// reg content 
 reg[0] = TCONA;			// Channel A transparent control register 
 reg[1] = TCONB;			// Chan B Transparent Control Reg
  
 Print("\nTransparent Contgrol Regs TCON");
 if(ch >= 0) 
  {
   cnt = ch + 1;  
   Print(" 0x%08x", reg[ch] );
  }  
 else 
  {
   cnt = -ch;
   for(cn = 0; cn < cnt; ++cn) 
    {
     Print(" 0x%08x", reg[cn] );
    }
  }

 { 
  ABIT ds_bt[] = {
    {"00(D4)", TCON_DS_D4},
    {"01(D3)", TCON_DS_D3},
    {"10(D2)", TCON_DS_D2},
    {"11(D1)", TCON_DS_D1},
    {0, 0}
  };
  
  PrintABIT(ch, "1:0 Data Sampling (DS)", reg, TCON_DS, ds_bt); 
 }

  PrintSBIT(ch, "4   RTS Pin Low", reg, TCON_RTS, "1(Low)", "0(High)");
}
#endif								/* _S3C4530_ */

////////////////////////////////////////////////////////////////////////////////
// view HDLC registers /////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////


void HdlcRegView(void)
{
 while(1)
  {
   U8 it;
   Print("\n\nHDLC Registers");
   Print("\n                                              A       B");
   Print("\n[M] HMODE     (Mode register)              %08x %08x",HMODEA    , HMODEB);
   Print("\n[C] HCON      (Control register)           %08x %08x",HCONA     , HCONB);
   Print("\n[S] HSTAT     (Status Register)            %08x %08x",HSTATA    , HSTATB); 
   Print("\n[I] HINTEN    (Interrupt Enable)           %08x %08x",HINTENA   , HINTENB); 
   Print("\n[B] HBRGTC    (BRG Time Constant Register) %08x %08x",HBRGTCA   , HBRGTCB);
   Print("\n[0] HSAR0     (HDLC station address 0)     %08x %08x",HSAR0A    , HSAR0B);
   Print("\n[1] HSAR1     (HDLC station address 1)     %08x %08x",HSAR1A    , HSAR1B);
   Print("\n[2] HSAR2     (HDLC station address 2)     %08x %08x",HSAR2A    , HSAR2B);
   Print("\n[3] HSAR3     (HDLC station address 3)     %08x %08x",HSAR3A    , HSAR3B);
   Print("\n[A] HMASK     (address mask register)      %08x %08x",HMASKA    , HMASKB);
   Print("\n    HDMATXPTR (Tx Buffer Descr. Pointer)   %08x %08x",HDMATXPTRA, HDMATXPTRB); 
   Print("\n    HDMARXPTR (Rx Buffer Descr. Pointer)   %08x %08x",HDMARXPTRA, HDMARXPTRB); 
   Print("\n    HMFLR     (Maximum Frame Length)       %08x %08x",HMFLRA    , HMFLRB);
   Print("\n    HRBSR     (DMA Receive Buffer Size)    %08x %08x",HRBSRA    , HRBSRB);
   Print("\n    HPRMB     (DMA Preamble Register)      %08x %08x",HPRMBA    , HPRMBB);

#ifdef _S3C4530_
   if(is_s3c4530(syscfg_pd_id)) 
   { 
     Print("\n    HSYNC     (HDLC Sync Regster)          %08x %08x",HSYNCA    , HSYNCB);
     Print("\n[T] TCON      (Transparent Control reg)    %08x %08x",TCONA     , TCONB);
     Print("\n    TSACFG    (TSA Configuration reg)      %08x"     ,TSACFG);
     Print("\n    TSASLOT   (TSA A&B Time Slot reg)      %08x %08x",TSAACFG   , TSABCFG);
   }
#endif								/* _S3C4530_ */

   Print("\n[Q] Quit");
   Print("\nSelect Test Item : "); 
   it = get_upper();   

   switch(it)
    {
     case 'M' : 
// HDLC Mode registers /////////////////////////////////////////////////////////
      PrintHMODE(0);			// Print HMODEA register 
      PrintHMODE(1);			// Print HMODEB register 
      break;

     case 'C' : 
// HDLC Control registers //////////////////////////////////////////////////////
      PrintHCON(0);			// HCONA   
      Print("\nPress any key to continue"); 
      get_byte(); 
      PrintHCON(1);			// HCONB
      break;

     case 'S' : 
// HDLC Status registers /////////////////////////////////////////////////////// 
      PrintHSTAT(0);			// HSTATA   
      Print("\nPress any key to continue"); 
      get_byte(); 
      PrintHSTAT(1);			// HSTATB
      break;

     case 'I' : 
// 
      PrintHINTEN(0); 			// HINTENA
      Print("\nPress any key to continue"); 
      get_byte(); 
      PrintHINTEN(1); 			// HINTENB
      break;

     case 'B' : 
      PrintHBRGTC(0); 			// HBRGTCA 
      PrintHBRGTC(1); 			// HBRGTCB
      break;

     case '0' : 
// HDLC station address 0 //////////////////////////////////////////////////////
      PrintHSAR("HSAR0A", HSAR0A);	
      PrintHSAR("HSAR0B", HSAR0B);
      break;

     case '1' : 
// HDLC station address 1 //////////////////////////////////////////////////////
      PrintHSAR("HSAR1A", HSAR1A);	
      PrintHSAR("HSAR1B", HSAR1B);
      break;

     case '2' :
// HDLC station address 2 //////////////////////////////////////////////////////
      PrintHSAR("HSAR2A", HSAR2A);	
      PrintHSAR("HSAR2B", HSAR2B);
      break;

     case '3' :
// HDLC station address 3 //////////////////////////////////////////////////////
      PrintHSAR("HSAR3A", HSAR3A);	
      PrintHSAR("HSAR3B", HSAR3B);
      break;

     case 'A' :
// HDLC address mask register //////////////////////////////////////////////////
      PrintHSAR("HMASKA", HMASKA);	
      PrintHSAR("HMASKB", HMASKB);
      break; 
#ifdef _S3C4530_
     case 'T' : 
// Transparent Contgrol Regs
       if(is_s3c4530(syscfg_pd_id)) { 
         PrintTCON(-2); 
       }
       break;	
#endif								/* _S3C4530_ */ 
     case 'Q' : return;
     default  : 
      break;
    }

   Print("\nPress any key to continue"); 
   it = get_byte();
   if(it == 'Q')
    {
     return;
    } 
  }
} 

////////////////////////////////////////////////////////////////////////////////
// HDLC Reset Test /////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

void HdlcResetTest (void)
{
 U8 cn; 
 U8 clet; 
 U8 csel;
 U8 it;
 U8 warn;

 cn = 0;
 clet = 'A';
 csel = 'B';
 do 
  {
   Print("\nHDLC Reset");
   Print("\n[%c] HDLC %c Select"     , csel, csel);// Select HDLC Channel 
   Print("\n[T] HDLC %c Tx Reset"    , clet );	 
   Print("\n[R] HDLC %c Rx Reset"    , clet );
   Print("\n[D] HDLC %c DMA Tx Reset", clet );
   Print("\n[E] HDLC %c DMA Rx Reset", clet );
   Print("\n[I] HDLC Initialize");
   Print("\n[Q] Quit HDLC Reset Test");
   Print("\nSelect Test Item : "); 
   it = get_upper();
	
   warn = 1;
   switch(it)
    {
     case 'A' :
// select HDLC channel A /////////////////////////////////////////////////////// 
      Print("\nHDLC Channel A selected."); 
      cn   =  0;
      clet = 'A';
      csel = 'B';
      warn = 0;				// no warning 
      break;

     case 'B' :
// Select HDLC Chennel B /////////////////////////////////////////////////////// 
      Print("\nHDLC Channel B selected."); 
      cn =  1;
      clet = 'B';
      csel = 'A';
      break;
     
     case 'T' : 
// Reset Tx for selected HDLC channel ////////////////////////////////////////// 
      Print("\nHDLC Channel %c Tx reseted.",clet); 
      HCON(cn) |= HCON_TxRS;
      break;

     case 'R' : 
// Reset Rx for selected HDLC channel ////////////////////////////////////////// 
      Print("\nHDLC Channel %c Rx reseted.",clet); 
      HCON(cn) |= HCON_RxRS;
      break;

     case 'D' : 
// Reset DMA Tx for selected channel /////////////////////////////////////////// 
      Print("\nHDLC Channel %c DMA Tx reseted.",clet); 
      HCON(cn) |= HCON_DTxRS;
      break;	

     case 'E' : 
// Reset DMA Rx for selected channel /////////////////////////////////////////// 
      Print("\nHDLC Channel %c DMA Rx reseted.",clet); 
      HCON(cn) |= HCON_DRxRS;
      break;

     case 'I' : 
      HDLC_Init();
      warn = 0;				// no warning message
      break;

     case 'Q' : 
      return;

     default  : 
      Print("\nInvalid Test Item Selected"); 
      warn = 0;				// no warning message
      break;
    }

// 
   if(warn == 1)
    {
     Print("\nAfter Reset Command HDLC registers are changed."); 
     Print("\nSelect HDLC Initialize to set correct values of HDLC registers"); 
    }

   Print("\nPress any key to Continue HDLC Test") ; 
   it = get_byte();
  } 
 while(it != 'Q' && it != 'q');
}

////////////////////////////////////////////////////////////////////////////////
// HDLC Loopback Function //////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

//#define HDLC_BYTE_FILLING		
#define HDLC_LOOP_DATA_MAX_LEN	 1500  

int HdlcLoopback(int cn, int lcount, int quiet)
{
// Tx & Rx data buffer
 static U8 txdata[2][HDLC_LOOP_DATA_MAX_LEN+4];
 static U8 rxdata   [HDLC_LOOP_DATA_MAX_LEN+4];
 U8 txsended[2] = {1, 1};

 int tm1en;				// timer 1 enable flag 
 TIME tm;				// TIME struct 
 U32 startms;				// start time, ms
 U32 curms;				// end time, ms 
 U32 status;				// Rx status
 int rlen;				// received length
 int recv;				// HDLCRecv return code 
 int more;				// more flag 

 U32 rxframes[2];			// receive frame count
 U32 missed[2];				// missed packets count
 U32 errminlen[2];			// minimum length error count 
 U32 erraddr[2];			// address error count 
 U32 errlen[2];				// length error count  
 U32 errseq[2];				// sequence error count 
 U32 errdata[2];			// filling data error count
 U32 errstatus[2];

 U32 hash;
 U32 lcnt[2];
 U32 tcnt[2];				// tx packet number  
 U32 rcnt[2];				// rx packet number 
 U32 tbytes[2];				// tx bytes / sec
 int pos;				// buffer posuition
 int rvl;				// return code 
 int i;					// channel number 
 U8 it = 0;				// console input 

// Tx frames length 
 int datalen = 120;

 if(quiet == 0)
  {
   Print("\nInput packet count (10000 default)");
   lcount = get_number(8, 16);
   if(lcount == 0)
    {
     lcount = 10000;
    }
  
   Print("\nInput packet length (120 default) ");
   datalen = get_number(8, 16);
   if(datalen == 0) { datalen = 120; }
  }

 if(datalen < 8)  { datalen = 8; }  
 if(datalen > HDLC_LOOP_DATA_MAX_LEN) 
   {datalen = HDLC_LOOP_DATA_MAX_LEN;}

//
 rvl = 1;

// 
 Print("\nHDLC Channel");
 switch(cn)
  {
   case  0 : Print(" A"); break;  	// Channel A test 
   case  1 : Print(" B"); break;	// Channel B test 
   case -1 : Print("s A & B"); break;	// All channels test 
   default :
    return(0);
  }

 if(gHdlcBaudRate < 500000)
  {
   hash = 10; 
  }
 else
  {
   hash = 100; 
  }

#ifdef HDLC_BYTE_FILLING		
 Print(" Loopback Test ... ");
#else
 Print(" Loopback Test (W)... ");
#endif
 Print("\n#=%d packets sended.", hash); 
 Print("\nTo break test press [Q].");

// 
 tm1en = TimerEnabled(1);		// read timer 1 enable flag 	
 if(tm1en == 0)
  {
// timer 1 disabled => enable timer 1 //////////////////////////////////////////
   TimerReset(1);				// reset timer 1 
   TimerInit(1,(ONE_SECOND/TICKS_PER_SECOND));	// init timer 1 
   TimerStart(1);				// start timer 1 
  }

 GetSysTime(1,& tm); 			// read start time, convert to ms
 startms = tm.tm_ms + (tm.tm_sec + tm.tm_min * 60) * 1000;

// clear Rx & Tx buffers, enable HDLC channels /////////////////////////////////

 Disable_Int(nGLOBAL_INT);		// disable global int 
 more = 0;				// clear more flag 
 for(i = 0; i < 2; ++i)
  {
   txsended[i]  = 1; 
   lcnt[i]      = lcount;
   tcnt[i]      = 0;			// clear transmit packet number
   rcnt[i]      = 0; 			// clear receive packet number
// 
   rxframes[i]  = 0;			// rx frames 
   missed[i]    = 0;			// total rx errors
   errminlen[i] = 0;			// minimum length error count 
   erraddr[i]   = 0;			// address error count 
   errlen[i]    = 0;			// length error count  
   errseq[i]    = 0;			// sequence error count 
   errdata[i]   = 0;			// filling data error count
   errstatus[i] = 0;

   if(cn == -1 || i == cn)		// 
    {
     more |= (1 << i);			// channel on => set more bit 
// enable Rx & Tx, DMA Rx & Tx 
     HCON(i)    |=  HCON_DTxEN | HCON_DRxEN | HCON_TxEN | HCON_RxEN;
    }
  }  
 Enable_Int(nGLOBAL_INT);		// enable global int 

 while(more)
  {
   for(i = 0; i < 2; ++i)
    {
// tx loop /////////////////////////////////////////////////////////////////////
     if(i != cn && cn != -1)
      {
       continue;
      }

     if(tcnt[i] >= lcnt[i])
      {
       if(HDLCTxBD_count(i) == 0)
        {				// all bd transmitted 
         more &= ~(1 << i);		// clear more bit
         if(more ==0)
          {
           U32 dms;
           GetSysTime(1,& tm); 
           dms = curms = tm.tm_ms + tm.tm_sec * 1000;
           while(curms < dms + 500)
            {
             GetSysTime(1,& tm); 
             curms = tm.tm_ms + tm.tm_sec * 1000;
            }
          }
        } 
      }


     while(tcnt[i] < lcnt[i])
      {
// channel i tx loop ///////////////////////////////////////////////////////////
       U8 * data = txdata[i];       
       if(txsended[i] == 1)
        {
// txdata[i] sended => new filling /////////////////////////////////////////////
#ifdef HDLC_BYTE_FILLING
// byte filling 
         data[0] =  0xFF;		// station address
         data[1] =  0x0;		// 
         data[2] =  datalen & 0xFF; 	// data length (low byte)
         data[3] = (datalen >> 8)&0xFF; // data length (hight byte)
         data[4] =  tcnt[i] & 0xFF;	// transmit number
         data[5] = (tcnt[i] >> 8)&0xFF;	// 
         data[6] = (tcnt[i] >>16)&0xFF;	// 
         data[7] = (tcnt[i] >>24)&0xFF;	// 

         for(pos = 8; pos < datalen; ++pos)
          {				// fill packet 
           data[pos] = tcnt[i] + pos;
          }    
#else
// word filling 
         U32 * ptr;			// word pointer 
         U32 val; 			// filling value
         int wlen;			// packet length, words  

         ptr = (U32 *) data;
        *ptr++ = (datalen << 16)| 0x00FF; // address + datalen 
        *ptr++ = tcnt[i];			// tx sequence number 
         wlen = (datalen + 3) >> 2;	// words (include tailing words) 
         val  = tcnt[i];			
         for(pos = 2; pos < wlen; ++pos)
          {				
           *ptr++ = val++;  		// words filling 
          }
#endif
// end txdata[i] filling
        }
       
       if(HDLCSend(i, data, datalen) == 0)
        {
         txsended[i] = 0;		// data not sended =>  
         break;				// txdata[i] ready to send
        }
       else
        {
         tcnt[i]++;			// next transmit number
         txsended[i] = 1;		// tx data sended 
         if(tcnt[i] != 0 && (tcnt[i] % hash) == 0)
          {
           Print("#");
          }
        }

       if(kbd_hit())
        {
         it = get_byte();	
         if(it == 'Q' || it == 'q')
          {
           lcnt[0] = tcnt[0];
           lcnt[1] = tcnt[1];
          }
        }
      }					// ch i end loop
    }					// 

   if(kbd_hit())
    {
     it = get_byte();	
     if(it == 'Q' || it == 'q')
      {
       lcnt[0] = tcnt[0];
       lcnt[1] = tcnt[1];
      }
    }

   for(i = 0; i < 2; ++i)
    {
//     int recvdelta = 128 + 64;
// receive looop /////////////////////////////////////////////////////////////// 
     if(i != cn && cn != -1)
      {
       continue;
      } 
 
     while(1)
      {
#ifndef HDLC_BYTE_FILLING
       U32 * ptr;
       U32 val;
       int wlen;
#endif 							// HDLC_BYTE_FILLING

       int fillerr;
       int plen; 			//
       int num; 			//

// receive channel i loop ////////////////////////////////////////////////////// 
       rlen = sizeof(rxdata);
       recv = HDLCRecv(i, &status, rxdata, & rlen);
       if(recv < 0)
        {				// reveive error  
         rvl = 0;			// clear return code 
         break;				// break from channel i recv loop 
        }
       else
       if(recv == 0)
        {				// no rx data 
         break;
        }       

// CRC Error (CE), Non-octet Aligned Frame (NO), Overrun (OV), Rx Abort (ABT)
// Frame Length Violation (FLV)
       if( 
          (status & 
           (HDLCRxBD_CRC | 
            HDLCRxBD_NO  | 
            HDLCRxBD_OV  | 
            HDLCRxBD_ABT |
            HDLCRxBD_FLV)
          ) != 0 
        )
        {
         rvl = 0;			
         missed[i]    += 1;		// channel error count 
         errstatus[i] += 1;		// status error count 
        }
         
       rxframes[i]++;			// total rx frames 
       if(rlen < 8)
        {
// minimal length error ////////////////////////////////////////////////////////
         rvl = 0;
         if(quiet == 0)
          {
           Print("\nRx Frame minimum length error. Length = %d\n", rlen);
          }
         missed[i]    += 1;		// channel error count 
         errminlen[i] += 1;		// channel i minimal length error count 
         continue;
        }

       if(rxdata[0] != 0xFF && rxdata[1] != 0x00)
        {
// address error ///////////////////////////////////////////////////////////////
         rvl = 0;
         if(quiet == 0)
          {
           Print("\nRx Frame address error.Address [0x%02x%02x],Length=[%d],Wait Num=[%d]", 
            rxdata[0], rxdata[1], rlen, rcnt[i] );
           Print("\nData");
           for(pos = 0; pos < 8; ++pos)
            {
             Print(" %02x", rxdata[pos]);
            }
          }
         missed[i]  += 1;		// channel i error count 	
         erraddr[i] += 1;		// channel i address error count 
         continue;
        } 

// read frame length from frame 
       plen = (U32) rxdata[2] | ((U32) rxdata[3] << 8);

       if(plen + 2 != rlen || rlen != datalen + 2) 
        {
// frame length error //////////////////////////////////////////////////////////
         if(quiet == 0)
          {
           Print("\nRx Frame length error. Frame length=%d, writed in packet=%d",
             rlen	,
             plen
            );
          }
         rvl = 0;			
         missed[i] += 1;		// channel i error count 
         errlen[i] += 1;		// channel i frame length error count  	
         continue;
        }

#ifdef HDLC_BYTE_FILLING
       num = 
         (U32) rxdata[4]        |		// lo    byte
        ((U32) rxdata[5] <<  8) |		// lo mi byte
        ((U32) rxdata[6] << 16) |		// hi
        ((U32) rxdata[7] << 24);	
#else
       num = *((U32 *) (rxdata + 4));
#endif  						// HDLC_BYTE_FILLING
 
       if(rcnt[i] != num)
        {
// sequence number error ///////////////////////////////////////////////////////

         if(quiet == 0)
          {
           Print("\nRx Sequence number error (wait =%d, rx = %d)\n",
             rcnt[i]	,
             num
            );
          } 

         rvl = 0;
         rcnt[i] = num + 1;		// reset wait rx num	
         missed[i] += 1;		// missed packets count
         errseq[i] += 1;		// channel i sequence error count 
         continue;
        }

// wait packet number equal received packet number /////////////////////////////
// => check packet filling 
       fillerr = 0;

#ifdef HDLC_BYTE_FILLING
       for(pos = 8; pos < rlen - 2; ++pos)	// check packet filling 
        {					
         if(rxdata[pos] != (U8)(rcnt[i] + pos))
          {
           fillerr = 1;
           break;
          } 
        }
#else 	
       wlen = (rlen - 2) >> 2;		// length full words		
       ptr = (U32 *)(rxdata + 8);	// filling data pointer
       val = rcnt[i];			
       for(pos = 2; pos < wlen; ++pos)
        {
         if(*ptr++ != val++)
          {
           fillerr = 1;
           break;
          }
        }

       if(fillerr == 0 && ((rlen - 2) & 0x3) != 0)
        {
         U32 mask;
         switch((rlen - 2) & 0x3)
          {
           case  1 : mask = 0x000000FF; break;
           case  2 : mask = 0x0000FFFF; break;
           case  3 : mask = 0x00FFFFFF; break;
           default : mask = 0x00000000; break;
          }

         if( (*ptr & mask) != (val & mask) )
          {
           fillerr = 1;
          }
        }
#endif 							// HDLC_BYTE_FILLING
         
       if(fillerr == 1)
        {
         if(quiet == 0)
          {
           Print("\nRx frame filling error. Position=%d", pos);
          }
         rvl         = 0;		// clear return code 
         missed[i]  += 1;		// channel i rx error count 
         errdata[i] += 1;		// channel i data filling error count   

         if(quiet == 0)
          {
           Print("\n");
           for(pos = 0; pos < rlen; ++pos)
            {
             Print("%02x ", rxdata[pos]);
             if(((pos + 1) % 16) == 0)
              {
               Print("\n");
              }
            }     
           Print("\n");
           { break; }
          }
        }    

// next wait frame ///////////////////////////////////////////////////////////// 
       rcnt[i]++; 			
       if(kbd_hit())
        {
         it = get_byte();
         if(it == 'Q' || it == 'q')
          {
           lcnt[0] = tcnt[0];
           lcnt[1] = tcnt[1];
          } 
        }
// end of recv chan i
      }	
// end of recv loop 
    }
  }

 Disable_Int(nGLOBAL_INT);		// disable global interrupt 	
 for(i = 0; i < 2; ++i) 
  {
   if(i == cn || cn == -1)
    {
     HCON(i) &=
     ~(
       HCON_DTxEN | HCON_TxEN |		// disable DMA Tx 
       HCON_DRxEN | HCON_RxEN		// disable DMA Rx 
      );
//
#ifdef HDLC_RESET
     Print("\nResetting HDLC %c",(i?'B':'A'));
     HDLCRx_reset(i);			// reset Rx
     HDLCTx_reset(i);			// reset Tx
#endif 								// HDLC_RESET
    }
  }

// 
 GetSysTime(1,& tm); 
 curms = tm.tm_ms + (tm.tm_sec + tm.tm_min * 60) * 1000;
 if(tm1en == 0)
  {
   TimerStop(1);
  }


 for(i = 0; i < 2; ++i)
  {
   U32 deltams;				// diff time ms
   deltams = curms - startms;   
   if(deltams == 0)
    {					// 
     tbytes[i] = 0; 
    }
   else
    { 
     tbytes[i] = tcnt[i] * datalen;	// transmit bytes 
     if(tbytes[i] > 0xFFFFFFFF / 1000)
      {					// U32 overflow 
       deltams /= 1000;			// diff time sec
       if(deltams == 0)
        {				
         tbytes[i] = 0; 
        }
       else
        {
         tbytes[i] /= deltams;
        }
      }
     else
      {
       tbytes[i] = tbytes[i] * 1000 / deltams;
      }
    }
  }

 Print("\nTx frames          %8d %8d", tcnt     [0],tcnt     [1]);
 Print( " Tx bytes/sec       %8d %8d", tbytes   [0],tbytes   [1]); 
 Print("\nRx frames          %8d %8d", rxframes [0],rxframes [1]);
 Print( " Rx frames with Err %8d %8d", missed   [0],missed   [1]);
 Print("\nRx status Err      %8d %8d", errstatus[0],errstatus[1]);
 Print( " Rx min length Err  %8d %8d", errminlen[0],errminlen[1]);
 Print("\nRx address error   %8d %8d", erraddr  [0],erraddr  [1]);
 Print( " Rx length error    %8d %8d", errlen   [0],errlen   [1]);
 Print("\nRx sequence num Err%8d %8d", errseq   [0],errseq   [1]);
 Print( " Rx filling error   %8d %8d", errdata  [0],errdata  [1]);
 HdlcErrorRpt();			// Print statistics
 return(rvl);
}

////////////////////////////////////////////////////////////////////////////////
// HDLC internal loopback //////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

// cn =  0, HDLC Channel A Test
// cn =  1, HDLC Channel B Test 
// cn = -1, HDLC Channel A & B Test 
    
int HdlcInternalLoopback(int cn,int lcnt, int quiet)
{
 int rvl;
 int i;
 for(i = 0; i < 2; ++i)
  {
   if(i == cn || cn == -1)
    { 					// set loopback mode bit
     HCON(i) |= HCON_TxLOOP;
    }
  }  

 rvl = HdlcLoopback(cn,lcnt,quiet);

 for(i = 0; i < 2; ++i)
  {
   if(i == cn || cn == -1)
    {					// clear loopback mode bit 
     HCON(i) &=~HCON_TxLOOP;
    }
  }
 return(rvl);
}

////////////////////////////////////////////////////////////////////////////////
// HDLC External Loopback Function /////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

int HdlcExternalLoopback(int cn,int lcnt,int quiet)
{
 int rvl;
 rvl = HdlcLoopback(cn,lcnt,quiet);
 return(rvl);
}

////////////////////////////////////////////////////////////////////////////////
// HDLC Single transfer test ///////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

void HdlcSingleTransfer(int cn) 
{
 U8 data[512];				// data buffer 
 int len;				// packet length 
 int pos;				// data buffer position 
 int tnum;				// Tx packet number

 if(cn < 0 || cn > 1)
  {					// invalid channel number 
   return;				// 
  } 

 Print("\n\nHDLC Channel %c Single Transfer", 'A' + cn);

// buffer length ///////////////////////////////////////////////////////////////
 Print("\nInput the Tx BufLength : ");
 len = get_number(10,3);		// read length from console 
 if(len <   6){ len = 6; } 		// set to mimimum limit 
 if(len > 512){ len = 512;} 		// set to maximum limit  

// enable HDLC Tx & Rx 
 HCON(cn) |= HCON_DTxEN | HCON_DRxEN | HCON_TxEN | HCON_RxEN;
 Enable_Int(nGLOBAL_INT);		// enable global int

 tnum = 0;				// Tx packet number
 while(1)
  {
   U8 it;
   Print("\nSelect Transmit(T) or Quit(Q) ? ");
   it = get_upper();
   if(it == 'Q')
    {					
     break;				// exit from test 
    }  

// fill sended packet //////////////////////////////////////////////////////////
   data[0] = 0xFF;			// address
   data[1] = (U8) len;			// length 
   data[2] = (U8) (tnum & 0xFF);	// packet number 
   data[3] = (U8)((tnum >>  8) & 0xFF);
   data[4] = (U8)((tnum >> 16) & 0xFF);
   data[5] = (U8)((tnum >> 24) & 0xFF);

   for(pos = 6; pos < len; ++pos)
    {					
     data[pos] = tnum + pos;		// fill packet 
    }    

   if(HDLCSend(cn,data,len) == 0)
    {
    }
   else
    {
     tnum++;				// packet sended
    } 
  }

 Disable_Int(nGLOBAL_INT);		// disable global interrupt 
 HCON(cn) &= ~(HCON_DTxEN | HCON_TxEN | HCON_DRxEN | HCON_RxEN);	
 Print("\nTx frames                %d", tnum);
#ifdef HDLC_RESET
 Print("\nResetting HDLC %c",(cn?'B':'A'));
 HDLCRx_reset(cn);			// reset Rx BD list 
 HDLCTx_reset(cn);			// reset Tx BD list 
#endif 								// HDLC_RESET
}


////////////////////////////////////////////////////////////////////////////////
// HDLC Multiple Transfer test /////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////


void HdlcMultipleTransfer(int cn)
{
 U8 data[512];				// data buffer 
 int len;				// packet len
 int cnt;				// packet cnt 
 int pos;				// buffer position 
 int tnum;				// packet number 
 U32 status;
 int recv;

 if(cn < 0 || cn > 1)
  {					// invalid channel 
   return;				// number 
  } 

 Print("\n\nHDLC Channel %c Multiple Transfer", 'A' + cn);

// Tx buffer length ////////////////////////////////////////////////////////////

 Print("\nInput the Tx BufLength : ");	// 
 len = get_number(10,3);		// read length from console 
 if(len <   6){ len = 6; } 		// set length to minimum limit 
 if(len > 512){ len = 512;} 		// set length to maximum limit

// Tx frames count /////////////////////////////////////////////////////////////
 Print("\nInput Packet count     : ");
 cnt = get_number(10,6);
 if(cnt <   1){ cnt = 1; } 

 if(cnt > 10)
  {
   Print("\n#=10 packets sended."); 
  }
 Print("\nTo break test press [Q].");

// enable HDLC Tx & Rx, global interrupt   
 HCON(cn) |= (HCON_DTxEN | HCON_TxEN | HCON_DRxEN | HCON_RxEN);	
 Enable_Int(nGLOBAL_INT);

 tnum = 0;				// packet number 
 while(tnum < cnt)
  {
   if(kbd_hit())
    {
     U8 it;
     it = get_byte();
     if(it == 'Q' || it == 'q')
      {
       break;
      }  
    }  
  
// 
   data[0] = 0xFF;			// address
   data[1] = (U8) len; 			// length 
   data[2] = (U8) (tnum & 0xFF);	// packet number 
   data[3] = (U8)((tnum >>  8) & 0xFF); 
   data[4] = (U8)((tnum >> 16) & 0xFF);
   data[5] = (U8)((tnum >> 24) & 0xFF); 

   for(pos = 6; pos < len; ++pos)
    {					// fill packet 
     data[pos] = tnum + pos;
    }    

   if(HDLCSend(cn,data,len) == 0)
    {
    }
   else
    {
     tnum++;				// 
     if(tnum != 0 && (tnum % 10) == 0)
      {
       Print("#");
      }
    } 

//
   while(1)
    {
     len = sizeof(data);
     recv = HDLCRecv(cn, &status, data, & len);
     if(
       recv == -1 ||
       recv ==  0
      )
      {
       break;
      }  
    }
  }

 Disable_Int(nGLOBAL_INT);		// disable global int
 HCON(cn) &= ~(HCON_DTxEN | HCON_TxEN | HCON_DRxEN | HCON_RxEN);	
 Print("\nTx frames                %d", tnum);
#ifdef HDLC_RESET
 Print("\nResetting HDLC%c",(cn?'B':'A'));
 HDLCRx_reset(cn);			// reset Rx BD list 
 HDLCTx_reset(cn);			// reset Tx BD list 
#endif 							// HDLC_RESET
}


void HdlcMultipleReceive(int cn)
{
 U8 data[1024];				// data buffer 
 U32 status;
 int len;				// packet len
 int pos;				// buffer position 
 int num;
 int tnum;				
 int tlen;

 if(cn < 0 || cn > 1)
  {					// invalid channel number 
   return;
  } 

 Print("\n\nWaiting for a HDLC%c Frame Received", 'A' + cn);
 Print("\nTo break test press [Q].");

// 
 HCON(cn) |= HCON_DTxEN | HCON_TxEN | HCON_DRxEN | HCON_RxEN;
 Enable_Int(nGLOBAL_INT);		// enable global int

 tnum = 0;
 tlen = 6;
 while(1) 
  {
   if(kbd_hit())
    {
     U8 it;
     it = get_byte(); 
     if(it == 'Q' || it == 'q')
      {
       return;
      }  

     if(it == 'T' || it == 't')
      {
// Transmit ////////////////////////////////////////////////////////////////////
       if(tlen <    6) {tlen =    6;}	// set Tx length to minimum limit 	
       if(tlen > 1024) {tlen = 1024;}	// set Tx length to maximum limit 

       data[0] = 0xFF;			// address
       data[1] = (U8)  tlen; 		// length 
       data[2] = (U8) (tnum & 0xFF);	// packet number 
       data[3] = (U8)((tnum >>  8)&0xFF); 
       data[4] = (U8)((tnum >> 16)&0xFF);
       data[5] = (U8)((tnum >> 24)&0xFF); 

       for(pos = 6; pos < tlen; ++pos)
        {				// fill packet 
         data[pos] = tnum + pos;
        }    
// 
       if(HDLCSend(cn,data,tlen) == 0)
        {
        }
       else
        {
         tnum += 1;			// next Tx packet number 
         tlen += 1;			// next Tx length 
        } 
      }
    }

   len = 1024;				// packet len
   if(HDLCRecv(cn, & status, data, & len) == 0)
    {
     continue;
    } 

// dump packet content /////////////////////////////////////////////////////////

   Print("\n");
   pos = 0;				// start buffer position 
   while(pos < len)
    {
     Print("\n");
     for(num = 0; num < 16; ++num)	// line position 
      {
       if(pos + num < len)
        {
         Print("%02x ", data[pos+num]);
        }
       else
        {
         Print("   ");
        }
      }

     for(num = 0; num < 16; ++num)
      {
       if(pos + num < len)
        {
         U8 ch; 
         ch = data[pos+num];
         if(ch >= 32 && ch < 123)
          {
           Print("%c",ch);
          }
         else
          {
           Print(".");
          }
        }
       else
        {
         Print(" ");
        }
      }

//
     pos += 16;
    }
   Print("\n");
  }
 
 Disable_Int(nGLOBAL_INT);		// enable global int
 HCON(cn) &= ~(HCON_DTxEN | HCON_TxEN | HCON_DRxEN | HCON_RxEN);
#ifdef HDLC_RESET
 Print("\nResetting HDLC%c",(cn?'B':'A'));
 HDLCRx_reset(cn);			// reset Rx BD list 
 HDLCTx_reset(cn);			// reset Tx BD list 
#endif 							// HDLC_RESET
}	

////////////////////////////////////////////////////////////////////////////////
// HDLC Test Main Function /////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

void HdlcTest(void) 
{
// 
 HDLC_Init();				// Initialize HDLC channel A,B

 while(1) 
  {
   U8 it;
// 
   Print("\nHDLC Block Diagnostic Program.");
   Print("\n[1] HDLC A LoopBack.");		// 
   Print("\n[2] HDLC B LoopBack.");		// 	
   Print("\n[3] HDLC A & B LoopBack.");		// 	
   Print("\n[4] HDLC A Single Transfer.");	// 
   Print("\n[5] HDLC A Multiple Transfer.");	// 
   Print("\n[6] HDLC A Receive.");		// 
   Print("\n[7] HDLC B Single Transfer.");	// 
   Print("\n[8] HDLC B Multiple Transfer.");	// 
   Print("\n[9] HDLC B Receive.");		// 
   Print("\n[A] HDLC A External LoopBack.");	// 	
   Print("\n[B] HDLC B External LoopBack.");	// 
   Print("\n[V] HDLC Report View.");
   Print("\n[C] HDLC Report Clear.");
   Print("\n[G] Global Variable View.");
   Print("\n[R] HDLC registers view.");
   Print("\n[T]      set HDLC baudrate.");
   Print("\n[S] Reset HDLC.");
   Print("\n[Q] Quit HDLC Test.");

   Print("\nSelect Test Item : "); 
   it = get_upper();
	
   switch(it)
    {
     case '1' : HdlcInternalLoopback(0,10000, 0);break; // HDLC a Internal LoopBackTest
     case '2' : HdlcInternalLoopback(1,10000, 0);break; // HDLC B Internal LoopBackTest
     case '3' : HdlcInternalLoopback(-1,10000,0);break; // HDLC A & B Internal LoopBackTest
     case '4' : HdlcSingleTransfer(0);  break; 		// HDLC A Single Transfer
     case '5' : HdlcMultipleTransfer(0); break;		// HDLC A Multiple Transfer
     case '6' : HdlcMultipleReceive(0); break;		// HDLC A Recive
     case '7' : HdlcSingleTransfer(1); break;		// HDLC B Single Transfer
     case '8' : HdlcMultipleTransfer(1); break;		// HDLC B Multiple Transfer
     case '9' : HdlcMultipleReceive(1); break;		// HDLC B Multiple Receive
     case 'A' : HdlcExternalLoopback(0,10000, 0);break;	// HDLC A External LoopBackTest
     case 'B' : HdlcExternalLoopback(1,10000, 0);break;	// HDLC B External LoopBackTest
     case 'V' : HdlcErrorRpt(); break;			// HDLC Report View
     case 'C' : HdlcClrErrReport(); break;		// HDLC Report Clear
     case 'G' : HdlcVarView(); break;	 	 	// View Global Variables
     case 'R' : HdlcRegView(); break;			// View HDLC Registers
     case 'S' : HdlcResetTest(); break;		 	// Reset Tests 
     case 'T' : HdlcSetBrg(); break;			//
     case 'Q' : 
      return;

     default  : 
      Print("\nInvalid Test Item Selected") ; 
      break ;
    }

   Print("\nPress any key to continue"); 
   it = get_byte();
   if(it == 'Q' || it == 'q')
    {
     return;
    }
  }
}



