#ifndef _EVM50100_H_
#define _EVM50100_H_


/* 			*/
/* Modified by 		*/
/* Dmitriy Cherkashin 	*/
/* dch@ucrouter.ru	*/
/* 2002, 2003		*/
/*/			*/

#ifdef  _S3C4530_	
#define _KS32C50100_
#else
#ifndef _KS32C50100_ 
#define _KS32C50100_ 
#endif
#endif 								// _S3C4530_

/*=====================================================================*/
/*                                                                     */
/*                     SYSTEM CLOCK CONTROL                            */
/*                                                                     */
/*   * fMCLK   SYSTEM CLOCK Oscillator frequency                       */
/*   * REF_CYCLE  Unit[ms], Default refresh cycle is 14ms              */
/*                                                                     */
/*=====================================================================*/
#define MHz      1000000

#ifdef  _S3C4530_
/* compiled with S3C4530 support */
#define fMCLK                 50*MHz       
#define MCLK2                 (fMCLK/2)
#define fMCLK_MHz	      50 		
#else
#ifdef  _KS32C50100_
/* compiled without S3C4530 support */
#define fMCLK                 50*MHz       
#define MCLK2                 (fMCLK/2)
#define fMCLK_MHz	      50 		// unit MHz 	
#else
/* KS32C5000(A)*/
#define fMCLK                 33*MHz       
#endif
#endif

// Delay for clock stabilization when CPU clock change
#define Delay4ClkCange    100      
#define REF_CYCLE          16      // Unit[ms]. default refresh cycle 16ms. 	/* - */

/*=====================================================================*/
/*                                                                     */
/*                      DRAM MEMORY                                    */
/*                                                                     */
/*                                                                     */
/*=====================================================================*/
//#define DownLoadArea	0x1000050   //User code down load area


////////////////////////////////////////////////////////////////////////////////
// SERIAL CHANNEL (UART0/UART1) ////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

#define baudrate       38400    

////////////////////////////////////////////////////////////////////////////////
// TIMER ///////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

/* This is the number of timer ticks that occur per second. */
#define TICKS_PER_SECOND        100

/* Number of ticks for one second  */
#define ONE_SECOND              1000    


/* Define data types for ks32c6x00 */

#define U8            unsigned char
#define S8            signed char
#define U16           unsigned short
#define S16           signed short
#define U32           unsigned long
#define S32           signed long
#define REG8          volatile unsigned char
#define REG16         volatile unsigned short
#define REG32         volatile unsigned long
#define BYTE          U8

/* max/mins */
#define USHORT_MAX   65535  /* The maximum value of a uint16 variable. */

#ifndef NULL
#define NULL 0
#endif

////////////////////////////////////////////////////////////////////////////////
// main.c //////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

extern U32 syscfg_pd_id;		// SYSCFG PD_ID field

#ifdef  _S3C4530_ 
#define is_s3c4530(id) ( ((id) == SYSCFG_PD_S3C4530X) || ((id) == SYSCFG_PD_S3C4530A) )
#else
#define is_s3c4530(id) (0)
#endif								/* _S3C4530_ */

extern char Image$$RO$$Base[];		//
extern char Image$$RO$$Limit[];		//
extern char Image$$RW$$Base[];		//
extern char Image$$RW$$Limit[];		//
extern char Image$$ZI$$Base[];		// 
extern char Image$$ZI$$Limit[];		// 

extern void Main(void);			// main Menu function 
extern void InitEvb(void);		// Init evaluation board
extern void ViewSysConfig(void);  	// View System Configureation


////////////////////////////////////////////////////////////////////////////////
// isr.c ///////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////


/* Macro fucntions */

#define	Enable_Int(n)       INTMASK &= ~(1<<(n))/* clear interrupt source bit */ 
#define	Disable_Int(n)	    INTMASK |= (1<<(n))	/* set interrupt source bit */
#define Clear_PendingBit(n) INTPEND  = (1<<(n))	/* clear pending pit */
#define SetPendingBit(n)    INTPNDTST= (1<<(n))  /* set pending bit via Interrupt Pending Test Register */ 


/* 
 *     Prototype for interrupt functions 
 */


#define	MAXHNDLRS	21			/* Maximum # of interrupt  handlers */

/* Function Prototype for Interrupt. ATTENTION : call before enable interrupt */
extern void InitInterrupt(void);
/* Clear all interrupt related register. Disable all interrupts (include interrupt mask 
register global bit), clear pending register, all interrupt FIQ interrupts  */
extern void ClrIntStatus(void);  		

/* Function Prototype for Exception Vector */
extern void ISR_UndefHandler(REG32 *adr);	/* Undefined instruction  */
extern void ISR_PrefetchHandler(REG32 *adr);	/* Prefetch Abort */
extern void ISR_AbortHandler(REG32 *adr);	/* Data abort handler */
extern void ISR_SwiHandler(void);		/* Software interrupt */
extern void ISR_IrqHandler(void);		/* IRQ */	
extern void ISR_FiqHandler(void);		/* FIQ */

/* Function Prototype for Interrupt Setup */
extern void SysSetInterrupt(REG32 vector, void (*)());
extern void * SysGetInterrupt(REG32 vector);

extern void ViewIntConfig(void);		/* Print INTMODE, INTMASK, INTPRI, INTPEND, INTOFFSET */
extern void IntTest(void);			

/* Interactive request to enter interruspts priority for each of 21 sources */
extern void SetPriority(void);			
/* Interactive set interrupt mode register, interrupt mask register */
extern void SetIntMode(void);			

////////////////////////////////////////////////////////////////////////////////
// uart.c //////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// SYMBOL DEFINITIONS //////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

#define KEY_NL          0x0A		/* new line LF */
#define KEY_CR          0x0D		/* carriage return */
#define KEY_BSP         0x08		/* back space */
#define KEY_ESC         0x1B		/* escape */
#define KEY_CTRLZ       0x1A
#define KEY_RUBOUT      0x7F

#define CONSOLE            0  		/* default console channel */

////////////////////////////////////////////////////////////////////////////////
// UART INITIALIZE FUNCTIONS ///////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
extern U32  UART_Initialize(void);	/* Init UART0, UART1 parameters in sysconf.h */
extern int  UARTSetSpeed(U32 ch,int sp);/* set UART channel speed */
extern int  UARTGetSpeed(U32 ch);	/* read UART channel speed */ 
extern int  UARTWaitTxComp(int /*ch*/); /* wait tx completion in polling mode */
extern int  UARTSetLoopBack(int /*ch*/, int on);/* set loop back mode */
 
////////////////////////////////////////////////////////////////////////////////
// UART INTERRUPT SERVICE ROUTINES & CONTROL FUNCTIONS /////////////////////////
////////////////////////////////////////////////////////////////////////////////

extern void UARTTxIntOn(int /*ch*/);	/* Enable UART0/1 Tx interrupt, Set UART0/1 Tx Interrupt pending bit */
extern void UARTTxIntOff(int /*ch*/);	/* Diasable UART0/1 Tx Interrupt, Clear UART0/1 Tx Interrupt pending bit*/	
extern void UARTRxIntOn(int /*ch*/);	/* Enable UART0/1 Rx&Status Interrupt */	
extern void UARTRxIntOff(int /*ch*/);	/* Disable UART0/1 Rx&Status Interrupt */	
 				 
/************************************************************************/
/*            UART STANDARD I/O FUNCTIONS FOR INTERRUPT             	*/
/************************************************************************/
extern  int  i_getc(int /*channel*/, U8 * /*ch*/); /* get char in interrupt Rx queue*/ 		
//extern  U32  i_gets(U32 /*channel*/, U8 *);	/* unused contain ERROR */
/* put char in interrupt mode, if last transmission not completed, return 0  */
extern  int  i_putc(int /*channel*/, U8 /*ch*/);
/* put string to UART */
extern  int  i_puts(int /*channel*/, char * /*string*/);
/* formatted output to console */
extern  int  i_flush(int /*channel */);

/************************************************************************/
/*            UART STANDARD I/O FUNCTIONS FOR POLLING               	*/
/************************************************************************/
/* put char to UART0/UART1 in polling mode */
extern int  put_char(int /*channel*/, char /*data*/);	
extern int  put_str(int /* ch */, char * /*str*/);
/* get char from UART0/UART1 in polling mode */ 
extern char get_char(int /*channel*/); 			
extern int  rcv_char(int /* ch */, U8 * /*val*/);

/* Can be used in polling mode */
extern void put_byte(char /* ch */);	/* put char to console */
extern char get_byte(void);		/* get char from console */
extern char get_upper(void);		/* get upper char from console */ 
extern char get_last(void);		/* get last received from console charecter */ 
extern int  kbd_hit(void);		/* return 1, if console buffer not empty */			

////////////////////////////////////////////////////////////////////////////////
extern void  UartTest(void);		/* main UART test menu */
extern U32   UARTAutoLoopBack(int);  	/* Internal loopback poll */
extern void  StringEcho(U32 /*ch*/);	/* String echo in interrupt mode */
extern void  UARTConfigView(void);	/* View UART0/UART1 configuration */

extern void PrintMBIT(char * mes);
extern void PrintUBIT(int ch, char * mes, U32 * val, U32 msk);
extern void PrintSBIT(int ch, char * mes, U32 * st, U32 msk, char * v1, char * v0);
extern void PrintEBIT(int ch, char * mes, U32 * st, U32 msk);

typedef struct 
{
 char * str;
 U32    val;
}ABIT;

extern void PrintABIT(int ch, char * mes, U32 * st, U32 msk, ABIT * abit);


////////////////////////////////////////////////////////////////////////////////
// memory.c ////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////


// return free dram regions, in input size - sizeof arrays base, limit,
// in output count of free regions 
extern int FreeDramRegions(U32 * base, U32 * limit, int *size);


/* constants for memory allocation functions from cacheble and notcachable areas */

/* DRAM1 (if used)............0x2000000 */
/* DRAM0 (16Mbyte)............0x1000000 */
//#define HEAPEND               0x1FF0000		/* from RW$$Limit to DRAM0 Limit - 64Kb */	
//#define AllocNonCacheAreaBase 0x5FF0000		/* from DRAM0 Limit - 64K  		*/ 
//#define AllocNonCacheAreaEnd  0x6000000		/* to   DRAM0 Limit  			*/

/************************************************************************/ 
/* NON-CACHEABLE AREA CONTROL BIT (from KS32C50100 manual) 		*/
/* 									*/
/* Allthow the cache affects the entire system memory, it is sometimes 	*/
/* necessary to define  non-cacheable areas when the consistency of 	*/
/* data stored in memory and the cache must be ensured. To support 	*/
/* this, the KS32C50100 provides a non-cacheable control bit in the 	*/
/* address field, ADDR[26] (0x400_0000). 				*/		
/* 									*/
/* If ADDR[26] in the ROM/SRAM, flash memory, DRAM, or external I/O 	*/
/* bank's access address is 0, then accessed data is cacheable. If the	*/
/* ADDR[26] value is 1,  the accessed data is non-cacheable 		*/    
/************************************************************************/ 

extern void MemoryTest(void);		/* Memory Test Select Function */			

//                                     
/*
 *  Function for byte copy from srcp to dstp with count times
 *  (bcopy: Byte copy, scopy: Halfword, wcopy: word)
 *
 *  Where dstp is destination address,
 *        srcp is soruce address
 *        words is memory test size (words)
 */

extern int bcopy(void * /*srcp*/, void * /*dstp*/, int /*words*/, int);
extern int scopy(void * /*srcp*/, void * /*dstp*/, int /*words*/, int);	
extern int wcopy(void * /*srcp*/, void * /*dstp*/, int /*words*/, int);	

/*
 * Memory data compare test functions
 * srcp  = source address  
 * dstp  = destination address
 * words = memory test size (words)
 */

extern int bcomp(void * /*srcp*/, void * /*dstp*/, int /*words*/, int);	
extern int scomp(void * /*srcp*/, void * /*dstp*/, int /*words*/, int);	
extern int wcomp(void * /*srcp*/, void * /*dstp*/, int /*words*/, int);	


extern int  PatternGen(unsigned int /*seed*/);	/* Generate random fill pattern */		
extern void PrintRotSlash(int);			/* print memory test progress*/		

/*
 * Memory word, halfword, byte copy & compare test program
 * source address is src, destination address is dest, test memory size (words), 
 * size test repeat times is repeat.
 */

extern int MemTest(U32 * /*src*/,U32 * /*dest*/,int /*size*/,int /*repeat*/, int);

/* 
 *  DRAM Memory dump test program.
 *  Memory dump initial address is 0x1000000 (requested) 
 *                  end address is 0x1000100 (requested)	
 */

extern void MemoryDump(void);					
extern void MemDump(U32 * /*start address*/,U32 * /*End address*/);

////////////////////////////////////////////////////////////////////////////////
// pollio.c ////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

extern unsigned long Print(char * /*format */, ...);
// Read a number from the console
extern unsigned long get_number(int , int /* digits */);  

////////////////////////////////////////////////////////////////////////////////
// system.c ////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////


/* Cache memory direct access address */
#define Set0CacheRAM    0x10000000  	/* Set 0 Cache ram address */
#define Set1CacheRAM    0x10800000  	/* Set 1 Cache ram address */
#define TagRAM          0x11000000  	/* Tag ram address */
#define NonCacheArea    0x04000000  	/* ADDR[26] NonCacheable control bit */


void PrintCacheConfig(void);		/* Print Cache Configuration */
extern void CacheTest(void);		/* Cache  test top module */
extern void CacheFlush(void);		/* Cache flush (clear tag RAM) */


// Main 10N March Algorithm ////////////////////////////////////////////////////
// addr       = memory region start address
// word count = memory region size (words)
// patter     = pattern 
// prog       = 1 print progress indicator
// 
// return   1 = success
//          0 = failure  

extern int March10N32(unsigned int */*addr*/,int /*word count*/,unsigned int /*pattern*/, int prog);

/*
 *  All 10N March Test
 *  March-10N Test for Tag-RAM, 4K-SRAM, 8K-SRAM, Set0, Set1
 */

extern int March10NAllTest(void);
extern int TagMarch10(void);        	// test tag ram (256 words)
extern int SramMarch10_4k(void);   	// test SRAM in 4k mode
extern int SramMarch10_8k(void);  	// test SRAM in 8k mode	
extern int Set0March10(void);        	// test cache set 0 in SRAM disable mode
extern int Set1March10(void);      	// test cache set 1 in SRAM disable mode

/* init cache, where cmd : */
extern void CacheInit(int /*cmd*/);

#define CACHEINIT_4K	0		/* 0 = 4Kb - SRAM, 4Kb - cache */
#define CACHEINIT_8K    1 		/* 1 = 0Kb - SRAM, 8Kb - cache */
#define CACHEINIT_0K    2 		/* 2 = 8Kb - SRAM, 0Kb - cache */ 
#define CACHEINIT_OFF   2		/* 2 = Off */ 


////////////////////////////////////////////////////////////////////////////////
// timer.c /////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////


#define tmDATA(t)           (t*fMCLK_MHz * 1000 - 1)    // t is time tick,unit[ms]
#define t_data_ms(t)        (t*fMCLK_MHz * 1000 - 1)    // t is time tick,unit[ms]
#define t_data_us(t)        (t*fMCLK_MHz - 1)  		// t is time tick,unit[us]

typedef struct 
{
 volatile unsigned int tm_ms;		/* millisecond */
 volatile unsigned int tm_sec;		/* sec */
 volatile unsigned int tm_min;		/* min */
 volatile unsigned int tm_hour; 	/* hour */
 volatile unsigned int tm_mday;		/* month day */
 volatile unsigned int tm_mon;		/* month */	
 volatile unsigned int tm_year;		/* year */
}TIME; 


extern void TimerStart(int t);		/* start timer 0,1 */
extern void TimerStop(int t);		/* stop timer 0,1 */

/* init timer, set interval mode, set timer ISR, enable timer interrupt source */
extern void TimerInit(int /*timer dev*/,int /*timer data ms*/);	 
extern void TimerReset(int /* timer device number */);	/* reset timer */		
extern int  TimerEnabled(int /* timer */);
extern int  TimerWaitMs(int /*num*/ , U32 /* mswait */ );
extern void GetSysTime(int /* timer */,TIME * );	/* get system time */			
extern void tm0isr(void);    		/* timer0 interrupt service */	
extern void tm1isr(void);    		/* timer1 interrupt service */		

extern void TimerTest(void);		/* timers test main function */	


////////////////////////////////////////////////////////////////////////////////
// iic.c ///////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

extern void IICTest(void);		// main test function  
extern void IICSetup(void);		// reset IIC, set prescaler  
extern int  IICWriteReadTest(int lcnt);	// write & read test 
extern void IICReadTest(void);		// read test  
extern void IICWriteTest(void);		// write test 
extern void IICWriteByteTest(void);	// read addr, byte from console and write to EEPROM
extern void IICReadByteTest(void);	// read addr from console and read byte from EEPROM
extern int  IICByteTest(void);		// byte write & read test 

// write & read buffer in interrupt mode ///////////////////////////////////////

extern int  IICWrite(U8 /*devaddr*/,U32 /*addr*/,U8 */*data*/,U32 /*len*/);
extern void IICRead(U8 /*devaddr*/,U32 /*addr*/,U8 */*data*/,U32 /*len*/);

// write & read byte in pooling mode ///////////////////////////////////////////

extern int  IICWriteByte(U32 /*addr*/, U8 /*ch*/);	// write byte in pooling mode 
extern int  IICReadByte(U32 /*addr*/, U8 * /*ch*/);	// read byte in pooling mode 

extern int  MacAddrReadIIC(U8 * data);
extern int  MacAddrWriteIIC(U8 * data);

////////////////////////////////////////////////////////////////////////////////
// dma.c ///////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////


extern void GdmaTest(void);		// GDMA0/GDMA1 Test Top Module
extern void GDMAStartUpDialog(void);	// GDMA Configure Dialog 
extern void GdmaMemMemTest(void);	// GDMA Memory to Memory test Module
extern void UartToMemoryTest(int un, int gdma);	// UART to memoty Test  
extern void MemoryToUartTest(int un, int gdma);	// Memory to UART test 
extern void GdmaReset(unsigned cn);	 // Reset GDMA	
extern void GdmaIntEnable(unsigned cn);	 // Enable GDMA Channel Interrupt & Global interrupt  
extern void GdmaIntDisable(unsigned cn); // Disable GDMA Channle Interrupt
extern void GDMA0isr(void);		 // GDMA 0 ISR 
extern void GDMA1isr(void);		 // GDMA 1 ISR 

// copy function using GDMA 0/1 (global interrupt must be enabled) /////////////
extern void gdma0copy(U32 dmadst, U32 dmasrc,int Size, int Width);
extern void gdma1copy(U32 dmadst, U32 dmasrc,int Size, int Width);

// gdma0copy = copy using GDMA 0 
// gdma1copy = copy using GDMA 1
// dmasrc    = source address 
// dmadst    = destination address 
// Size      = transfer count
// Width     = 0 - byte (8 bit)
//             1 - half word (16 bit)
//             2 - word (32 bit) 

extern int DmaAutoTest(U32 src,U32 dst,int tsize,int lsize) ;	

////////////////////////////////////////////////////////////////////////////////
// hdlcinit.c & hdlcmain.c /////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

extern void HdlcTest(void);		// HDLC Test Main Function
extern void HDLC_Init(void);		// HDLC Channals A,B initialize
// Init Tx Buffer Descriptors ////////////////////////////////////////////////// 
// cn = channel number 
//      0 = HDLCA
//      1 = HDLCB
// np = Null List Position 
//      0 = circular buffer descriptor list   
//      1 - (HDLC_MAX_TX_DES-1) = BD[np-1]->Next = 0; 
extern void TxBD_init(int cn, int np); 
extern void RxBD_init(int cn);		// Init Rx Buffer Descriptors

extern int  HDLCSend(int cn, U8 * data, int size);
extern int  HDLCRecv(int cn, U32 * status, U8 * data, int * size);
extern int  HDLCRx_reset(int cn);	// reset Rx 
extern int  HDLCTx_reset(int cn);	// reset Tx 
extern int  HDLCTxBD_count(int cn);	// count Tx BD with owner DMA

// 
extern void HDLCRxA_isr(void);		//
extern void HDLCTxA_isr(void);		//
extern void HDLCRxB_isr(void);		//
extern void HDLCTxB_isr(void);		//

// HDLC Internal Loopback Function, cn - channel number (-1 all channels), 
// pcnt = tx frames count, (quiet == 1) => don't print error messages  
extern int HdlcInternalLoopback(int cn,int pcnt,int quiet); 
extern void HdlcRegView(void);		// Register View
extern void HdlcErrorRpt(void);		// HDLC Statistic

extern void HdlcResetTest(void);	// Reset Tests
extern void HdlcSingleTransfer(int);	//
extern void HdlcMultipleTransfer(int);	//
extern void HdlcClrErrReport(void);	// Clear Statistics 
extern void HdlcVarView(void);		// View Global Variables 

// HDLC Statistics /////////////////////////////////////////////////////////////
extern volatile U32   gHDLCRxDone;	// used in ISR CPU Mode
extern U32            gHdlcBaudRate;	// 

////////////////////////////////////////////////////////////////////////////////
// iop.c ///////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

extern void IopTest(void);			/* I/O Function Test Main module */	
extern void IoPortRw(void);			/* Read/Write port */
extern void SetupIoport(void);			/* Setup I/O port registers */

/* External interrupt service routine */

extern void ext0isr(void);			/* External interrupt 0 ISR */
extern void ext1isr(void);			/* External interrupt 1 ISR */
extern void ext2isr(void);			/* External interrupt 2 ISR */
extern void ext3isr(void);			/* External interrupt 3 ISR */

extern void ExtIntReq(int /*channel*/);		/* test external interrupt request */
extern void ExtIntAllEnable(void);		/* Enable External Interrupt 0,1,2,3 */
extern void ExtIntAllDisable(void);		/* Disable External Interrupt 0,1,2,3 */

////////////////////////////////////////////////////////////////////////////////
// zmport.c ////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

extern int ZmTest(void);

#endif 							// _EVM50100_H_
