/********************************************************************/
/*       MAC Header for KS32C5000                                   */
/*------------------------------------------------------------------*/
/*      Copyright (C) 1997 Samsung Electronics.                     */
/*------------------------------------------------------------------*/
/*      Modified, programmed by hbahn                               */
/*                                                                  */
/*      Description : 1998-2-10 first edited for KS32C5000          */ 
/********************************************************************/

/* 			*/
/* Modified by 		*/
/* Dmitriy Cherkashin 	*/
/* dch@ucrouter.ru	*/
/* 2002			*/
/*			*/

#ifndef _MAC_
#define _MAC_

#define MAC_ADDR_SIZE         6      /* dst,src addr is 6bytes each*/ 

#define MaxRxFrameSize  	1520 // Rx Frame Max Size 
#define MAC_MAX_RX_DES  	64   // Max number of Rx Frame Descriptors 
#define MAC_MAX_TX_DES  	16   // Max number of Tx Frame Descriptors

////////////////////////////////////////////////////////////////////////////////
// Tx/Rx common descriptor structure ///////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

typedef struct FrameDes 
 {	
  U32 DataPtr;		  	// Data pointer 	
// cf: RX-reserved, TX-Reserved(25bits) + Control bits(7bits)
  U32 Reserved; 		// 
  U32 StatusLength;		// status & length field
  U32 NextDes;			// next descriptor
 } sFrameDes;

/************************************************************************/
/* 	Data structure of Rx Frame descriptor 				*/
/*									*/ 
/* 	| 31|  30              16 | 15                          0	*/
/* 	----------------------------------------------------------	*/
/* 	| O |             Frame Data Pointer                     |	*/
/* 	----------------------------------------------------------      */
/* 	|                      Reserved                          |	*/
/* 	----------------------------------------------------------      */ 
/* 	| Rx Status               | Frame Length                 |    	*/
/* 	----------------------------------------------------------	*/
/* 	| Next Frame              | Descriptor Pointer           |	*/
/* 	----------------------------------------------------------	*/
/* 									*/
/* 	[31] 	Ownership bit (O) 					*/
/*              0 = CPU							*/
/* 		1 = BDMA						*/
/* 									*/
/* 	[30:0]  Frame data pointer					*/
/* 		Address of the frame data to be saved.			*/
/* 									*/
/* 	[15:0]  Frame length 						*/
/* 		The size of the received frame. 			*/
/* 									*/
/* 	[31:16] Rx status						*/
/* 		The Rx status field of the receive frame is updated	*/
/* 		by the MAC after reception is complete. 		*/
/* 									*/
/* 	[31:0] 	Next frame descriptor pointer				*/
/* 		Address of next frame descriptor 			*/
/************************************************************************/

////////////////////////////////////////////////////////////////////////////////
// TxRx Frame Descriptor ///////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

#define	fOwnership_BDMA	0x80000000 // 0:CPU, 1:BDMA
#define	fOwnership_CPU	0x7fffffff // 0:CPU, 1:BDMA

/************************************************************************/
/* Rx Descriptor Status Bits 						*/
/* 									*/
/* [19]	 Over maximum size (OvMax)			      	      	*/
/*   3	 Set if the received frame data size exceeds the maximum  frame */
/* 	 size. 								*/
/* 									*/
/* [21]  Control received (CtlRcv)				      	*/
/*   5 	 Set if the received packet is a MAC control frame.	        */
/* 									*/
/* [22]  Interrupt on receive (IntRx)				        */
/*   6   Set if reception of packet caused an interrupt condition.      */
/* 	 This includes Good received, if the Engood bit,MACRXCON [14],	*/
/*	 is set. 							*/
/* 									*/
/* [23]  Receive 10 Mb/s status (Rx10Stat)				*/
/*   7   Set if packet was received over the 10-Mbyte/s interface. 	*/
/*	 Reset if packet was received over the MII.			*/
/* 									*/
/* [24]  Alignment error (AlignErr)					*/
/*   8   Frame length in bits was not a multiple of eight and the CRC 	*/
/*       was invalid. 							*/
/* 									*/
/* [25]  CRC error (CRCErr) 						*/
/*   9   CRC at end of packet did not match the computed value, or else */
/* 	 the PHY asserted Rx_er during packet reception.		*/
/* [26]  Overflow error (Overflow)					*/
/*  10   The MAC receive FIFO was full when it needed to store a 	*/
/*       received byte. 						*/
/* [27]  Long error (LongErr) 						*/
/*  11   Received a frame longer than 1518 bytes. Not set if the Long 	*/
/*       enable bit in the receive control register is set. 		*/
/* [29]  Receive parity error (RxPar) 					*/
/*  13   MAC receive FIFO has detected a parity error. 			*/
/* [30]  Good received (Good) 						*/
/*  14   Successfully received a packet with no errors. If EnGood = 1,  */
/*       an interrupt is generated on each packet that is received 	*/
/*       successfully. 							*/
/* [31]  Reception halted (RxHalted) 					*/
/*  15   Reception interrupted by user clearing RxEN or setting Haltlmm */
/*       in the MAC control register. 					*/
/************************************************************************/

#define	RXFDST_OvMax	0x0008	// Over Maximum Size
#define	RXFDST_CtlRecd	0x0020	// set if packet received is a MAC control frame.
#define	RXFDST_IntRx	0x0040	// Interrupt on Receive 
#define RXFDST_Rx10Stat	0x0080	// set if packet was received via the 10bits interface reset if packet 
				// was received via MII
#define	RXFDST_AlignErr	0x0100	// Alignment Error
#define	RXFDST_CRCErr	0x0200	// CRC error
#define	RXFDST_Overflow	0x0400	// MAC receive FIFO was full when it 
				// needed to store a received byte
#define	RXFDST_LongErr	0x0800	// received a frame longer than 1518 bytes 
#define	RXFDST_RxPar	0x2000	// MAC receive FIFO has detected a parity error
#define RXFDST_RxGood	0x4000	// successfully received a packet with no errors
#define RXFDST_RxHalted	0x8000	// Reception interrupted by clearing RxEN

/************************************************************************/
/* 	Data structure of Tx Frame descriptor 				*/
/* 									*/
/* 	| 31|  30              16 | 15  |6  5| 4 | 3 | 2 | 1 | 0 |	*/
/* 	----------------------------------------------------------	*/
/* 	| O |             Frame Data Pointer                     |	*/
/* 	----------------------------------------------------------	*/
/* 	|         Reserved              | WA | A | L | T | C | P |	*/
/* 	----------------------------------------------------------	*/
/* 	| Tx Status               | Frame Length                 |	*/
/* 	----------------------------------------------------------	*/
/* 	| Next Frame              | Descriptor Pointer           |	*/
/* 	----------------------------------------------------------	*/
/* 									*/
/* 	[31] 	Ownership bit (O)					*/
/* 		0 = CPU 1 = BDMA					*/
/* 	[30:0]	Frame data pointer					*/
/* 		The address of the frame data to be transmitted.	*/
/* 	[0] 	No-padding mode (P)					*/
/* 		0 = Padding mode 	1 = No-padding mode		*/
/* 	[1] 	No-CRC mode (C)						*/
/* 		0 = CRC mode		1 = No-CRC mode 		*/
/* 	[2] 	MAC transmit interrupt enable after transmission	*/
/* 		of this frame (T) 					*/
/* 		0 = Disable 		1 = Enable			*/
/* 	[3] 	Little-Endian mode (L)					*/
/* 		0 = Big-endian 		1 = Little-endian		*/
/* 	[4] 	Frame data pointer increment/decrement (A)		*/
/* 		0 = Decrement 		1 = Increment			*/
/* 	[6:5] 	Widget alignment control (WA)				*/
/* 		(Non-aligned data must be transmitted without alignment */
/* 	 	 control.)						*/
/* 		00 = No invalid bytes 	01 = One invalid byte		*/
/* 		10 = Two invalid bytes	11 = Three invalid bytes	*/
/* 	[15:0]	Frame length						*/
/* 		The size of the transmit frame.				*/
/* 	[31:16] Tx status						*/
/* 		This Tx frame status field is updated by the MAC after 	*/
/* 		transmission.						*/
/* 	[31:0]	Next frame descriptor pointer				*/
/* 		The address of the next frame descriptor.		*/
/************************************************************************/

// Tx Frame Descriptor's control bits
#define	TXFDCTRL_PaddingMode	     0x00 // Padding mode 
#define	TXFDCTRL_NoPaddingMode	     0x01 // No-padding mode
#define	TXFDCTRL_NoCRCMode	     0x02 // No-CRC mode
#define	TXFDCTRL_CRCMode	     0x00 // CRC mode
#define	TXFDCTRL_MACTxIntEn	     0x04 // transmit interrupt enable after transmission
#define	TXFDCTRL_LittleEndian	     0x08 // Little-endian 
#define	TXFDCTRL_BigEndian	     0x00 // Big-endian 
#define	TXFDCTRL_SourceAddrDecrement 0x00 // Frame data pointer decrement
#define	TXFDCTRL_SourceAddrIncrement 0x10 // Frame data pointer increment
// 
#define	TXFDCTRL_WidgetAlign00	     0x00 // No Invalid bytes
#define	TXFDCTRL_WidgetAlign01	     0x20 // 1 Invalid byte
#define	TXFDCTRL_WidgetAlign10	     0x40 // 2 Invalid bytes
#define	TXFDCTRL_WidgetAlign11	     0x60 // 3 Invalid bytes

/************************************************************************/
/*[19:16]Transmit collision count (TxCollCnt)				*/
/* 3:0   Count of collisions during transmission of a single packet. 	*/
/* 	 After 16 collisions, TxColl is zero, and ExColl is set. 	*/
/*[20]   Excessive collision (ExColl)					*/
/*  4    16 collisions occured in the same packet. 			*/
/*[21] 5 Transmit deferred (TxDefer) 					*/
/*[22] 6 Paused 							*/
/*[23]   Interrupt on transmit (IntTx) 					*/
/*  7    Set if transmission of packet caused an interrupt condition. 	*/
/*       This includes the enable completion (EnComp), MACTXCON [14],   */
/*       if enabled. 							*/
/*[24]   Underrun (Under)						*/
/*  8    MAC transmit FIFO becomes empty during transmission.		*/
/*[25]   Deferral (Defer) 						*/
/*  9    MAC defers for max_deferral 0.32768 ms for 100 M bits/s or 	*/
/*       3.27680 ms for 10 M bits/s. 					*/
/*[26]   No carrier (NCarr) 						*/
/* 10	 Carrier sense is not detected during the entire transmission 	*/
/*       of a packet (from the SFD to the CRC). 			*/
/*[27]   SQE error (SQErr) 						*/
/* 11    After transmit frame, set if the fake collision (COL) signal 	*/
/*       did not come from the PHY for 1.6 ms. 				*/
/*[28]   Late collision (LateColl) 					*/
/* 12    A collision occurred after 512 bit times (64 byte times). 	*/
/*[29]   Transmit parity error (TxPar) 					*/
/* 13	 MAC transmit FIFO detected a parity error. 			*/
/*[30]   Completion (Comp) 						*/
/* 14    MAC completed a transmit or discard of one packet. 		*/
/*[31]   Transmission halted (TxHalted) 				*/
/* 15    Transmission halted by clearing TxEn or setting the Haltlmm 	*/
/*       in the MAC control register. Or, an interrupt was generated 	*/
/* 	 by an error condition (not completion). 			*/
/************************************************************************/

// Tx Frame descriptor's Status

#define	TXFDST_CollCnt  0x000F	// Transmit collision count
#define	TXFDST_ExColl	0x0010	// Excessive Collision
#define	TXFDST_TxDeffer	0x0020	// Transmit deffered 
#define TXFDST_Paused	0x0040	// Paused : holding data transmission DMA to MAC
#define	TXFDST_IntTx	0x0080	// Interrupt on Transmit 
#define	TXFDST_Under	0x0100	// Underrun */
#define	TXFDST_Defer	0x0200	// Mac defers for Max_DEFERRAL:=0.32768ms 
				// for 100Mbits/s, := 3.2768ms for 10Mbits/s
#define	TXFDST_NCarr	0x0400	// No Carrier sense is detected during the 
				// entire transmission of a packet from SFD 
				// to CRC
#define	TXFDST_SQErr	0x0800	// fake collision signal didn't come from 
				// PHY for 1.6us.
#define TXFDST_LateColl	0x1000	// Late collision 
#define	TXFDST_TxPar	0x2000	// Transmit Parity Error
#define	TXFDST_Comp	0x4000	// MAC transmit or discards one packet
#define TXFDST_TxHalted	0x8000	// Transmission was halted by clearing TxEn..

////////////////////////////////////////////////////////////////////////////////
// MAC Frame Structure /////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

typedef __packed struct ETH_HEADER {	
	U8 DestinationAddr[6];	// destination MAC address
	U8 SourceAddr[6];	// source MAC address	
	U8 LengthOrType[2];	// Length or Type
} etheader ;

typedef __packed struct MACFrame {	
	etheader 	Header ;
	U8 		LLCData[1506] ;
} sMACFrame;

////////////////////////////////////////////////////////////////////////////////
// MAC Tx Status Structure /////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

typedef struct MACTxStatus 
{
 U32 MacTxGood;
 U32 ExCollErr;		// Transmit collission count 
 U32 TxDefferedErr;	// Transmit deferred
 U32 sPaused;		// Paused
 U32 UnderErr;		// Underrun
 U32 DeferErr;		// Deferral
 U32 NCarrErr;		// No carrier
 U32 sSQE;		// Signal quality error
 U32 LateCollErr;	// Late collision
 U32 TxParErr;		// Transmit parity error
 U32 sTxHalted;		// Transmission halted
} 
 pMACTxStatus;


////////////////////////////////////////////////////////////////////////////////
// MAC Rx Status Structure /////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

typedef struct MACRxStatus 
{
 U32 OvMaxSize;
 U32 sCtlRecd;		// Control frame received
 U32 sRx10Stat;		// Receive 10-Mb/s status
 U32 AllgnErr;		// Alignment error
 U32 sCRCErr;		// CRC error
 U32 OverflowErr;	// Overflow error
 U32 sLongErr;		// Long error
 U32 RxParErr;		// Receive parity error
 U32 sRxHalted;		// Reception halted
} 
 pMACRxStatus;


////////////////////////////////////////////////////////////////////////////////
// BDMA Tx Status Structure ////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

typedef struct BDMATxStatus 
{
 U32 BTxNLErr;		// BDMA Tx null list (BTxNL)
 U32 BTxNOErr;		// BDMA Tx not owner (BTxNO)
 U32 BTxEmptyErr;	// BDMA Tx buffer empty (BTxEmpty)
} 
 pBDMATxStatus;


////////////////////////////////////////////////////////////////////////////////
// BDMA Rx Status Structure ////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

typedef struct BDMARxStatus 
{
 U32 BdmaRxCnt;
 U32 BdmaRxGood;
 U32 BRxNLErr;		// BDMA Rx null list (BRxNL)
 U32 BRxNOErr;		// BDMA Rx not owner (BRxNO)
 U32 BRxMSOErr;		// BDMA Rx maximum size over (BRxMSO)
} 
 pBDMARxStatus ;



/****************************************/
/*  PHY(LXT970A) Register Description   */
/****************************************/
/* PHY_CNTL_REG     : 0x00              */
/* PHY_STATUS_REG   : 0x01              */
/* PHY_ID_REG1      : 0x02              */
/* PHY_ID_REG2      : 0x03              */
/* PHY_ANA_REG      : 0x04              */
/* PHY_ANLPAR_REG   : 0x05              */
/* PHY_ANE_REG      : 0x06              */
/* PHY_ECNTL_REG1   : 0x10              */
/* PHY_INTEN_REG    : 0x11              */
/* PHY_INTSTAT_REG  : 0x12              */
/* PHY_ECNTL_REG2   : 0x13              */
/* PHY_CHIPSTAT_REG : 0x14              */
/****************************************/

////////////////////////////////////////////////////////////////////////////////
// [4:0] STACON // PHY register address (addr) /////////////////////////////////
//////////////////////////////////////////////////////////////////////////////// 

#define	PHY_CNTL_REG 	  0x00	/* Control register (Address 0) */
#define	PHY_STATUS_REG 	  0x01	/* Status register (Address 1) */
#define	PHY_ID_REG1    	  0x02	/* PHY identififcation Register 1 (Address 2) */
#define	PHY_ID_REG2    	  0x03	/* PHY identififcation Register 2 (Address 3)*/
#define	PHY_ANA_REG    	  0x04	/* Auto Negotiation Advertisement Register (Address 4) */
#define	PHY_ANLPAR_REG 	  0x05	/* Auto Negotiation Link Partner Ability Register (Address 5) */
#define	PHY_ANE_REG    	  0x06	/* Auto Negotiation Expansion (Address 6)*/
#define	PHY_ECNTL_REG1 	  0x10  /* Mirror Register (Address 16, Hex 10)*/
#define	PHY_INTEN_REG     0x11	/* Interrupt Enable Register (Address 17, Hex 11) */
#define	PHY_INTSTAT_REG   0x12	/* Interrupt Status Register */
#define	PHY_ECNTL_REG2 	  0x13	/* Extended control register */
#define	PHY_CHIPSTAT_REG  0x14	/* Chip Status Register */

// [9:5] STACON // PHY address. The 5-bit address of the PHY device ////////////
// to read or writen /////////////////////////////////////////////////////////// 	
#define PHYHWADDR         0x20	/* A0 = 1, A1,2,3,4 = 0 */

////////////////////////////////////////////////////////////////////////////////	
// PHY control register ////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////	

#define RESET_PHY	(1 << 15)	/* Reset (R/W,Self Clearing) */
// 1 =   Reset chip.
// 0 =   Enable normal operation.

#define ENABLE_LOOPBACK	(1 << 14)	/* Loopback mode (R/W) */
// 1 =   Enable loopback mode. When Loopback is enabled, during 100
// 	 Mbps operation, the LXT970A disconnects its transmitter and receiver
// 	 from the network. Data sent by the controller passes through the chip
// 	 and then gets looped back to the MII. During 10 Mbps operation the
// 	 preamble, SFD, and data loop directly back to the MII.
// 0 =   Disable loopback mode.
#define DR_100MB	(1 << 13)	/* Speed selection (R/W)*/
// 1 =   100 Mbps
// 0 =   10  Mbps
// 
// NOTE  If auto-negotiation is enabled, this bit is ignored. 
// 	 If auto-negotiation is disabled, the default value of bit 0.13 
//       is determined by pin CFG0.    

#define ENABLE_AN	(1 << 12)	/* Auto-Negotiatiation Enable (R/W)*/
// 0.12  Auto-Negotiation Enable
// 1 =   Enable auto-negotiate process (overrides speed select and 
// 	 duplex mode bits).
// 0 =   Disable auto-negotiate process.
// 
// NOTE  The default value of bit 0.12 is determined by pin MF0.

#define PHY_POWER_DOWN  (1 << 11)	/* Power down */
// 0.11  Power Down 
// 1 =   Enable power down.
// 0 =   Enable normal operation.
//
// NOTE  The LXT970A internally maintains all set values of the 
//       configuration registers upon exiting power-down mode.
//       A delay of 500 ns minimum is required from the time 
//       power down is cleared until any register can be written.

#define PHY_MAC_ISOLATE	(1 << 10)	/* Isolate */
// 0.10  Isolate 
// 1 =   Electrically isolate LXT970A from MII.
// 0 =   Normal operation.
// 
// NOTE  The default value of bit 0.10 is determined by pin TRSTE.
#define RESTART_AN	(1 << 9)	/* Restart Auto-negotiation (R/W, Self Cleared)*/
// 0.9   Restart Auto-Negotiation 
// 1 =   Restart auto-negotiation process.
// 0 =   Normal operation.
// 
// NOTE  If auto-negotiation is enabled, the default value of bit 0.9 
//       is determined by pin CFG0. If auto-negotiation is disabled, the
//       default value of bit 0.9 = 0.
#define PHY_FULLDUPLEX	(1 << 8)	/* Duplex mode */
// 0.8   Duplex Mode 
// 1 =   Enable full-duplex.
// 0 =   Enable half-duplex. 
// 
// NOTE  If auto-negotiation is enabled, this bit is ignored. 
//       If auto-negotiation is disabled, the default value of bit 
//       0.8 is determined by pin FDE.
#define PHY_COL_TEST	(1 << 7)	/* Colision Test */
// 0.7   Collision Test
// 1 =   Enable COL signal test. Bit 0.14 must be enabled to use this bit.
//       This bit is used in conjunction with bit 0.14 to test the COL output.
// 0 =   Disable COL signal test.
// 
// NOTE  This bit is ignored unless loopback is enabled (0.14 = 1).

// 0.6:4 Transceiver Test Mode Not Supported. (RO)
#define PHY_TX_TEST_MODE  0x70 		/* Transceiver Test Mode Not */

// 0.3   Master-Slave Enable. Not Supported. (RO)
#define PHY_MASTER_SL_EN   0x08		/* Master-Slave Enable. */

// 0.2   Master-SlaveValue. Not Supported. (RO)
#define PHY_MASTER_SL_VAL  0x02   	/* Master-SlaveValue. */

// 0.1:0 Reserved (R/W)

////////////////////////////////////////////////////////////////////////////////
// PHY Status register /////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

#define PHYSTAT_100BASET4      (1<<15)	/* 100Base-T4 */ 	 
// 1.15 100BASE-T4 Not Supported. Read Only. Default Value=0

#define PHYSTAT_100BASEX_FD    (1<<14)	/* 100Base-X full-duplex */ 
// 1.14 100BASE-X full-duplex. Read Only. Default Value=1
// 	1 = LXT970A able to perform full-duplex 100BASE-X.

#define PHYSTAT_100BASEX_HD    (1<<13)  /* 100BaseX half-duplex */	 
// 1.13 100BASE-X half-duplex. Read Only. Default Value=1 
//      1 = LXT970A able to perform half-duplex 100BASE-X.

#define PHYSTAT_10BASE_FD      (1<<12)	/* 10BaseT full-duplex */ 	 
// 1.12 10 Mb/s full-duplex. Read Only Default Value=1 
//      1 = LXT970A able to operate at 10 Mb/s in full-duplex mode.

#define PHYSTAT_10BASE_HD      (1<<11)	/* 10BaseT half-duplex */	 
// 1.11 10 Mb/s half-duplex. Read Only. Default Value=1 
//      1 = LXT970A able to operate at 10 Mb/s in half-duplex mode. 
					
#define PHYSTAT_100BASET2_FD   (1<<10)	/* 100BaseT2 full-duplex */ 	 
// 1.10 100BASE-T2 full-duplex. Not Supported. Read Only. Default Value=0

#define PHYSTAT_100BASET2_HD   (1<<9)	/* 100BaseT2 half-duplex */ 	 
// 1.9  100BASE-T2 half-duplex. Not Supported. Read Only. Default Value=0
// 1.8  Reserved Ignore on read. Read Only. Default Value=0

#define PHYSTAT_MASTERSL_FAULT (1<<7)	/* Master-Slave Fault */	 
// 1.7  Master-Slave Configuration Fault. Not Supported. RO,0

#define PHYSTAT_MFPREAMB_SUPPR (1<<6)	 
// 1.6  MF Preamble Suppression.  Read Only. Default Value = 0 
//      0 = LXT970A will not accept management frames with preamble
//      suppressed.

#define PHYSTAT_AN_COMPLETE    (1<<5)	/* Auto-Negotiation Complete */
// 1.5  Auto-Negotiation Complete. Read Only. Default Value=0
//      1 = Auto-negotiation process complete.
//      0 = Auto-negotiation process not complete. 

#define PHYSTAT_REMOTE_FAULT   (1<<4)	/* Remote fault */	 
// 1.4  Remote Fault. RO/Latching Hight (This bit remains High until read,
//      and then return Low).
//      1 = Remote fault condition detected.
//      0 = No remote fault condition detected.
// 
//      This bit is set when:
//      Link partner transmits a remote fault condition (bit 5.13 = 1)
//      Link partner protocol is anything other than CSMA-CD 
//      (bits 5.4:0 = <00001>)
//      Link partner advertises T4 capability only (bits 5.9:5  <10000>)

#define PHYSTAT_AUTO_NEG_AB   (1<<3)	/* Ato-Neg ability */	 
// 1.3  Auto-Neg. Ability. Read Only. Default Value = 1 
//      1 = LXT970A is able to perform auto-negotiation. 

#define PHYSTAT_LINK_UP       (1<<2)	/* Link Up Flag */
// 1.2  Link Status. Read Only/Latching Low (This bit remain Low until read,
//      and then return High). 
//      1 = Link is up.
//      0 = Link is down.  0

#define PHYSTAT_JUBBER_DET    (1<<1)	/* Jubber detect */	
// 1.1  Jabber Detect (10BASE-T Only)
// 	1 = Jabber condition detected.
//      0 = No jabber condition detected. RO/LH 0

#define PHYSTAT_EXT_CAP       (1<<0)	/* Extended capability */	 
// 1.0  Extended Capability. Read Only. Default Value = 1. 
//      1 = Extended register capabilities. 


////////////////////////////////////////////////////////////////////////////////
// PHY_ANA_REG // PHY Auto-negotiation Advertisement register //////////////////
////////////////////////////////////////////////////////////////////////////////

#define PHYANA_NEXTPAGE	      (1<<15)	/* Next Page */
// 4.15    Next Page. Not Supported RO 0
// 4.14    Reserved Ignore on read. RO 0

#define PHYANA_REMOTE_FAULT   (1<<13)	/* Remote Fault */
// 4.13    Remote Fault. R/W 0 
// 	   1 = Remote fault.
// 	   0 = No remote fault. 
// 4.12:11 Reserved Ignore on read. R/W 0
// 

#define PHYANA_PAUSE          (1<<10) 	/* Pause */
// 4.10    Pause. R/W 0 
// 	   1 = Pause operation is enabled for full-duplex links.
// 	   0 = Pause operation is disabled. 
// 

#define PHYANA_100BASE_T4     (1<<9)	/* 100BASE-T4 */
// 4.9     100BASE-T4. R/W 0
// 	   1 = 100BASE-T4 capability is available.
// 	   0 = 100BASE-T4 capability is not available.
// 	   (The LXT970A does not support 100BASE-T4, but allows this bit 
//         to be set to advertise in the Auto-Negotiation sequence 
//         for 100BASE-T4 operation. An external 100BASE-T4 transceiver 
//         could be switched in if this capability is desired.)
// 

#define PHYANA_DR100_TX_FULL	(1<<8)	/* 100BASE-TX full-duplex */
// 4.8 	   100BASE-TX full-duplex
// 	   1 = DTE is 100BASE-TX full-duplex capable.
// 	   0 = DTE is not 100BASE-TX full-duplex capable.

#define PHYANA_DR100_TX_HALF	(1<<7)	/* 100BASE-TX */
// 4.7     100BASE-TX R/W Note 3
//         1 = DTE is 100BASE-TX capable.
//         0 = DTE is not 100BASE-TX capable. 

#define PHYANA_DR10_TX_FULL	(1<<6)	/* 10BASE-T full-duplex */
// 4.6 	   10BASE-T full-duplex R/W Note 4
//         1 = DTE is 10BASE-T full-duplex capable.
//         0 = DTE is not 10BASE-T full-duplex capable. 

#define PHYANA_DR10_TX_HALF	(1<<5)	/* 10BASE-T */
// 4.5     10BASE-T 
//         1 = DTE is 10BASE-T capable.
//         0 = DTE is not 10BASE-T capable. 
// 4.4:0   Selector Field, S<4:0> R/W 00001
//         <00001> = IEEE 802.3
//         <00010> = IEEE 802.9 ISLAN-16T
//         <00000> = Reserved for future Auto-Negotiation development
//         <11111> = Reserved for future Auto- Negotiation development
//         Unspecified or reserved combinations should not be transmitted.


///////////////////////////////////////////////////////////////////////////////
// PHY_ANLPAR_REG // Auto Negotiation Link Partner Ability Register ///////////
///////////////////////////////////////////////////////////////////////////////

#define PHYANLPAR_NEXTP           (1<<15)
// 5.15	   Next Page. RO N/A
//         1 = Link Partner has ability to send multiple pages.
//         0 = Link Partner has no ability to send multiple pages. 

#define PHYANLPAR_ACK             (1<<14)     
// 5.14    Acknowledge RO N/A
//         1 = Link Partner has received Link Code Word from LXT970A.
//         0 = Link Partner has not received Link Code Word from LXT970A. 

#define PHYANLPAR_REMFAULT        (1<<13) 
// 5.13    Remote Fault RO N/A
//         1 = Remote fault.
//         0 = No remote fault. 
// 5.12:11 Reserved Ignore on read. RO N/A

#define PHYANLPAR_PAUSE           (1<<10)
// 5.10    Pause RO N/A
//         1 = Pause operation is enabled for link partner.
//         0 = Pause operation is disabled. 

#define PHYANLPAR_100BASET4        (1<<9) 
// 5.9     100BASE-T4 RO N/A
//         1 = Link Partner is 100BASE-T4 capable.
//         0 = Link Partner is not 100BASE-T4 capable. 

#define PHYANLPAR_100BASETX_FD      (1<<8)
// 5.8     100BASE-TX full-duplex RO N/A
//         1 = Link Partner is 100BASE-TX full-duplex capable.
//         0 = Link Partner is not 100BASE-TX full-duplex capable. 

#define PHYANLPAR_100BASETX_HD      (1<<7)
// 5.7     100BASE-TX RO N/A
//         1 = Link Partner is 100BASE-TX capable.
//         0 = Link Partner is not 100BASE-TX capable. 

#define PHYANLPAR_10BASET_FD        (1<<6)
// 5.6     10BASE-T full-duplex RO N/A
//         1 = Link Partner is 10BASE-T full-duplex capable.
//         0 = Link Partner is not 10BASE-T full-duplex capable. 

#define PHYANLPAR_10BASET_HD        (1<<5)
// 5.5     10BASE-T RO N/A
//         1 = Link Partner is 10BASE-T capable.
//         0 = Link Partner is not 10BASE-T capable. 

#define PHYANLPAR_SEL_FIELD          0x1F 
// 5.4:0   Selector Field S[4:0] RO N/A
//         <00001> = IEEE 802.3
//         <00010> = IEEE 802.9 ISLAN-16T
//         <00000> = Reserved for future Auto-Negotiation development
//         <11111> = Reser ved for fut ure Aut o-Negotiation development
//         Unspecified or reserved combinations shall not be transmitted.

///////////////////////////////////////////////////////////////////////////////
// PHY_ANE_REG // Auto Negotiation Expansion (Address 6) //////////////////////
///////////////////////////////////////////////////////////////////////////////

// 6.15:5  Reserved Ignore. RO 0
#define PHYANE_PAR_DET_FAULT        (1<<4)	
// 6.4 	   Parallel Detection Fault. RO/LH 0 
// 	   1 = Parallel detection fault has occurred.
// 	   0 = Parallel detection fault has not occurred.
#define PHYANE_LP_NEXT_PAGE         (1<<3)	/* Link Partner Next Page Support */
// 6.3 	   Link Partner Next Page Able  RO 0
// 	   1 = Link partner is next page able.
// 	   0 = Link partner is not next page able.
#define PHYANE_NEXT_PAGE            (1<<2)	/* Next Page Support */
// 6.2 	   Next Page Able Not Supported. RO 0
#define PHYANE_PAGE_REC             (1<<1)	/* Page Received */ 
// 6.1 	   Page Received RO/LH 0
// 	   1 = 3 identical and consecutive link code words have been 
// 	   received from link partner.
// 	   0 = 3 identical and consecutive link code words have not 
//         been received from link partner.
#define PHYANE_LP_AUTONEG           (1<<0)	/* Link Partner Auto Neg Ability */
// 6.0 	   Link Partner Auto Neg Able RO 0
// 	   1 = Link partner is auto-negotiation able.
// 	   0 = Link partner is not auto-negotiation able. 

///////////////////////////////////////////////////////////////////////////////
// PHY_ECNTL_REG1 // Mirror Register (Address 16, Hex 10) /////////////////////
///////////////////////////////////////////////////////////////////////////////

//16.15:0  User Defined This register is intended for use in checking 
//         the MII serial port and has no effect on chip operation. R/W 0

///////////////////////////////////////////////////////////////////////////////
// PHY_INTEN_REG // Interrupt Enable Register (Address 17, Hex 11) ////////////
///////////////////////////////////////////////////////////////////////////////


// 17.15:4 Reserved Write as 0; ignore on read. R/W N/A
#define PHYINTEN_MIIDRVLVL	   (1<<3)
// 17.3    MIIDRVLVL R/W 0
//         1 = Reduced MII driver levels. Pull-down strength of the 
//         MII driver is reduced by a factor of 10, and the pull-up 
//         strength is reduced by a factor of 8. 
//         Reduced driver levels on the MII I/O pins are recommended for
//         managed multi-port applications.
//         0 = High-strength MII driver levels that can effectively 
//         source 50 - 60 mA. Series termination resistors (55....)are 
//         recommended on all output signals when using this level to 
//         avoid undershoot or overshoot. 

#define PHYINTEN_LNK_CRITERIA	   (1<<2)
// 17.2    LNK CRITERIA R/W 0
//         1 = Enhanced link loss criteria. Link loss 
//             criteria is independent of symbol error rate. Loss of 
//             scrambler lock for more than 1 - 2 msec will brings the
//             link down. Link up criteria is based on symbol error rate.
//         0 = Standard link criteria. 
//             Both link up and link loss are based on symbol error rate.
// 

#define PHYINTEN_INTEN	   	   (1<<1)
// 17.1    INTEN R/W 0
//         1 = Enable interrupts. 
//             Must be enabled for bit 17.0 or 19.12 to be effective.
//         0 = Disable interrupts.

#define PHYINTEN_TINT	   	   (1<<0)
// 17.0    TINT R/W 0
//         1 = Forces MDINT Low and sets bit 18.15 = 1. Also forces 
//             interrupt pulse on MDIO when bit 19.12 = 1.
//         0 = Normal operation. This bit is ignored unless the 
//             interrupt function is enabled (17.1 = 1). 


////////////////////////////////////////////////////////////////////////////////
// PHY_INTSTAT_REG // Interrupt Status Register ////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

#define PHYINTSTAT_MINT            (1<<15)
// 18.15   MINT RO N/A
//         1 = Indicates MII interrupt pending.
//         0 = Indicates no MII interrupt pending. This bit is cleared by 
//         reading
//         Register 1 followed by reading Register 18.

#define PHYINTSTAT_XTALOK          (1<<14)
// 18.14   XTALOK RO 0
//         1 = Indicates that the LXT970A is fully powered up and the 
//             on-chip clocks are stable.
//         0 = Indicates that XTAL circuit is not stable.
// 18.13:0 Reserved Ignore RO 0

////////////////////////////////////////////////////////////////////////////////
// PHY_ECNTL_REG2 // Configuration Register (Address 19, Hex 13) ///////////////
////////////////////////////////////////////////////////////////////////////////


// 19.15   Reserved Write as 0; ignore on read. R/W N/A
#define PHYECNTL_100BASETX_TXMIT_TEST (1<<14)		/* Txmit Test */
// 19.14   Txmit Test (100BASE-TX) R/W 0
//         1 = 100BASE-T transmit test enabled, LXT970A transmits data
//         regardless of link status. This function is the analog of the 
//         link test function (19.8) for 100BASE-TX.
//         0 = Normal operation.

#define PHYECNTL_REPEATER_MODE      (1<<13)		/* Repeater Mode */
// 19.13   Repeater Mode  R/W
//         1 = Enable Repeater Mode.
//         0 = Enable DTE Mode.

#define PHYECNTL_MDIO_INT           (1<<12)	 	/* MDIO INT */
// 19.12   MDIO_INT R/W 0
//         1 = Enable interrupt signaling on MDIO (if 17.1 = 1).
//         0 = Normal operation (MDIO Interrupt disabled).
//         Bit is ignored unless the interrupt function is enabled (17.1 = 1).

#define PHYECNTL_10BASET_TPLOOPBACK (1<<11)		/* TP LoopBack */ 
// 19.11   TP Loopback (10BASE-T) R/W 0
//         1 = Disable 10BT TP Loopback. Data transmitted by the MAC will not
//         loopback to the RXD, RX_DV, and CRS pins.
//         0 = Enable 10BT TP Loopback - Preamble, SFD, and data are directly
//         looped back to the MII.

#define PHYECNTL_10BASET_SQE        (1<<10)		/* SQE */
// 19.10   SQE (10BASE-T)  R/W 0
//         1 = Enable SQE.
//         0 = Disable SQE (Default).

#define PHYECNTL_10BASET_JABBER     (1<< 9)		/* Jubber disable */
// 19.9    Jabber (10BASE-T) R/W 0
//         1 = Disable jabber.
//         0 = Normal operation (jabber enabled).

#define PHYECNTL_10BASET_LINK_TEST  (1<< 8)		/* Link Test (10BASE-T) */
// 19.8    Link Test (10BASE-T)  R/W 0
//         1 = Disable 10BASE-T link integrity test.
//         0 = Normal operation (10BASE-T link integrity test enabled). 

#define PHYECNTL_LEDC_MODE          (0x03<<6)	   /* LEDC Mode */
// 19.7:6  LEDC Programming bits Determine condition indicated by LEDC. R/W 0,0
//         bit 7 bit 6 Description
//          0     0    LEDC indicates collision
//          0     1    LEDC is off
//          1     0    LEDC indicates activity.
//          1     1    LEDC is continuously on (for diagnostic use).

#define PHYECNTL_LEDC_COL          (0x00<<6)		/* collision */
#define PHYECNTL_LEDC_OFF          (0x01<<6)		/* LEDC is off */
#define PHYECNTL_LEDC_ACT          (0x02<<6)		/* LEDC indicate activity */
#define PHYECNTL_LEDC_ON           (0x03<<6)		/* LEDC is on */

#define PHYECNTL_ADVTXCLOCK        (1<<5) 
// 19.5    Advance TX Clock R/W 0
//         1 = TX clock is advanced relative to TXD<4:0> and TX_ER by 1/2
//         TX_CLK cycle.
//         0 = Normal operation.

#define PHYECNTL_5BSYMBOL          (1<<4)		/* 5B Symbol */
// 19.4    5B Symbol/4B Nibble (100BASE-X only) R/W
//         1 = 5-bit Symbol Mode (Bypass encoder/decoder);
//             RXD<4:0> symbol data is not aligned.
//         0 = 4-bit Nibble Mode (Normal operation).

#define PHYECNTL_SCRAMBLER         (1<<3)		/* Scrambler */
// 19.3    Scrambler (100BASE-X only) R/W
//         1 = Bypass transmit scrambler and receive descrambler.
//         0 = Normal operation (scrambler and descrambler enabled).
//         In FX mode, the LXT970A automatically bypasses the Scrambler.
//         Selecting Scrambler bypass in FX mode causes the LXT970A to also
//         bypass the 4B/5B encoder and enable Symbol mode MII operation.

#define PHYECNTL_100BASEFX         (1<<2)		/* 100BASE-FX */
// 19.2    100BASE-FX R/W 
//         1 = Enable 100BASE fiber interface.
//         0 = Enable 100BASE twisted-pair interface. 

// 19.1    Reserved Write as 0; Ignore on read. R/W 0

#define PHYECNTL_TX_DISCONNECT     (1<<0)		/* Transmit Disconnect */
// 19.0    Transmit Disconnect  R/W 0
//         1 = Disconnect TP transmitter from line.
//         0 = Normal operation.

///////////////////////////////////////////////////////////////////////////////
// Chip Status Register (Address 20, Hex 14) //////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

// 20.15:1 4 Reserved Ignore on read. RO N/A

#define	PHYCHIPSTAT_LINKUP 	   (1<<13)	/* Link status */
// 20.13   Link RO 0
//         1 = Link is up.
//         0 = Link is down.
//         Link bit 20.13 is a duplicate of bit 1.2, except that it is a 
//         dynamic indication, whereas bit 1.2 latches Low.

#define	PHYCHIPSTAT_FULLDUPLEX     (1<<12)	/* Full Duplex Mode */
// 20.12   Duplex Mode RO 
//         1 = Full-duplex.
//         0 = Half-duplex. 

#define	PHYCHIPSTAT_100Mbps	   (1<<11)	/* 100 Mbps Mode */	
// 20.11   Speed  RO 
//         1 = 100 Mbps operation.
//         0 = 10 Mbps operation.
// 20.10   Reserved Ignore on read. RO N/A

#define	PHYCHIPSTAT_AUTONEG_COMP   (1<<9)	/* Auto-Negotiation Complete */
// 20.9    Auto-Negotiation Complete RO/LH 0
//         1 = Auto-negotiation process complete.
//         0 = Auto-negotiation process not complete.
//         Auto-Negotiation Complete bit 20.9 is a duplicate of bit 1.5.

#define	PHYCHIPSTAT_PAGE_REC       (1<<8)	/* Page Received */
// 20.8    Page Received RO/LH 0
//         1 = Three identical and consecutive link code words 
//             have been received.
//         0 = Three identical and consecutive link code words 
//             have not been received.
//         Page Received bit 20.8 is a duplicate of bit 6.1
// 20:7    Reserved Ignore on read. RO 0
// 20.6    Reserved Ignore on read. RO 0
// 20.5    Reserved Ignore on read. RO N/A
// 20.4    Reserved Ignore on read. RO N/A
// 20.3    Reserved Ignore on read. RO N/A

#define	PHYCHIPSTAT_LOW_VOLTAGE    (1<<2) 	/* Low-Voltage Fault */
// 20.2    Low-Voltage RO N/A
//         1 = Low-voltage fault on VCC has occurred.
//         0 = No fault. 
// 20.1    Reserved Ignore on read. RO N/A
// 20.0    Reserved Ignore on read. RO N/A

///////////////////////////////////////////////////////////////////////////////
// Initialize Ethernet and MAC controller /////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

extern void MacInitialize(void);	/* MAC Initialize */

///////////////////////////////////////////////////////////////////////////////
// Manage and initialize PHY //////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

extern void ResetPhyChip(void);
// MII Station write ////////////////////////////////////////////////////////// 
extern void MiiStationWrite(U32 PhyInAddr, U32 PhyAddr, U32 PhyWrData);
// MII Station read ///////////////////////////////////////////////////////////
extern U32  MiiStationRead(U32 PhyInAddr, U32 PhyAddr);


///////////////////////////////////////////////////////////////////////////////
// MAC and BDMA interrupt service routine /////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

extern void MAC_Tx_isr(void);			/* MAC  Tx interrupt Service Routine */
extern void MAC_Rx_isr(void);			/* MAC  Rx interrupt Service Routine */
extern void BDMA_Tx_isr(void);			/* BDMA Tx interrupt Service Routine */
extern void BDMA_Rx_isr(void);			/* BDMA Rx interrupt Service Routine */

/* Send Packet
 */
extern int MACSend(U8* data, int size);		/* Send Packet */
//  Size is length of data without ethernet header, but
//  data buffer (Data) must contain ethernet header and sended data
extern int MACRecv(U8 * data, U32 * status, int * size)  ;
extern int MACRxFlush(void);
extern int MACRxBD_count(void);
extern void MultiTransfer(void);		/* transmit multiple packets */
extern void ControlFrameTransfer(void);		/* transmit control frame */	
extern void SingleTransfer(void);		/* Singele Packet Transfer */


/*
 * Receive Packet
 */
extern void GetRxFrameData(U8 *pFrameData, U32 FrameLength, U32 RxStatus) ;
extern void RxMacFrame(void) ;			


///////////////////////////////////////////////////////////////////////////////
// Debug Information //////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

extern void MacDebugStatus(void);
extern void ReadErrReport(void);
extern void ClearErrReport(void);
	
/* Loopback test  */

// loopback test (internal & PHY & external, LOPBACK bit in MACCON not seted)
// aut = auto mode, no input from console 
extern int  InterruptLoopBackTest(int aut);	
// 

extern void PhyNoColLoopBack(int aut);		
extern void PhyColLoopBack(int aut);		
extern void MacInternalLoopBack(int aut);	
extern void MacAllLoopBack(void);		

////////////////////////////////////////////////////////////////////////////////
// Diagnostic test MAC and BDMA controller /////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

extern void MacTest(void);			/* main menu for mac tests */
extern void BdmaBufferTest(void);
extern void MacTxHaltReqTest(void); 		/* Tx HaltImm Request */
	
/* Utility */
extern int  PrintMACFrame(U8 * data,int len, int crcpres); /* Print MAC Frame */
extern void CaptureAllRxFrame(void);/*-*/		/* Capture All Frames */
extern void CaptureMyRxFrame(void);/*-*/
extern void CaptureBroadFrame(void);/*-*/
extern void CaptureCntlFrame(void);/*-*/
extern void CaptureMatchedDest(void);/*-*/
extern void ShowRecevedFrameData(void);/*-*/
	
#endif /* _MAC_ */
