#include "ks32c50.h"
#include "evm50100.h"
#include "mac.h"

/* 			*/
/* Modified by 		*/
/* Dmitriy Cherkashin 	*/
/* dch@ucrouter.ru	*/
/* 2002, 2003		*/
/*			*/

extern U32 AllTestResult ;		

U32 syscfg_pd_id;			// SYSCFG PD_ID field

// C Program Entry Point
void C_Entry(void)
{
#if 0
// UART0 Line Control Register ////////////////////////////////////////////////// 
 
 UARTLCON0 = 				// UART0 Line Control register 
  UARTLCON_WL8    |			// Word Length 8 bit 
  UARTLCON_PMD_NO;   			// No parity

// UART0 Control register /////////////////////////////////////////////////////// 

 UARTCONT0 =  				// UART0 Control Register
  UARTCON_RXM_INTREQ |			// rx mode = interrupt request
  UARTCON_TXM_INTREQ;      		// tx mode = interrupt request

// UART0 BaudRate Register ////////////////////////////////////////////////////// 

 UARTBRD0  = 
//  0x00a20; 	//  9600		// cnt0 = 162 cnt1 = 0, 0
//  0x00500; 	// 19200		// cnt0 =  80 cnt1 = 0, 1
    0x00280; 	// 38400		// cnt0 =  40 cnt1 = 0, 2
//  0x001a0; 	// 57600		// cnt0 =  26 cnt1 = 0, 3
//  0x000d0; 	// 115200		// cnt0 =  13 cnt1 = 0, 4
//  0x00060; 	// 230400		// cnt0 =   6 cnt1 = 0, 5
//  0x00020; 	// 460800   		// cnt0 =   2 cnt1 = 0, 6 not available 
#endif
 Main();
}

////////////////////////////////////////////////////////////////////////////////
// View System Configuration ///////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////


void ViewSysConfig(void)
{
 U8 * RO_BaseAddr = (U8 *) Image$$RO$$Base;	// Image Read Only Base
 U8 * RO_EndAddr  = (U8 *) Image$$RO$$Limit;	// Image Read Only Limit
 int  RO_Size = RO_EndAddr - RO_BaseAddr;	// Image Read Only Size

// 
 U8 * RW_BaseAddr = (U8 *) Image$$RW$$Base;	// Image Read/Write Base
 U8 * RW_EndAddr  = (U8 *) Image$$RW$$Limit;	// Image Read/Write Limit
 int  RW_Size = RW_EndAddr - RW_BaseAddr;	// Image Read/Write Size

// 
 U8 * ZI_BaseAddr = (U8 *) Image$$ZI$$Base;	// Image Zero Init Base
 U8 * ZI_EndAddr  = (U8 *) Image$$ZI$$Limit;	// Image Zero Init Limit
 int  ZI_Size = ZI_EndAddr - ZI_BaseAddr;	// Image Zero Init Size


 Print("\nSystem Configuration");

// Data Bus width for ROM bank 0 ///////////////////////////////////////////////

 Print("\nROM/Flash Bank 0 Data Bus Size is.");
 switch(EXTDBWTH & EXTDBWTH_DSR0)
  {
   case 1 : Print("BYTE");  break;	// ROM/Flash Bank 0 bus size = byte
   case 2 : Print("SHORT"); break;	// ROM/Flash Bank 0 bus size = half word
   case 3 : Print("WORD");  break;	// ROM/Flash Bank 0 bus size = word	
  }

// Stall Enable bit ////////////////////////////////////////////////////////////

 Print("\nSystem Manager Status") ;
 Print("\n  Stall Enabled (SE)..............");

 if((SYSCFG & SYSCFG_STALL) != 0)
  {
   Print("1(enabled)");
  } 
 else
  {
   Print("0(disabled)");
  } 


// Cacahe enable ///////////////////////////////////////////////////////////////

 Print("\n  Cache Enabled (CE)..............");
 if((SYSCFG & SYSCFG_CACHE) != 0)
  {
   Print("1(enabled)");
  } 
 else
  {
   Print("0(disabled)");
  } 


// Cache size //////////////////////////////////////////////////////////////////

 Print("\n  Cache Mode (CM).................");
 switch(SYSCFG & SYSCFG_CACHE_MODE)
  {
   case SYSCFG_CACHE_MODE_4KB_CACHE :
    Print("00 (4K SRAM, 4K CACHE)");
    break;

   case SYSCFG_CACHE_MODE_8KB_CACHE :
    Print("01 (0K SRAM, 8K CACHE)");
    break;

   case SYSCFG_CACHE_MODE_8KB_SRAM  :  
    Print("10 (8K SRAM, 0K CACHE)");
    break;

   default 			    :		
    Print("11 (unused)"); 
    break;
  }

/* Write buffer enable */

 Print("\n  Write Buffer Enabled (WE).......");
 if((SYSCFG & SYSCFG_WRITE_BUFF) != 0)
  {
   Print("1 (enabled)");
  }
 else
  {
   Print("0 (disabled)");
  } 

//
 Print("\n  Internal SRAM Base Address......0x%08x",((SYSCFG & 0xFFC0) << 10) );
 Print("\n  Special Register Bank Base Ptr..0x%08x",( SYSCFG & 0x3FF0000) );

 Print("\n  Product Identifier..............0x%x",   (SYSCFG >> 26) & 0x7);
 Print("\n  Sync. DRAM Mode.................%d"  ,   (SYSCFG >> 31));

//////////////////////////////////////////////////////////////////////////////// 
// content of CLCCON /////////////////////////////////////////////////////////// 
//////////////////////////////////////////////////////////////////////////////// 


 Print("\nClock control register CLKCON.....0x%08x", CLKCON);
 Print("\n  Clock Divided Value.............0x%08x",(CLKCON & CLKCON_DIV_MASK));

//
 Print("\n  ROM bank 5 wait.................");
 if((CLKCON & CLKCON_ROM5_WAIT_EN) != 0)
  {
   Print("1 (enable)");			// enable additional wait state
  }
 else
  {
   Print("0 (disable)");
  }

//////////////////////////////////////////////////////////////////////////////// 
//  
//////////////////////////////////////////////////////////////////////////////// 
//////////////////////////////////////////////////////////////////////////////// 

 Print("\n  ROM bank 5 address/data bus Mux.");
 if((CLKCON & CLKCON_ROM5_MUX_EN) != 0)
  {
   Print("1 (enable)");
  }
 else
  {
   Print("0 (disable)");
  }


 Print("\n  Mux bus Address Cycle (tAC).....");
 switch(CLKCON & CLKCON_ROM5_MUX_TAC) 
  {
   case CLKCON_ROM5_MUX_TAC_1   : Print("(00) 1 MCLK"); break;
   case CLKCON_ROM5_MUX_TAC_2   : Print("(01) 2 MCLK"); break;
   case CLKCON_ROM5_MUX_TAC_3   : Print("(02) 3 MCLK"); break;
   case CLKCON_ROM5_MUX_TAC_UN  : Print("(unused)");    break; 
   default 			: break;
  }


// 
 Print("\n  Test bit is set.................");
 if((CLKCON & CLKCON_TEST_BIT) != 0)
  {
   Print("1");
  }
 else
  {
   Print("0");
  }


/* Image ReadOnly Base, Limit, Size */ 
/* Print("\nRead Only Memory Used"); */
 Print("\n  Read Only Base..................0x%08x"   , RO_BaseAddr);
 Print("\n  Read Only End...................0x%08x"   , RO_EndAddr );
 Print("\n  Read Only Size..................0x%08x %d", RO_Size, RO_Size);

/* Image ReadWriet Base, Limit, Size */
/* Print("\nRead/Write Memory Used"); */
 Print("\n  Read/Write Base.................0x%08x"   , RW_BaseAddr);
 Print("\n  Read/Write End..................0x%08x"   , RW_EndAddr );
 Print("\n  Read/Write Size.................0x%08x %d", RW_Size, RW_Size);

/* Image ZI Base, Limit, Size */
/* Print("\nRead/Write Memory Used") ; */
 Print("\n  ZI Base.........................0x%08x"   , ZI_BaseAddr);
 Print("\n  ZI End..........................0x%08x"   , ZI_EndAddr );
 Print("\n  ZI Size.........................0x%08x %d", ZI_Size, ZI_Size);
}


////////////////////////////////////////////////////////////////////////////////
// Type definition for Error Report ////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

struct ALLERRRPT 
{
 int sram;				// 0 internal SRAM test 
 int mem;				// 1 memory test 
 int uart;				// 2 UART test 
 int timer;				// 3 timer test 
 int dma;				// 4 DMA test 
 int iic;				// 5 IIC test 
 int mac;				// 6 MAC test 	
 int hdlc;				// 7 HDLC test 
} 
 ALLERR[4];


////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////


void AllTests(void)
{
 U32 rbase [12];			// free mem region base 
 U32 rlimit[12];			// free mem region limit 
 int rcnt; 				// count of regions 
 U32 mbase;				// max free mem region base address
 int msize;				// max free mem region size 
 U32 * msrc;  			 	// memory test source address
 U32 * mdst;   				// memory test destination address
 U32 mtsize;  				// memory test size

 int testnum = 0;			// test number 
 int err = 0;				// error flag 
 TIME tm;				// time struct
 int  cm;				// cache mode

//
 rcnt = 12;
 if(FreeDramRegions(rbase,rlimit,&rcnt) == 0)
  {
   rcnt = 0;
  }

 msize = 0;				// max free mem region size 
 for(cm = 0; cm < rcnt; ++cm)
  {
   int size; 
   size = rlimit[cm] - rbase[cm];
   if(size > msize)
    {
     msize = size;
     mbase = rbase[cm];
    }
  } 

 Print("\nMax free DRAM region address=0x%08x size=%d", mbase, msize);


 if(msize > 0)
  {
// allign mem test region to 64K
   U32 base; 
   if(mbase == 0)
    {
     mbase  = 0x10000;
     msize -= 0x10000;
    }   

   base = ((mbase + 0xFFFF) >> 16) << 16;
   msize -= (base - mbase);		// reset size 
   mbase  = base;			// reset base 
   msrc = (U32 *) mbase;   		// memory source address
   mtsize = msize / 8;  		// memory Test size (words)
   mdst = msrc + mtsize;   		// memory destination address
  }


// TIMER 0 Initialized for mesuring the test time //////////////////////////////

 TimerReset(0);					// reset timer 0
 TimerInit (0,(ONE_SECOND/TICKS_PER_SECOND));	// init timer  0
 TimerStart(0);					// start timer 0

 for(cm = 0; cm < 4; ++cm)
  { 
// Clear Error Report //////////////////////////////////////////////////////////
   ALLERR[cm].sram  = 0;		// clear SRAM  test error flag 
   ALLERR[cm].mem   = 0; 		// clear MEM   test error flag 
   ALLERR[cm].uart  = 0;		// clear UART  test error flag 
   ALLERR[cm].timer = 0; 		// clear TIMER test error flag 	
   ALLERR[cm].dma   = 0;		// clear DMA   test error flag 
   ALLERR[cm].iic   = 0; 		// clear IIC   test error flag 
   ALLERR[cm].mac   = 0;		// clear MAC   test error flag 
   ALLERR[cm].hdlc  = 0; 		// clear HDLC  test error flag 
  }

////////////////////////////////////////////////////////////////////////////////
// Cache/Internal SRAM test ////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

 CacheInit(CACHEINIT_OFF);
 Print("\nCache OFF");
 Print("\n%d.Cache tag Set0 Set1 10N March Test .... ", testnum++);

// 
 Print("\n8K Cache ON");
 CacheInit(CACHEINIT_8K);
 if(	
   TagMarch10()  == 0 ||  		// test tag ram (256 words)
   Set0March10() == 0 ||     		// test cache set 0 in SRAM disable mode
   Set1March10() == 0        		// test cache set 1 in SRAM disable mode
  )
  {
   err = 1;
   ALLERR[0].sram = 1;
  }

// 4Kb Cache mode (4Kb SRAM) /////////////////////////////////////////////////// 
 Print("\n%d.Internal SRAM (4Kb mode) 10N March Test .... ", testnum++);
 Print("\n4K Cache ON");		// 4Kb Cache 
 CacheInit(CACHEINIT_4K);		// set cache mode 
 if(SramMarch10_4k() == 0)		// test SRAM in 4k mode
  {
   err = 1;
   ALLERR[1].sram = 1;
  }

// 8Kb SRAM mode /////////////////////////////////////////////////////////////// 
 Print("\n%d.Internal SRAM (8Kb mode) 10N March Test .... ", testnum++);
 Print("\nCache OFF");			// Cache Off
 CacheInit(CACHEINIT_OFF);		// Set cache Mode
 if(SramMarch10_8k() == 0)		// test SRAM in 8k mode	
  {
   err = 1;
   ALLERR[2].sram = 1;
  }

 for(cm = 0; cm < 3; ++cm) 
  {
   switch(cm)
    {
     case 0 :
// disable CACHE // (8Kb SRAM mode) //////////////////////////////////////////// 
      Print("\nCache OFF");		// Cache Off
      CacheInit(CACHEINIT_OFF);		// Set cache Mode
      break;

     case 1 :
// 4Kb Cache mode (4Kb SRAM) /////////////////////////////////////////////////// 
      Print("\n4K Cache ON");		// 4Kb Cache 
      CacheInit(CACHEINIT_4K);		// set cache mode 
      break;
 
     case 2 :
// 8Kb Cache mode (0Kb SRAM) /////////////////////////////////////////////////// 
      Print("\n8K Cache ON");
      CacheInit(CACHEINIT_8K);
      break;
    }      

   {
    int lsize = 1;   			// memory test loop size
    int rvl = 1;

// MEMORY Test //////////////////////////////////////////////////////////////////
    Print("\n%d. Memory Test .... ", testnum++);
    if(msize <= 0)
     {					// internal error 
      rvl = 0;
     }
    else
     {
      Print("\n   Source addr=0x%08x. Dest addr=0x%08x. Words=%d", msrc, mdst, mtsize);
      Print("\n   To skip test press ESC");
      rvl = MemTest(msrc,mdst,mtsize,lsize, 0x10000/4);
     }  

    if(rvl == 0) 
     {
      err = 1;
      ALLERR[cm].mem = 1;
      Print("\nFail.");
     }
    else 
    if(rvl == -1) 
     {
      ALLERR[cm].mem = -1;
      Print("\nSkipped.");
     }
    else 
     {
      Print("\nOk.");
     }
   }

// UART Test
   Print("\n%d. UART Test ....  ", testnum++);
   if(UARTAutoLoopBack(-1) == 0)
    {
     err = 1;
     ALLERR[cm].uart = 1;
     Print("\nFail.");
    }
   else 
    {
     Print("\nOk.");
    }

   Print("\n%d. GDMA Channel 0/1 Test .... ", testnum++);
   GdmaReset(0); 
   GdmaReset(1); 
   GdmaIntEnable(0);
   GdmaIntEnable(1);

   Print("\n   Source addr=0x%08x. Dest addr=0x%08x. Words=%d", msrc, mdst, mtsize);
   if(DmaAutoTest((U32)msrc,(U32)mdst,mtsize,1) == 0) 
    {
     err = 1;
     ALLERR[cm].dma = 1;
     Print("\nFail.");
    }
   else 
    {
     Print("\nOk.");
    }
   GdmaIntDisable(0);
   GdmaIntDisable(1);

////////////////////////////////////////////////////////////////////////////////
// MAC Test ////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

   Print("\n%d. MAC Test .... ", testnum++);
   Print("\n\nMAC - PHY LoopBackTest(Without Collision)");
   Print("\nSet PHY mode : Full-Duplex, 100Mb, Loop Back");
   ResetPhyChip();			// Reset the PHY chip and start Auto-negotiat
   MacInitialize();			// init MAC
   MiiStationWrite
    ( 
     PHY_CNTL_REG	, 		// Control register 
     PHYHWADDR		, 		// device address
     (
      PHY_FULLDUPLEX 	|		// Full-Duplex Mode 
      DR_100MB 	   	| 		// 100Mb
      ENABLE_LOOPBACK			// Loop Back mode 
     )
   );

// 
   if(InterruptLoopBackTest(1) == 0) 
    {
     err = 1;
     ALLERR[cm].mac = 1;
     Print("\nFail.");
    }
   else 
    {
     Print("\nOk.");
    }

   ResetPhyChip();

////////////////////////////////////////////////////////////////////////////////
// HDLC Test ///////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

   Print("\n%d. HDLC Test .... ", testnum++);
   if(
     HdlcInternalLoopback(0,1000,1) == 0 ||	// 
     HdlcInternalLoopback(1,1000,1) == 0	// 
    )
    {
     err = 1;
     ALLERR[cm].hdlc = 1;
     Print("\nFail.");
    }
   else 
    {
     Print("\nOk.");
    }

// IIC Test 

   Print("\n%d. IIC Test .... ", testnum++);
   if(IICWriteReadTest(1) == 0)
    {
     err = 1;
     ALLERR[cm].iic = 1;
     Print("\nFail.");
    }
   else  
    {
     Print("\nOk.");
    }

// 
    {
// 3 seconds time delay ////////////////////////////////////////////////////////
     U32 start;					// start time seconds 
     U32 sec;					// current time sec
     TIME tm;					// TIME struct 

     TimerReset(1);				// reset timer 1
     TimerInit(1,(ONE_SECOND/TICKS_PER_SECOND));// init timer 1
     TimerStart(1);				// start timer 1
     GetSysTime(1,&tm);				// get time 
     start = tm.tm_sec + tm.tm_min * 60;	// start time 
     while(1)
      {
       GetSysTime(1,&tm);			// read time from timer 1
       sec = tm.tm_sec + tm.tm_min * 60;	// time in seconds 
       if(sec > start + 3)
        {					// 
         break;
        } 
      }   
// 
     TimerStop(1);				// stop timer 1
    }
  }

// 
 Print("\n\nAll Test Finished"); 
 CacheInit(CACHEINIT_OFF);

 Print("\nAll Test Error Report");

// 
#define ALLERRFMT      "\n%-20s%12s%12s%12s"	// format 
#define ALLERR_OK         "Ok     "		// Ok message
#define ALLERR_FAIL      "Fail    "		// Fail message 
#define ALLERR_SKIPPED "Skipped   "		// 

// 
 Print(ALLERRFMT,
   "",
   "Cache OFF",
   "4K Cache ON",
   "8K Cache ON"
  );

 Print
  (
    ALLERRFMT,
   "Internal SRAM",
    ALLERR[0].sram  == 0 ? ALLERR_OK : ALLERR_FAIL,
    ALLERR[1].sram  == 0 ? ALLERR_OK : ALLERR_FAIL,
    ALLERR[2].sram  == 0 ? ALLERR_OK : ALLERR_FAIL
  );

 {
  char * m0;
  char * m1;
  char * m2;

  switch(ALLERR[0].mem)
   {
    case  0 : m0 = ALLERR_OK;      break;
    case  1 : m0 = ALLERR_FAIL;    break;
    case -1 : m0 = ALLERR_SKIPPED; break;
    default : m0 = ""; break;
   } 

  switch(ALLERR[1].mem)
   {
    case  0 : m1 = ALLERR_OK;      break;
    case  1 : m1 = ALLERR_FAIL;    break;
    case -1 : m1 = ALLERR_SKIPPED; break;
    default : m1 = ""; break;
   } 

  switch(ALLERR[2].mem)
   {
    case  0 : m2 = ALLERR_OK;      break;
    case  1 : m2 = ALLERR_FAIL;    break;
    case -1 : m2 = ALLERR_SKIPPED; break;
    default : m2 = ""; break;
   } 

  Print(ALLERRFMT,
   "Memory Test",
    m0	,
    m1	,
    m2
   );
 }
 
 Print(ALLERRFMT,
   "UART Test",
    ALLERR[0].uart == 0 ? ALLERR_OK : ALLERR_FAIL,
    ALLERR[1].uart == 0 ? ALLERR_OK : ALLERR_FAIL,
    ALLERR[2].uart == 0 ? ALLERR_OK : ALLERR_FAIL
  );

 Print(ALLERRFMT,
   "TIMER Test",
    ALLERR[0].timer== 0 ? ALLERR_OK : ALLERR_FAIL,
    ALLERR[1].timer== 0 ? ALLERR_OK : ALLERR_FAIL,
    ALLERR[2].timer== 0 ? ALLERR_OK : ALLERR_FAIL
  );

 Print(ALLERRFMT,
   "DMA Test",
    ALLERR[0].dma  == 0 ? ALLERR_OK : ALLERR_FAIL,
    ALLERR[1].dma  == 0 ? ALLERR_OK : ALLERR_FAIL,
    ALLERR[2].dma  == 0 ? ALLERR_OK : ALLERR_FAIL
  );

 Print(ALLERRFMT,
   "IIC Test",
    ALLERR[0].iic  == 0 ? ALLERR_OK : ALLERR_FAIL,
    ALLERR[1].iic  == 0 ? ALLERR_OK : ALLERR_FAIL,
    ALLERR[2].iic  == 0 ? ALLERR_OK : ALLERR_FAIL
  );

 Print(ALLERRFMT,
   "MAC Test",
    ALLERR[0].mac  == 0 ? ALLERR_OK : ALLERR_FAIL,
    ALLERR[1].mac  == 0 ? ALLERR_OK : ALLERR_FAIL,
    ALLERR[2].mac  == 0 ? ALLERR_OK : ALLERR_FAIL
  );

 Print(ALLERRFMT,
   "HDLC Test",
    ALLERR[0].hdlc == 0 ? ALLERR_OK : ALLERR_FAIL,
    ALLERR[1].hdlc == 0 ? ALLERR_OK : ALLERR_FAIL,
    ALLERR[2].hdlc == 0 ? ALLERR_OK : ALLERR_FAIL
  );


 TimerStop(0);

// 
 GetSysTime(0, & tm);			/* get time from Timer 0 */
 Print
  (
   "\ntime = %d:%d:%d.%d"	,
    tm.tm_hour		,		/* hour */
    tm.tm_min		,		/* minute */
    tm.tm_sec		,		/* second */
    tm.tm_ms				/* millisecond */
  );

 if(err != 0 ) 
  {	
   Print("\nFail");
  }
 else
  {
   Print("\nOk");
  }
}


////////////////////////////////////////////////////////////////////////////////
// main menu ///////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////


void Main(void)
{
//
 InitEvb();

//
 while(1)
  {
   char it;				// selected item

   Print("\n\n") ;
#ifdef _S3C4530_ 
   Print("\nS3C4530 TEST");
#else
   Print("\nKS32C50100 TEST");
#endif
   Print("\n[V] View Configuration [C] Cache/SRAM Test");
   Print("\n[M] Memory Test        [T] Timer Test");
   Print("\n[U] UART Test          [I] Interrupt Test");
   Print("\n[L] MAC/Ethernet Test  [H] HDLC Test");
   Print("\n[D] GDMA Test          [S] I2C Test");
   Print("\n[G] I/O Port Functions [A] All Test");
   Print("\n[P] Program Download");

// Select test item ////////////////////////////////////////////////////////////

   Print("\nSelect Test Item: ");
   it = get_upper();			// read character from console

   switch(it)
    {
     case 'V' : ViewSysConfig();break;	// Show System Config
     case 'C' : CacheTest();    break;	// cache test dialog
     case 'M' : MemoryTest();   break;	// memory test dialog
     case 'T' : TimerTest();    break;	// Timer 0/1, WatchDog test
     case 'U' : UartTest(); 	break;	// UART 0/1 test
     case 'I' : IntTest();      break;	// interrupt test 
     case 'L' : MacTest();      break;  // Ethernet test 
     case 'H' : HdlcTest();	break;	// HDLC test dialog 
     case 'D' : GdmaTest(); 	break;	// GDMA test dialog 
     case 'S' : IICTest(); 	break;	// IIC test dialog 
     case 'G' : IopTest();      break;	// I/O Port test
     case 'A' : AllTests();     break;  // All tests 
     case 'P' : ZmTest();       break;	// ZModem menu

     default  : Print("\nNo Test Item Selected"); 
 	break ;
    }

   Print("\nPress Any Key to Continue"); 
   get_byte();
  }
}


////////////////////////////////////////////////////////////////////////////////
// Initialize for Command Line Interface ///////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

void InitEvb(void) 
{
// SYSCFG PD_ID field
 syscfg_pd_id = (SYSCFG & SYSCFG_PD_ID);//
 InitInterrupt();			// Init interrupts
 UART_Initialize();  			// Initialize UART channel 0,1
 IICSetup();				//	
 MacInitialize();			// init MAC
 HDLC_Init();				//	
 SysSetInterrupt(nGDMA0_INT, GDMA0isr);	//
 SysSetInterrupt(nGDMA1_INT, GDMA1isr);	//
 CacheInit(CACHEINIT_OFF); 		// 
 Enable_Int(nGLOBAL_INT);		//
}


