/*************************************************************************/
/*                                                                       */
/* FILE NAME                                      VERSION                */
/*                                                                       */
/*      system.c                   KS32C5000, KS32C50100   : version 1.0 */
/*                                                                       */
/* COMPONENT                                                             */
/*                                                                       */
/*                                                                       */
/* DESCRIPTION                                                           */
/*                                                                       */
/*                                                                       */
/* AUTHOR                                                                */
/*                                                                       */
/*      Young Sun KIM, Samsung Electronics, Inc.                         */
/*                                                                       */
/* DATA STRUCTURES                                                       */
/*                                                                       */
/*                                                                       */
/* FUNCTIONS                                                             */
/*                                                                       */
/*            Cache & internal SRAM test module.                         */
/*                                                                       */
/* DEPENDENCIES                                                          */
/*                                                                       */
/*                                                                       */
/* HISTORY                                                               */
/*                                                                       */
/*         NAME            DATE                    REMARKS               */
/*                                                                       */
/*        in4maker      09-15-1998      Created initial version 1.0      */
/*                                                                       */
/*************************************************************************/

/* 			*/
/* Modified by 		*/
/* Dmitriy Cherkashin 	*/
/* dch@ucrouter.ru	*/
/* 2002			*/
/*			*/

#include "ks32c50.h"
#include "evm50100.h"


/************************************************************************/
/*  This function is parts of 10N march Algorithms			*/
/*  Memory  read[pattern] & write[~pattern] or 				*/
/*          read[~pattern] & write[~pattern] 				*/
/*  on increment or decrement address.					*/
/*									*/	
/*  MarchSub1(base or end address,word count, pattern,address direction)*/
/************************************************************************/


static int MarchSub1(unsigned int * addr, int unit, unsigned int pattern, int dir, int prog)
{
 unsigned int i;
 unsigned int rdata;			/* read data */
 int rvl = 1;				// return value

// 
 for(i = 0; i < unit; ++i)
  {
   if(prog != 0)
    {
     PrintRotSlash((i / prog) & 0x3);
    }
  
   rdata = *addr;     			/* read memory data */
   if(rdata != (~pattern)) 
    {
     Print("\nError [0x%08x] = 0x%08x. Must be 0x%08x", addr, rdata, ~pattern);
     rvl = 0; 				// clear return value
    }
  *addr = pattern;   			/* write data pattern */
   addr += dir;				// to net position 
  }

 return(rvl); 				
}


/************************************************************************/
/* Main 10N March Algorithm						*/
/* March10N32(base address, word size, pattern)				*/
/************************************************************************/


int March10N32(unsigned int *addr, int unit, unsigned int pattern, int prog)
{
 unsigned int *tempAddr;
 int rvl = 1; 				/* error flag for march test */
 int i;

//     
 tempAddr = addr;  
   
/* Write pattern on memory */

 for(i = 0; i < unit; i++)
  {
   *tempAddr++ = pattern;
// 
   if(prog != 0)
    {
     PrintRotSlash((i / prog) & 0x3);
    }
  }
   
 /* Sub Algorithm for Read & Write compare in memory 
  * _MarchSub1(end or base address,memory word size,pattern,addr dir); 
  *1 means increment, -1 is decrement address 
  */

/* test from start to end */

//  
 if(
   MarchSub1
    (
     addr	,			/* start address */
     unit	,			/* word count */
     ~pattern	,			/* xor pattern */
      1		,			/* from start fo end */
     prog 
    ) == 0				
  ) 
  {
   rvl = 0;				// clear return value
  }

//   
 if(
   MarchSub1
    (
     addr	,			/* start */
     unit	,			/* number of words */
     pattern    , 			/* pattern */
     1		,			/* from the start to end */
     prog 
    ) == 0
  ) 
  {
   rvl = 0;				// clear return value
  }

/* from end */

 if(
   MarchSub1
    (
     addr+unit-1,			/* end word address */
     unit	,			/* number of the words */
     ~pattern	,			/* */
     -1		,			/* post decrement */
     prog 
    ) == 0
  ) 
  {
   rvl = 0;				// clear return value
  }

//   
 if(
   MarchSub1
   (
    addr+unit-1	,			/* end adress item */
    unit	,			/* number of the words */
    pattern 	,			/* pattern */
    -1		,			/* post decrement ! */
    prog 
   ) == 0
  ) 
  {
   rvl = 0;      			// clear return value
  } 

 tempAddr = addr;

/* Read & test memory */

 for(i = 0; i < unit; ++i)
  {
   if(prog != 0)
    {
     PrintRotSlash((i / prog) & 0x3);
    }

   if(*tempAddr != pattern)
    {
     rvl = 0;
     Print("\nError [0x%08x] = 0x%08x. Must be 0x%08x",tempAddr,*tempAddr,pattern);
    }
   tempAddr++;
  }

 if(prog != 0)
  {
   put_byte(' ');
   put_byte('\b');
  }

// 
 return(rvl);  				/* return error code */	
}

////////////////////////////////////////////////////////////////////////////////
// print content of SYSCFG /////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////


void PrintCacheConfig(void)
{
 U32 reg[2];
 reg[0] = reg[1] = SYSCFG;
 PrintMBIT("SYSCFG");
 Print("0x%08x", SYSCFG);
// (SE) Stall enable bit ///////////////////////////////////////////////////////
 PrintEBIT(0, "Stall enable (SE)", reg, SYSCFG_STALL);
// (CE) Cache enable bit ///////////////////////////////////////////////////////
 PrintEBIT(0, "Cache enable (CE)", reg, SYSCFG_CACHE);
// (WE) Write enable bit ///////////////////////////////////////////////////////
 PrintEBIT(0, "Write enable (WE)", reg, SYSCFG_WRITE_BUFF);

#ifdef _S3C4530_
 if(is_s3c4530(syscfg_pd_id)) {	
    PrintEBIT(0, "Fixed priority (FP)", reg, SYSCFG_FP);
 }	
#endif								/* _S3C4530_ */

 {
// (CM) Cache mode /////////////////////////////////////////////////////////////
  ABIT cm_bt[] = {
    {"00(4Kb SRAM, 4Kb cache)", SYSCFG_CACHE_MODE_4KB_CACHE},
    {"01(0Kb SRAM, 8Kb cache)", SYSCFG_CACHE_MODE_8KB_CACHE},
    {"10(8Kb SRAM, 0Kb cache)", SYSCFG_CACHE_MODE_8KB_SRAM},
    { 0, 0}
  };
  PrintABIT(0, "Cache mode (CM)", reg, SYSCFG_CACHE_MODE, cm_bt);
 }

 {
// product ID ///////////////////////////////////////////////////////////////////
  char *pstr = "(unknown)"; 	
#ifdef _S3C4530_
// compiled with S3C4530 support
  switch(SYSCFG & SYSCFG_PD_ID)
   {
    case SYSCFG_PD_KS32C50100 : pstr = "S3C4510"; break;
    case SYSCFG_PD_S3C4510B   : pstr = "S3C4510B"; break;
    case SYSCFG_PD_S3C4530X   : pstr = "S3C4530X"; break;
    case SYSCFG_PD_S3C4530A   : pstr = "S3C4530A"; break;
    default : break;	
   }
#else
#ifdef _KS32C50100_
// compiled with KS32C50100 support
  switch(SYSCFG & SYSCFG_PD_ID)
   {
    case SYSCFG_PD_KS32C5000  : pstr = "KS32C5000"; break;
    case SYSCFG_PD_KS32C50100 : pstr = "KS32C50100"; break;
    default : break;	
   }
#endif
#endif								/* _S3C4530_ */
  PrintMBIT("Product Identifier");
  Print("%s", pstr);
 }
}


/************************************************************************/
/*  Cache Memory(256 word Tag RAM,8k-bytes SRAM 			*/
/*  Cache memory test use by 10N march algorithm			*/
/*  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~		*/
/*  - Cache memory address range					*/
/*     Tag  = 0x1100_0000~0x1100_0400 (256word)				*/
/*  									*/
/*     . at Cache Mode : 8kb SRAM, 0kb Cache				*/
/*        SRAM = 0x3fe0000~0x3fe02000  (8k bytes)			*/
/************************************************************************/


int TagMarch10(void)
{
 unsigned int *tag;			// Tag memory address 	
 U32 syscfg;				// saved syscfg
 int rvl = 1;  				// Tag memory error flag for march test int delay = 0;
	
 tag  = (unsigned int *) TagRAM;	// tag RAM base address

// Disable Cache,Disable write buffer off,clear stall enable bit ///////////////

 syscfg  = SYSCFG;			// save syscfg 
 SYSCFG &= 				// Clear Cache Configuration Bits
  ~(
    SYSCFG_WRITE_BUFF	|		// clear write buffer bit
    SYSCFG_CACHE	|		// clear cache enable bit
    SYSCFG_STALL			// clear stall enable bit
   );


 Print("\n\nTag(256words) Test by 10N march algorithm");
 PrintCacheConfig();			// print new cache mode

/* tag(256 words) test by 10N march */

 Print("\n");
 if(March10N32(tag,256,0x00000000,32) == 0) { rvl = 0; } // 
 if(March10N32(tag,256,0x0000ffff,32) == 0) { rvl = 0; } // 
 if(March10N32(tag,256,0x00ff00ff,32) == 0) { rvl = 0; } // 
 if(March10N32(tag,256,0x0f0f0f0f,32) == 0) { rvl = 0; } // 
 if(March10N32(tag,256,0x33333333,32) == 0) { rvl = 0; } // 
 if(March10N32(tag,256,0x55555555,32) == 0) { rvl = 0; } // 

 if(rvl == 0) 
  {
   Print("\nFail.\n");
  }
 else
  {
   Print("\nOk.\n");
  }

//
 CacheFlush();				// clear tag memory 
 SYSCFG = syscfg;			// restore SYSCFG  
 return(rvl);
}


/************************************************************************/
/* Set0, Set1 Cache memory clear function 				*/
/* clear Stall Enable, Write Buffer Enable, Cache enable bits, and set  */
/* cache mode bits to 01 (0K - SRAM, 8K - CACHE). Then cache sets 0,1	*/ 
/* are cleared. Tag RAM is not cleared.					*/
/************************************************************************/


int CacheClear(void)
{
 unsigned int * tagram; 		// pointer to tag RAM
 unsigned int * set0;			// pointer to set 0 RAM
 unsigned int * set1;			// pointer to set 1 RAM
 U32 syscfg; 				// saved syscfg
 int rvl;				// return code (1 = success, 0 = failure)
 int err;				// error flag
 int i;					// index

//
 rvl = 1;				// start return value

// 
 set0 = (unsigned int *) Set0CacheRAM;	// pointer to set0 RAM
 set1 = (unsigned int *) Set1CacheRAM;	// pointer to set1 RAM

// 
 err  = 0;				// no error

 Print("\nSET0(4k),SET1(4K) Cache memory clear Start");

 syscfg = SYSCFG;			// save SYSCFG
 SYSCFG = 
  (
// Clear CACHE Configuration bits in SYSCFG: ///////////////////////////////////
     SYSCFG & 			/* Stall Enable(SE), Cache Enable (CE) */	
    ~SYSCFG_CACHE_MASK		/* Write Buffer enable (WE), Cache mode (CM)*/
   )
  |
// and set CACHE/SRAM mode to : SRAM 0k, cache 8k //////////////////////////////
 SYSCFG_CACHE_MODE_8KB_CACHE;		
 									
// Clear Cache SET0 ram (4K) ///////////////////////////////////////////////////
 for(i=0; i < 1024; i++) 	/* 1024 words */	
  {
   * set0 = 0x00000000;   	/* clear set 0 RAM word */		
     set0++; 			/* increment pointer in set0 */
  }

// Check set0 RAM is cleared ///////////////////////////////////////////////////

 set0 = (unsigned int *)Set0CacheRAM;	/* restore pointer */

 for(i = 0; i < 1024; ++i) 	/* for set0 items */
  {
   if(*set0 != 0x00000000) 
    {												
// non zero => set error flag and print diagnostig message /////////////////////
     err = 1;
     Print("\nClear Cache SET0 ERROR. SET0[0x%x] = 0x%x",set0, *set0);
    }
   set0++; 
  }
    
 if(err) 
  {
   rvl = 0;
   Print("\nFail. SET0 Cache Memory Not Cleared.");
  }
 else 
  {
   Print("\nSET0 Cache Memory Cleared.");
  }   

									
// Clear SET1 RAM //////////////////////////////////////////////////////////////

 for(i = 0; i < 1024; ++i) 
  {
   * set1 = 0x00000000;			// clear set 1 word 
   ++set1; 				// increment set 1 pointer 
  }

// 
 err = 0;

// check set 1 is cleared //////////////////////////////////////////////////////				 

 set1 = (unsigned int *)Set1CacheRAM;	// restore set 1 pointer 

 for(i = 0; i < 1024; ++i) 		// check set 1 items (4Kb = 1K words)
  {
   if(*set1 != 0x00000000) 
    {
// item is nozero ////////////////////////////////////////////////////////////// 

     err = 1;
     Print("\nClear Cache SET1 ERROR. SET1[0x%x] = 0x%x",set1, *set1);
    }

   set1++; 				// increment set1 pointer 
  }
    
 if(err) 
  {
   rvl = 0;
   Print("\nFail. SET1 Cache Memory Not Cleared.");
  }
 else 
  {
   Print("\nSET1 Cache Memory Cleared.");
  }

//
 err = 0;
 tagram = (unsigned int *) TagRAM;
 for(i=0; i < 256; i++) 
  {
   *tagram = 0x00000000;   // Clear tag ram 
    tagram += 1; 
  }

// 
 tagram = (unsigned int *) TagRAM;
 for(i = 0; i < 256; ++i) 
  {
   if(*tagram != 0x00000000)
    {
     err = 1;
     Print("Clear Cache Tag Ram Error. [0x%x] = 0x%x.\n",tagram, *tagram);
    }
   tagram += 1; 
  }


 if(err) 
  {
   rvl = 0;
   Print("\nFail. Tag Ram Not Cleared.");
  }
 else 
  {
   Print("\nTag Ram Cleared.");
  }

//
 SYSCFG = syscfg;			// restore syscfg
 return(rvl);
}

////////////////////////////////////////////////////////////////////////////////
// test internal SRAM in 4K mode ///////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////


int SramMarch10_4k(void)
{
 unsigned int * sram;			// internal SRAM pointer 
 U32 syscfg;				// saved syscfg
 int err = 0;  				// sram flag for march error

//
 Print("\n\nSRAM(4K bytes) test by 10N march algorithm.");

//
 syscfg = SYSCFG;			// save SYSCFG
 CacheClear();    			// Clear the cache memory

// get pointer to internal SRAM from SYSCFG ////////////////////////////////////
 sram = (unsigned int *) 
  (
    ((SYSCFG >> 6) & 0x3FF) << 16	// 6 - Internal SRAM base pointer offset
  );					// field length - 10 bit

//[0]	Stall Enable (SE)
//[1]	Cache Enable (CE)
//[2] 	Write Buffer Enable (WE)
//[3]	0
//[5:4] Cache Mode (CM)
// 	00 = 4Kb SRAM, 4Kb Cache 
//	01 = 0Kb SRAM, 8Kb Cache 
//	10 = 8Kb SRAM, 0Kb Cache
//NOTE: When you write 10 to this field, the cache enable bit is cleared
// 	automatically
//[15:6]Internal SRAM base pointer     


// Set internal SRAM 4Kb mode //////////////////////////////////////////////////

 SYSCFG = 
  (
    SYSCFG & 				// 
   ~SYSCFG_CACHE_MASK			// 
  )
  | SYSCFG_CACHE_MODE_4KB_CACHE;
 PrintCacheConfig(); 			// print cache mode 

// SRAM(1024 words) test by 10N march //////////////////////////////////////////

 Print("\n");
 if(March10N32(sram,1024,0x00000000,256) == 0) { err = 1; } // 00000000
 if(March10N32(sram,1024,0x0000ffff,256) == 0) { err = 1; } // 0000FFFF
 if(March10N32(sram,1024,0x00ff00ff,256) == 0) { err = 1; } // 00FF00FF
 if(March10N32(sram,1024,0x0f0f0f0f,256) == 0) { err = 1; } // 0F0F0F0F	
 if(March10N32(sram,1024,0x33333333,256) == 0) { err = 1; } // 33333333 
 if(March10N32(sram,1024,0x55555555,256) == 0) { err = 1; } // 55555555

// 
 SYSCFG = syscfg; 

//
 if(err) 
  {
   Print("\nFail.\n");
   return(0);
  }
 else 
  {
   Print("\nOk.\n");
   return(1);
  }
}


////////////////////////////////////////////////////////////////////////////////
// test internal SRAM in 8Kb mode //////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////


int SramMarch10_8k(void)
{
 unsigned int * sram;			// internam SRAM pointer 
 U32 syscfg;				// saved SYSCFG
 int err = 0;  				// sram flag for march error

 Print("\n\nSRAM(8K bytes) test by 10N march algorithm.");

 syscfg = SYSCFG;			// save SYSCFG 
 CacheClear();    			// Clear the cache memory

 sram = (unsigned int *) (((SYSCFG >> 6) & 0x3FF) << 16);

//[0]	Stall Enable (SE)
//[1]	Cache Enable (CE)
//[2] 	Write Buffer Enable (WE)
//[3]	0
//[5:4] Cache Mode (CM)
// 	00 = 4Kb SRAM, 4Kb Cache 
//	01 = 0Kb SRAM, 8Kb Cache 
//	10 = 8Kb SRAM, 0Kb Cache
//NOTE: When you write 10 to this field, the cache enable bit is cleared
// 	automatically
//[15:6]Internal SRAM base pointer     
 

// 
 SYSCFG = 
  (
    SYSCFG & 				// clear cache bits
   ~SYSCFG_CACHE_MASK			// 
  ) |
 SYSCFG_CACHE_MODE_8KB_SRAM;		// set internal SRAM to 8Kb mode
 PrintCacheConfig();			// print cache configuration

/* SRAM(2048 words) test by 10N march */
 Print("\n");
 if(March10N32(sram,2048,0x00000000,256) == 0) { err = 1; }
 if(March10N32(sram,2048,0x0000ffff,256) == 0) { err = 1; }
 if(March10N32(sram,2048,0x00ff00ff,256) == 0) { err = 1; }
 if(March10N32(sram,2048,0x0f0f0f0f,256) == 0) { err = 1; }
 if(March10N32(sram,2048,0x33333333,256) == 0) { err = 1; }
 if(March10N32(sram,2048,0x55555555,256) == 0) { err = 1; } 

//
 SYSCFG = syscfg; 			// restore SYSCFG

// 
 if(err) 
  {
   Print("\nFail.\n");
   return(0);
  }
 else 
  {
   Print("\nOk.\n");
   return(1);
  }
}

////////////////////////////////////////////////////////////////////////////////
// test SET1 RAM (Cache in 8Kb mode) ///////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////


int Set1March10(void)
{
 unsigned int * set1;			// Cache SET1 pointer
 U32 syscfg;				// saved SYSCFG
 int err = 0;  				// set0 flag for march error

// 
 Print("\n\nCache[SET1]4K bytes tested by 10N march algorithm.\n");

// 
 syscfg = SYSCFG;			// save SYSCFG		
 CacheClear();    			// Clear the cache memory

// 
 set1 = (unsigned int *) Set1CacheRAM;	// cache set1 pointer 

 SYSCFG = 
  (
    SYSCFG & 				// clear cache bits 
   ~SYSCFG_CACHE_MASK			// 
  )|
 SYSCFG_CACHE_MODE_8KB_CACHE;		// 01 0Kb SRAM, 8Kb cache 
 PrintCacheConfig();			// print cache config 

//
 Print("\n");
 if(March10N32(set1,1024,0x00000000,256) == 0) { err = 1; }
 if(March10N32(set1,1024,0x0000ffff,256) == 0) { err = 1; }
 if(March10N32(set1,1024,0x00ff00ff,256) == 0) { err = 1; }
 if(March10N32(set1,1024,0x0f0f0f0f,256) == 0) { err = 1; }
 if(March10N32(set1,1024,0x33333333,256) == 0) { err = 1; }
 if(March10N32(set1,1024,0x55555555,256) == 0) { err = 1; }


// 
 CacheFlush();				// clear cache tag RAM 
 SYSCFG = syscfg;			//
 
 if(err) 
  {
   Print("\nFail.\n");
   return(0);
  }
 else 
  {
   Print("\nOk.\n");
   return(1);
  }
}

////////////////////////////////////////////////////////////////////////////////
// test cache set0 RAM (cache in 8Kb mode) /////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////


int Set0March10(void)
{
 unsigned int * set0;			// cache set0 pointer
 U32 syscfg;				// saved SYSCFG
 int err = 0; 				// error flag 

 Print("\n\nCache Set0,4K bytes,tested by 10N march algorithm.\n");

//
 syscfg = SYSCFG;			// save SYSCFG
 CacheClear();    			// Clear the cache memory

// 
 set0 = (unsigned int *) Set0CacheRAM;	// set0 pointer 

 SYSCFG = 
  (
    SYSCFG & 				// clear cache bits
   ~SYSCFG_CACHE_MASK			//
  )|
 SYSCFG_CACHE_MODE_8KB_CACHE;		// 8 Kb Cache 
 PrintCacheConfig();			// print cache configuration
 

/* SET0(1024 words) test by 10N march */

 Print("\n");
 if(March10N32(set0,1024,0x00000000,256) == 0) { err = 1; } //
 if(March10N32(set0,1024,0x0000ffff,256) == 0) { err = 1; } //
 if(March10N32(set0,1024,0x00ff00ff,256) == 0) { err = 1; } //
 if(March10N32(set0,1024,0x0f0f0f0f,256) == 0) { err = 1; } //
 if(March10N32(set0,1024,0x33333333,256) == 0) { err = 1; } //
 if(March10N32(set0,1024,0x55555555,256) == 0) { err = 1; } //


//
 CacheFlush();				// clear cache tag ram 
 SYSCFG = syscfg;			// restore SYSCFG

 if(err) 
  {
   Print("\nFail.\n");
   return(0);
  }
 else 
  {
   Print("\nOk.\n");
   return(1);
  }
}


////////////////////////////////////////////////////////////////////////////////
// Internal SRAM read & write, compare test  ///////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

static void InterSramTest(void)
{
 unsigned int * addr;			// pointer in internal SRAM
 unsigned int * base;  			// Internal SRAM Base Address
 U32 syscfg;				// saved SYSCFG
 int cnt;				// internal ram words count 
 int err;				// error flag 
 char ch;
 int i;

//
 syscfg = SYSCFG;

// 
 base = (unsigned int *) (((SYSCFG >> 6) & 0x3FF) << 16);  


//[0]	Stall Enable (SE)
//[1]	Cache Enable (CE)
//[2] 	Write Buffer Enable (WE)
//[3]	0
//[5:4] Cache Mode (CM)
// 	00 = 4Kb SRAM, 4Kb Cache 
//	01 = 0Kb SRAM, 8Kb Cache 
//	10 = 8Kb SRAM, 0Kb Cache
//NOTE: When you write 10 to this field, the cache enable bit is cleared
// 	automatically
//[15:6]Internal SRAM base pointer     


 Print("\n\nInternal SRAM test."); 
 Print("\n8K SRAM test? If NO,default 4K SRAM tested?[Y/N]"); 

 ch = get_upper();
          
 if(ch == 'Y') 
  {
//////////////////////////////////////////////////////////////////////////////// 
// set cache mode : SRAM - 8Kb , 0Kb Cache ///////////////////////////////////// 

   SYSCFG = 
    (
     SYSCFG & 				// clear cache bits
    ~SYSCFG_CACHE_MASK
    )|
   SYSCFG_CACHE_MODE_8KB_SRAM;		// 8Kbyte cache mode 

   cnt = 2048;				// 
   PrintCacheConfig();			// print cache config
   Print("\nSRAM Address:0x%08x~0x%08x",base, base+cnt); 
  }
 else 
  {
// set cache mode : SRAM - 4Kb, 4Kb cache //////////////////////////////////////  

   SYSCFG = 
    (
      SYSCFG & 				// clear cache mask
     ~SYSCFG_CACHE_MASK
    )|
   SYSCFG_CACHE_MODE_4KB_CACHE;		// set 4Kb cache mode 
  
   cnt = 1024;				// internal RAM size = 4Kb (1K words)
   PrintCacheConfig();			// print cache config		
   Print("\nSRAM Address:0x%08x~0x%08x",base,base+cnt); 
  }

 Print("\n");

// 
 addr = base;				// 

 for(i = 0; i < cnt; ++i) 
   {
    * addr = (unsigned int) addr;     	// write memory data
      addr++;				// increment address pointer 
   }

//
 err = 0;

 addr = base;

 for(i = 0; i < cnt; ++i) 
  {
   if(*addr != (unsigned int) addr)
    {
     err = 1;
     Print("\nError [0x%08x] == 0x%08x",addr,*addr);
    } 
   addr++;
  }

// 
 CacheFlush();				// clear cache tag ram 
 SYSCFG = syscfg; 			// restore syscfg

// 
 if(err == 1)
  {
   Print("\nFail.\n");
  }
 else
  {
   Print("\nOk.\n");
  }
}

////////////////////////////////////////////////////////////////////////////////
// Cache flush function for re-configuration cache mode ////////////////////////
////////////////////////////////////////////////////////////////////////////////

void CacheFlush(void)
{
 int i;
 unsigned int * tagram; 

 tagram = (unsigned int *)TagRAM;
 SYSCFG &= ~SYSCFG_CACHE;  		// Disable cache before cache flush

 for(i = 0; i < 256; ++i) 
  {
   *tagram = 0x00000000;   		// Clear tag ram 
    tagram += 1; 
  }
 Print("\nCache Tag Ram cleared.");
}


////////////////////////////////////////////////////////////////////////////////
// Setup System manager dialog for Cache & SRAM test ///////////////////////////
////////////////////////////////////////////////////////////////////////////////


static void CacheConfigure(void)
{
 unsigned int n;
 REG32  Reg;
 char ch;

 Print("\n\nCache Configurations."); 

////////////////////////////////////////////////////////////////////////////////
// Select Stall enable or disable //////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

 while(1) 
  {
   Print("\nStall Enable?         [Y/N]"); 
   ch = get_upper(); 
   if(
     ch == 'N' || 
     ch == 'Y'
    )
    {
     break; 
    }
  } 

 if(ch == 'Y') 
  {
   SYSCFG |=  SYSCFG_STALL;		// enable stall
  }
 else
  {
   SYSCFG &= ~SYSCFG_STALL;		// disable stall
  }  

////////////////////////////////////////////////////////////////////////////////
// Select write buffer enable/disable //////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

 while(1) 
  {
   Print("\nWrite buffer Enable?  [Y/N]"); 
   ch = get_upper(); 
   if(
     ch == 'N' || 
     ch == 'Y'
    )
    {
     break; 
    }
  }

 if(ch == 'Y') 
  {
   SYSCFG |=  SYSCFG_WRITE_BUFF; 	// enable write buffer
  }
 else 
  {
   SYSCFG &= ~SYSCFG_WRITE_BUFF; 	// disable write buffer
  }

////////////////////////////////////////////////////////////////////////////////
// Select Cache mode ///////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

 SYSCFG &= ~SYSCFG_CACHE;      		// Disable cache before cache mode change  
 Reg = (~SYSCFG_CACHE_MODE & SYSCFG);	// clear previous CM bit
 while(1) 
  {
   Print("\nCache Mode"); 
   Print("\n0. 4K-bytes SRAM, 4K-bytes Cache."); 
   Print("\n1. 0K-bytes SRAM, 8K-bytes Cache."); 
   Print("\n2. 8K-bytes SRAM, 0K-bytes Cache."); 
   Print("\nSelect number ");
   n = get_upper();

   if(n == '0')
    {
// 4K-bytes SRAM, 4K-bytes Cache
     Reg |= SYSCFG_CACHE_MODE_4KB_CACHE; 
     break;
    }
   else
   if(n == '1')
    {
// 0K-bytes SRAM, 8K-bytes Cache
     Reg |= SYSCFG_CACHE_MODE_8KB_CACHE; 
     break;
    }
   else 
   if(n == '2')
    {
// 8K-bytes SRAM, 0K-bytes Cache
     Reg |= SYSCFG_CACHE_MODE_8KB_SRAM; 
     break;
    }
   else
    {
     continue;
    }
  }

 SYSCFG = Reg;

////////////////////////////////////////////////////////////////////////////////
// Select Cache enable/disable /////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

 if( (SYSCFG & SYSCFG_CACHE_MODE) == SYSCFG_CACHE_MODE_8KB_SRAM)
  {
   Print("\nCache Disabled"); 
   ch = 'N';
  }
 else
  { 
   do 
    {
     Print("\nCache Enable?         [Y/N]"); 
     ch = get_upper(); 
    } 
   while((ch != 'N') && (ch != 'Y'));
  }

// 
 CacheFlush();   			// Tag ram clear before cache enable

 if(ch == 'Y') 
  {
   SYSCFG |=  SYSCFG_CACHE; 
  }
 else 
  {
   SYSCFG &= ~SYSCFG_CACHE; 
  }

// print cache configuration ///////////////////////////////////////////////////

 PrintCacheConfig();
}

////////////////////////////////////////////////////////////////////////////////
// CPU clock control function //////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

static void CpuClockConfigure(void)
{
 unsigned int divider;			// clock divider 
 unsigned int CpuClk; 			// CPU Clock  
 unsigned long RefCycle;  		// for refresh cycle
 unsigned long BuffReg; 

// 
 U32 refextcon;				// new content REFEXTCON

//
#ifndef EX_UCLK				// 
 U32 uartbrd0;				// new content UARTBRD0
 U32 uartbrd1;				// new content UARTBRD1
 U32 uartclk0; 				// new UART0 baudrate 
 U32 uartclk1; 				// new UART1 baudrate 
 U32 cnt0;				// min counter 0
 U32 cnt1;				// min counter 1 
 U32 un;				// UART number 
#endif					// 

//  
 Print("\n\nCPU Clock change.");
      
 while(1)
  {
   unsigned int tmpdiv;			// clock divider 
   char ch;				//
   int i;				//

// 
   Print("\nEnter Clock divider value 0x");
//argument value is the number of hexa digit.
   divider = get_number(16, 0);  	// read divider from console 

// 
   tmpdiv = divider & 0xFFFF;		// clear upper 16 bits
   for(i = 0; i < 16; ++i)		// find first non sero bit
    {
     if((tmpdiv & (1 << i)) != 0)
      {
       tmpdiv &= (1 << i);		// clear other bits
/*[15:0] Clock Divided Value 						*/
/* 	 If all bits are 0, a non-divided clock is used. Only one bit 	*/ 
/*	 can be set in CLKCON[15:0]. That is, the clock dividing 	*/
/* 	 values is defined as 1,2,4,8,16 ... 				*/ 
/* 	 Internal system clock, fMCLK = fICLK / (CLKCON + 1) 		*/ 
       break;
      }
    }

   if(divider != tmpdiv)
    {
// input value for divider is not equal to checked /////////////////////////////
     Print("\nNot correct input divider = 0x%x", divider);
     Print("\nOnly one bit can be set in CLKCON[15:0]");
     Print("\nUsed = 0x%x", tmpdiv); 
     divider = tmpdiv;
    }

/* Calculate CPU Clock from arqument divider value */
          
   CpuClk = fMCLK / (divider + 1); 

   Print
    (
     "\nCPU clock frequency is %d.%d Mhz.", 
        CpuClk / 1000000, 
       (CpuClk / 1000) % 1000
    ); 
// 
   Print("\nReset clkcon register?[Y/N]"); 
   ch = get_upper();
   if(ch == 'Y')
    {
     break;
    }  
   else
   if(ch == 'N')
    {
     return; 
    }
  } 

//
 Print("\nNow Clock changed CLKCON=0x%x",divider);

/******************************************************************************/ 
/* [3:0]        Baud rate divisor value CNT1 				      */
/*               xxx0 = divide by 1 					      */
/*               xxx1 = divide by 16 					      */
/* [15:4] 	Time constant value for CNT0 				      */
/* 									      */
/* BRGOUT = (MCLK2 or UCLK) / (CNT0 + 1) / (16^CNT1) / 16		      */
/******************************************************************************/ 

/* Change UART Board Rate */
#ifndef EX_UCLK
 for(un = 0; un < 2; ++un)		// for UART0/UART1 
  {
   U32 uartbrd;				// UART baud rate
   U32 brgmin;				// min baudrate difference 
   U32 brgout;				// temp baudrate
   U32 brgdif;				// temp baudrate difference
   U32 c0;				// temp counter 0  
   U32 c1;				// temp counter 1
   U32 c2;

// 
   brgmin = 0xFFFFFFFF;			// init baudrate difference
//   
   if(un == 0)				// UART0 =>
    {					// read UARTBRD0
     brgout = UARTBRD0;			// register 
    }
   else
    {					// UART1 => 
     brgout = UARTBRD1;			// read UARTBRD1
    } 					// register


//
   cnt0 = (brgout >> 4) & 0xFFF;	// init min counter 0 (12 bit)	
   cnt1 =  brgout & 0xF;		// init min counter 1 

//
   for(c0 = 0; c0 < 4096; ++c0)		// c0 loop
    {
     for(c2 = 0; c2 <= 1; ++c2)		// c1 loop
      {
       switch(c2)
        {
         case  0 : c1 =  1; break;	// divide by  1
         case  1 : c1 = 16; break; 	// divide by 16
         default : break;
        }


// calculate baud rate /////////////////////////////////////////////////////////

       brgout = (CpuClk / 2) / (c0 + 1) / c1 / 16;

//
       if(brgout >= 38400)
        {				// 
         brgdif = brgout - 38400;	// 
        }
       else
        {
         brgdif = 38400 - brgout;
        }     

       if(brgdif < brgmin)
        {
         brgmin = brgdif;		// set min brg difference 
         cnt0 = c0;			// set min cnt0 counter  
         cnt1 = c2;			// set min cnt1 counter 
        }
      }
    }

// UARTBRD0/UARTBRD1 register //////////////////////////////////////////////////
 
   uartbrd = ((cnt0 & 0xFFF) << 4) | cnt1; 

   if(un == 0)
    {					
     uartbrd0 = uartbrd;		// UARTBRG0	
    }
   else
    {
     uartbrd1 = uartbrd;		// UARTBRG1
    } 
  } 

// new UART0 baudrate //////////////////////////////////////////////////////////

 cnt0 = (uartbrd0 >> 4) & 0xFFF;		// 
 cnt1 =  uartbrd0 & 0xF;			// 
 if(cnt1 == 0)
  {
   cnt1 = 1;
  }
 else
  {
   cnt1 = 16;
  }
 uartclk0 = (CpuClk / 2) / (cnt0 + 1) / cnt1 / 16;	

// new UART1 baudrate //////////////////////////////////////////////////////////

 cnt0 = (uartbrd1 >> 4) & 0xFFF;		// 
 cnt1 =  uartbrd1 & 0xF;			// 
 if(cnt1 == 0)
  {
   cnt1 = 1;
  }
 else
  {
   cnt1 = 16;
  } 
 uartclk1 = (CpuClk / 2) / (cnt0 + 1) / cnt1 / 16;	
#endif

/* Change DRAM refresh cycle */

// Get refresh count value /////////////////////////////////////////////////////
 BuffReg   = (REFEXTCON & 0xffe00000) >> 21;   // Get original refresh cycle
 RefCycle  = (2048 - BuffReg + 1) / fMCLK;     // Calculate refresh cycle
 BuffReg   =  2048 + 1 - (RefCycle * fMCLK / (divider + 1));

//Set refresh cycle //////////////////////////////////////////////////////////// 
 refextcon = (REFEXTCON & ~0xffe00000) | (BuffReg << 21);   

#ifndef EX_UCLK
 Print("\nUart Board Rate & DRAM refresh cycle");
 Print("\nwere changed automaitcally.\n");
#else
 Print("\nDRAM refresh cycle changed automaitcally.\n");
#endif

#ifndef EX_UCLK
 Print("\nUARTBRD0=0x%08x <- 0x%08x [%d]", UARTBRD0, uartbrd0, uartclk0);
 Print("\nUARTBRD1=0x%08x <- 0x%08x [%d]", UARTBRD1, uartbrd1, uartclk1);
#endif
 Print("\nREFEXTCON=0x%08x 0x%08x",refextcon,REFEXTCON);

/* Write divider value to clkcon register for change CPU clock  */
// Set CPU clock will be fMCLK/(divider+1)
 CLKCON = (divider & 0xFFFF) | (CLKCON & 0xFFFF0000);
 {
  int i; 
  for(i = 0; i < Delay4ClkCange; ++i); 	// delay for clock stabilizations
 }

/* Change UART Board Rate */
#ifndef EX_UCLK
 UARTBRD0 = uartbrd0; 			// set UARTBRD0 register 
 UARTBRD1 = uartbrd1; 			// set UARTBRD1 register 
#endif

/* Change DRAM refresh cycle */
 REFEXTCON = refextcon;   
}

////////////////////////////////////////////////////////////////////////////////  
// Search Pattern in cache SET0/1, DRAM bank 0/1/2/3 ///////////////////////////  
////////////////////////////////////////////////////////////////////////////////  


static void SearchPattern(void)
{
 unsigned long int  pattern;		// pattern 
 unsigned long int *Set0Pointer;	// base pointer for cache SET0 
 unsigned long int *Set1Pointer;	// base pointer for cache SET1
 unsigned long int *DramBasePoint;	// DRAM bank base point
 unsigned long int *DramNextPoint;	// DRAM bank next point
 unsigned long int i;
 U8 ch;

// 
 Print("\n\nEntern Search Pattern. 0x"); 
 pattern = get_number(16,0);

// search in cache SET 0 /////////////////////////////////////////////////////// 

 Print("\nSearch in Cache Set0."); 
 Set0Pointer  = (unsigned long int *)(Set0CacheRAM);
 for(i = 0; i < 1024; ++i) 
  {
   if(pattern == Set0Pointer[i]) 
    {
     Print
      (
       "\nSet 0 : Address = %08x, Data = %08x\n",
	  Set0Pointer + i, 
	*(Set0Pointer + i)
      );
    }
  }

// search in cache SET 1 /////////////////////////////////////////////////////// 

 Print("\nSearch in Cache Set1."); 
 Set1Pointer  = (unsigned long int *)(Set1CacheRAM);
 for(i = 0; i < 1024; ++i) 
  {
   if(pattern == Set1Pointer[i]) 
    {
     Print("Set 1 : Address = %08x, Data = %08x\n",
	Set1Pointer + i, Set1Pointer[i]);
    }
  }

 if((EXTDBWTH & EXTDBWTH_DSD0) != 0)
  {
   Print("\nSearch in DRAM Bank 0 [Y/N]"); 
   ch = get_upper();
 
   if(ch == 'Y')
    {
// DRAM bank 0 enabled => search in DRAM bank 0 ////////////////////////////////

     DramBasePoint = (unsigned long int *) (((DRAMCON0 >> 10) & 0x3FF) << 16);
     DramNextPoint = (unsigned long int *) (((DRAMCON0 >> 20) & 0x3FF) << 16);

     i = 0;
     while(DramBasePoint < DramNextPoint)
      {
       if (pattern == *DramBasePoint) 
        {
         Print
          (
           "DRAM 0 : Address = %08x, Data = %08x\n",
            DramBasePoint 	, 
           *DramBasePoint
          );
        }
       DramBasePoint += 1;
 
       i++;
       if((i % 1024) == 0)
        {
         PrintRotSlash((i / 1024) & 0x3);
        } 
      }
    }
  }

 if((EXTDBWTH & EXTDBWTH_DSD1) != 0)
  {
// DRAM bank 1 is enabled //////////////////////////////////////////////////////

   Print("\nSearch in DRAM Bank 1 [Y/N]"); 
   ch = get_upper(); 
 
   if(ch == 'Y')
    { 
     DramBasePoint = (unsigned long int *) (((DRAMCON1 >> 10) & 0x3FF) << 16);
     DramNextPoint = (unsigned long int *) (((DRAMCON1 >> 20) & 0x3FF) << 16);

     i = 0;
     while(DramBasePoint < DramNextPoint)
      {
       if (pattern == *DramBasePoint) 
        {
         Print
          (
           "\nDRAM 1 : Address = %08x, Data = %08x\n",
            DramBasePoint 	, 
           *DramBasePoint
          );
        }
       DramBasePoint += 1;
       i++;
       if((i % 1024) == 0)
        {
         PrintRotSlash((i / 1024) & 0x3);
        } 
      }
    }
  }

 if((EXTDBWTH & EXTDBWTH_DSD2) != 0)
  {
// DRAM base bank 2 is enabled /////////////////////////////////////////////////

   Print("\nSearch in DRAM Bank 2 [Y/N]"); 
   ch = get_upper();
   
   if(ch == 'Y')
    {
     DramBasePoint = (unsigned long int *) (((DRAMCON2 >> 10) & 0x3FF) << 16);
     DramNextPoint = (unsigned long int *) (((DRAMCON2 >> 20) & 0x3FF) << 16);

     i = 0;
     while(DramBasePoint < DramNextPoint)
      {
       if(pattern == *DramBasePoint) 
        {
         Print
          (
           "\nDRAM 0 : Address = %08x, Data = %08x\n",
              DramBasePoint 	, 
             *DramBasePoint
          );
        }
       DramBasePoint += 1;
       i++;
       if((i % 1024) == 0)
        {
         PrintRotSlash((i / 1024) & 0x3);
        } 
      }
    }
  }

 if((EXTDBWTH & EXTDBWTH_DSD3) != 0)
  {
// DRAM base bank 3 is enabled /////////////////////////////////////////////////

   Print("\nSearch in DRAM Bank 3 [Y/N]"); 
   ch = get_upper();
   if(ch == 'Y')
    {
     DramBasePoint = (unsigned long int *) (((DRAMCON3 >> 10) & 0x3FF) << 16);
     DramNextPoint = (unsigned long int *) (((DRAMCON3 >> 20) & 0x3FF) << 16);

     i = 0;
     while(DramBasePoint < DramNextPoint)
      {
       if (pattern == *DramBasePoint) 
        {
         Print
          (
           "DRAM 0 : Address = %08x, Data = %08x\n",
            DramBasePoint 	, 
           *DramBasePoint
          );
        }
       DramBasePoint += 1;
       i++;
       if((i % 1024) == 0)
        {
         PrintRotSlash((i / 1024) & 0x3);
        } 
      }
    }
  }
}

/////////////////////////////////////////////////////////////////////////////////
// print CLKCON content /////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////

static void PrintCLKCON(void)
{
 U32 reg[2];
 reg[0] = reg[1] = CLKCON; 
 PrintMBIT("CLKCON (Clock control register)");
 Print("0x%08x", CLKCON);
// 
 PrintMBIT("15:0 Clock Divided Value");
 Print("%x", (CLKCON & CLKCON_DIV_MASK));
// 16 ROM bank 5 wait enable
 PrintSBIT(0, "16 ROM bank 5 wait enable",       reg,CLKCON_ROM5_WAIT_EN   ,"1","0");     
// 
 PrintEBIT(0, "17 ROM bank 5 address/data Mux en",reg,CLKCON_ROM5_MUX_EN);     
 {
  ABIT mx_bt[] = {
    {"00(1 MCLK)", CLKCON_ROM5_MUX_TAC_1  },
    {"01(2 MCLK)", CLKCON_ROM5_MUX_TAC_2  },
    {"10(3 MCLK)", CLKCON_ROM5_MUX_TAC_3  },
    {"11(unused)", CLKCON_ROM5_MUX_TAC_UN },
    { 0, 0} 
  };
 
  PrintABIT(0, "19:18 Mux bus Address Cycle (tAC)",reg,CLKCON_ROM5_MUX_TAC,mx_bt);
 }

#ifdef _S3C4530_
 if(is_s3c4530(syscfg_pd_id)) {
   PrintEBIT(0,"20 ROM bank 5 wait 1 cycle delay",reg,CLKCON_ROM5_WAIT_DELAY); 
// 
   PrintEBIT(0,"21 ROM bank 4 wait enable",       reg,CLKCON_ROM4_WAIT_EN   );
   PrintEBIT(0,"22 ROM bank 4 wait 1 cycle delay",reg,CLKCON_ROM4_WAIT_DELAY);  
// 
   PrintEBIT(0,"23 ROM bank 3 wait enable",       reg,CLKCON_ROM3_WAIT_EN   );  
   PrintEBIT(0,"24 ROM bank 3 wait 1 cycle delay",reg,CLKCON_ROM3_WAIT_DELAY);
// 
   PrintEBIT(0,"25 ROM bank 2 wait enable",       reg,CLKCON_ROM2_WAIT_EN   );
   PrintEBIT(0,"26 ROM bank 2 wait 1 cycle delay",reg,CLKCON_ROM2_WAIT_DELAY);
// 
   PrintEBIT(0,"27 ROM bank 1 wait enable",       reg,CLKCON_ROM1_WAIT_EN   );
   PrintEBIT(0,"28 ROM bank 1 wait 1 cycle delay",reg,CLKCON_ROM1_WAIT_DELAY);
// 
   PrintEBIT(0,"29 ROM bank 0 wait enable",       reg,CLKCON_ROM0_WAIT_EN   );
   PrintEBIT(0,"30 ROM bank 0 wait 1 cycle delay",reg,CLKCON_ROM0_WAIT_DELAY);
 }
#endif								/* _S3C4530_ */

 PrintSBIT(0, "Test bit", reg, CLKCON_TEST_BIT,"1", "0");
}


static void PrintEXTACON(void)
{
//
 PrintMBIT("EXTACON 0&1");
 Print("0x%08x 0x%08x", EXTACON0, EXTACON1);
 Print("\nChip selection set-up time on nOE (tCOS0, tCOS1, tCOS2, tCOS3)");
 PrintMBIT("0:2   EXTACON0 tCOS0");Print("%d",(EXTACON0>>EXTACON0_TCOS0_OFFSET)&7);
 PrintMBIT("0:2   EXTACON1 tCOS2");Print("%d",(EXTACON1>>EXTACON1_TCOS2_OFFSET)&7);
 PrintMBIT("18:16 EXTACON0 tCOS1");Print("%d",(EXTACON0>>EXTACON0_TCOS1_OFFSET)&7);
 PrintMBIT("18:16 EXTACON1 tCOS3");Print("%d",(EXTACON1>>EXTACON1_TCOS3_OFFSET)&7);
//
 Print("\nAddress set-up time before nECS (tACS0, tACS1, tACS2, tACS3)");
 PrintMBIT("5:3   EXTACON0 tACS0");Print("%d",(EXTACON0>>EXTACON0_TACS0_OFFSET)&7);
 PrintMBIT("5:3   EXTACON1 tACS2");Print("%d",(EXTACON1>>EXTACON1_TACS2_OFFSET)&7);
 PrintMBIT("21:19 EXTACON0 tACS1");Print("%d",(EXTACON0>>EXTACON0_TACS1_OFFSET)&7);
 PrintMBIT("21:19 EXTACON1 tACS3");Print("%d",(EXTACON1>>EXTACON1_TACS3_OFFSET)&7);
//
 Print("\nChip selection hold time on nOE (tCOH0,tCOH1,tCOH2,tCOH3)");
 PrintMBIT("8:6   EXTACON0 tCOH0");Print("%d",(EXTACON0>>EXTACON0_TCOH0_OFFSET)&7);
 PrintMBIT("8:6   EXTACON1 tCOH2");Print("%d",(EXTACON1>>EXTACON1_TCOH2_OFFSET)&7);
 PrintMBIT("24:22 EXTACON0 tCOH1");Print("%d",(EXTACON0>>EXTACON0_TCOH1_OFFSET)&7);
 PrintMBIT("24:22 EXTACON1 tCOH2");Print("%d",(EXTACON1>>EXTACON1_TCOH3_OFFSET)&7);
//
 Print("\nAccess cycles (nOE low time (tACC0,tACC1,tACC2,tACC3)");
 PrintMBIT("11:9  EXTACON0 tACC0");Print("%d",(EXTACON0>>EXTACON0_TACC0_OFFSET)&7);
 PrintMBIT("11:9  EXTACON1 tACC2");Print("%d",(EXTACON1>>EXTACON1_TACC2_OFFSET)&7);
 PrintMBIT("27:25 EXTACON0 tACC1");Print("%d",(EXTACON0>>EXTACON0_TACC1_OFFSET)&7);
 PrintMBIT("27:25 EXTACON1 tACC2");Print("%d",(EXTACON1>>EXTACON1_TACC3_OFFSET)&7);
}

// print external bus width register content //////////////////////////////////// 

static void PrintEXTDBWTH(void) 
{
 int wn;
 PrintMBIT("EXTDBWTH Data Bus Width Register");
 Print("0x%0x", EXTDBWTH);

 for(wn  = 0; wn < 14; ++wn) {  
   U32 reg[2];
   static ABIT wd_bt[] = {
    {"00(disable)"  , 0},
    {"01(byte)"     , 1},
    {"10(half word)", 2},
    {"11(word)"     , 3},
    { 0, 0}
   };

   static ABIT wd_off[] = {
    {"ROM/SRAM/FLASH bank 0", EXTDBWTH_DSR0_OFF},
    {"ROM/SRAM/FLASH bank 1", EXTDBWTH_DSR1_OFF},
    {"ROM/SRAM/FLASH bank 2", EXTDBWTH_DSR2_OFF},
    {"ROM/SRAM/FLASH bank 3", EXTDBWTH_DSR3_OFF},
    {"ROM/SRAM/FLASH bank 4", EXTDBWTH_DSR4_OFF},
    {"ROM/SRAM/FLASH bank 5", EXTDBWTH_DSR5_OFF },

    {"DRAM bank 0"          , EXTDBWTH_DSD0_OFF},
    {"DRAM bank 1"          , EXTDBWTH_DSD1_OFF},
    {"DRAM bank 2"          , EXTDBWTH_DSD2_OFF},
    {"DRAM bank 3"          , EXTDBWTH_DSD3_OFF},

    {"External I/O bank 0"  , EXTDBWTH_DSX0_OFF},
    {"External I/O bank 1"  , EXTDBWTH_DSX1_OFF},
    {"External I/O bank 2"  , EXTDBWTH_DSX2_OFF},
    {"External I/O bank 3"  , EXTDBWTH_DSX3_OFF},
    { 0, 0}  
   };

   reg[0] = (EXTDBWTH >> wd_off[wn].val) & 0x3;
   PrintABIT(0, wd_off[wn].str, reg, 0x03, wd_bt);  
  }
}

// print ROMCON content  

static void PrintROMCON(int ch)
{
 U32 reg[6]; 				// regs content 
 int cnt;				// reg count 
 int cn;				// reg number 

 reg[0] = ROMCON0; 			// bank 0 config reg 
 reg[1] = ROMCON1; 			// bank 1 conf reg 	
 reg[2] = ROMCON2; 			// bank 2 conf reg
 reg[3] = ROMCON3; 			// bank 3 conf reg 	
 reg[4] = ROMCON4; 			// bank 4 conf reg
 reg[5] = ROMCON5; 			// bank 5 conf reg

 if(ch >= 0) {
   cnt = ch + 1;
 } else {
   cnt = -ch;
 }

// print ROMCON content /////////////////////////////////////////////////////////
 Print("\nROMCON"); 
 if(ch >= 0 && ch < 6) {
  Print("%d 0x%08x", ch, reg[ch]);
 } else {
   for(cn = 0; cn < cnt; ++cn) {
    Print("%d=%08x", cn, reg[cn]);
   } 
 }

 { 
// page mode configuration //////////////////////////////////////////////////////
  PrintMBIT("1:0 Page mode configuration (PMC)");
  for(cn = 0; cn < cnt; ++cn) 
   {
    if((cn == ch || ch < 0) == 0) 
     {
      continue;
     } 
//
    switch(reg[cn] & ROMCON_PMC) 
     {
      case ROMCON_PMC_NORMAL : Print(" 00/N "); break;
      case ROMCON_PMC_4WPAGE : Print(" 01/4w"); break;
      case ROMCON_PMC_8WPAGE : Print(" 10/8w"); break;
      case ROMCON_PMC_16WPAGE: Print(" 11/16"); break;
      default :
       break;    
     }
   }
  Print("\n00=Normal ROM; 01=4-word page; 10=8-word page; 11=16-word page");
 }

 {
// [3:2] Page address access time (tPA) /////////////////////////////////////////
  PrintMBIT("3:2 Page address access time (tPA)");
  for(cn = 0; cn < cnt; ++cn) 
   {
    if((cn == ch || ch < 0) == 0)
     {
       continue;
     }
    switch(reg[cn] & ROMCON_TPA)
     {
      case ROMCON_TPA_5CLK: Print(" 00/5 "); break;
      case ROMCON_TPA_2CLK: Print(" 01/2 "); break;
      case ROMCON_TPA_3CLK: Print(" 10/3 "); break;
      case ROMCON_TPA_4CLK: Print(" 11/4 "); break;
      default:
       break;
     }
   }
  Print("\n00=5 CLK;01=2 CLK;10=3 CLK; 11=4 CLK");
 }

 {
// [6:4]   Programmable access cycle (tACC) /////////////////////////////////////
  PrintMBIT("6:4 Programmable access cycle (tACC)");
  
  for(cn = 0; cn < cnt; ++cn)
   {
    if((cn == ch || ch < 0) == 0)
     {
      continue;
     }

    switch(reg[cn] & ROMCON_TACC)
     {
      case ROMCON_TACC_DISABLE : Print(" 000/D"); break;
      case ROMCON_TACC_2CLK    : Print(" 001/2"); break;
      case ROMCON_TACC_3CLK    : Print(" 010/3"); break;
      case ROMCON_TACC_4CLK    : Print(" 011/4"); break;
      case ROMCON_TACC_5CLK    : Print(" 100/5"); break;
      case ROMCON_TACC_6CLK    : Print(" 101/6"); break;
      case ROMCON_TACC_7CLK    : Print(" 110/7"); break;
      case ROMCON_TACC_UNUSED  : Print(" 111/R"); break;
      default:
       break;
     }
   }

   Print("\n000=Disable  ;001=2 cycles;010=3 cycles;011=4 cycles)");
   Print("\n100=5 cycles);101=6 cycles;110=7 cycles;111=Reserved)");
  }

//
 PrintMBIT("19:10  base pointer");
 for(cn = 0; cn < cnt; ++cn) {
   if(cn == ch || ch < 0) {
     Print("  %04x", ((reg[cn] & ROMCON_BASE_MASK) >> ROMCON_BASE_OFFSET));
   }
 }
 
//
 PrintMBIT("29:20  next pointer");
 for(cn = 0; cn < cnt; ++cn) {
   if(cn == ch || ch < 0) {
     Print("  %04x", ((reg[cn] & ROMCON_NEXT_MASK) >> ROMCON_NEXT_OFFSET));
   }
 }
}

// print DRAMCON /////////////////////////////////////////////////////////////////

static void PrintDRAMCON(int ch)
{
 U32 reg[4];				// DRAMCON regs 
 int cn, cnt, len;			// bank number, count, field length
 reg[0] = DRAMCON0; 			// DRAM bank 0 control reg 
 reg[1] = DRAMCON1; 			// DRAM bank 1 control reg 
 reg[2] = DRAMCON2; 			// DRAM bank 2 control reg 
 reg[3] = DRAMCON3; 			// DRAM bank 3 control reg 

 if(ch >= 0) 
  {
   cnt = ch + 1; 
   len = 15; 
  } 
 else 
  {
   cnt = -ch; 
   len = 40 / cnt; 
   if(len > 15) { len = 15; }
  } 

 Print("\nDRAMCON");
 for(cn = 0; cn < cnt; ++cn)
  {
   if(cn == ch || ch < 0) 
    {
     Print(" 0x%08x", reg[cn]);
    }
  }

 {
// [0] EDOMO mode
  PrintSBIT(ch, "0 EDO mode", reg, DRAMCON_EDO, "1/EDO", "0/norm");
 }
 
 {
// [2:1] CAS strobe time (tCS)
  ABIT tcs_bt[] = {
    {"00/1CLK", DRAMCON_TCS_1CLK},
    {"01/2CLK", DRAMCON_TCS_2CLK},
    {"10/3CLK", DRAMCON_TCS_3CLK},
    {"11/4CLK", DRAMCON_TCS_4CLK},
    {0, 0}
  };

  PrintABIT(ch, "2:1 CAS strobe time (tCS)", reg, DRAMCON_TCS, tcs_bt);
 }

  PrintSBIT(ch, "3:3 CAS precharge time (tCP)", reg, DRAMCON_TCP, 
   "1=2CLK", "0=1CLK");

  PrintSBIT(ch, "7 RAS to CAS delay(tRC or tRCD)", reg, DRAMCON_TRC, 
   "1=2CLK", "0=1CLK");

 {
  ABIT trp_bt[] = {
    {"00=1CLK", DRAMCON_TRP_1CLK},
    {"01=2CLK", DRAMCON_TRP_2CLK},
    {"10=3CLK", DRAMCON_TRP_3CLK},
    {"11=4CLK", DRAMCON_TRP_4CLK},
  };

  PrintABIT(ch, "9:8 RAS pre-charge time (tRP)", reg, DRAMCON_TRP, trp_bt); 
 }

 {
// 19:10 DRAM bank base pointer /////////////////////////////////////////////////
  int cn;
  PrintMBIT("19:10 DRAM bank base pointer ");
  for(cn = 0; cn < cnt; ++cn) 
   {
    if(cn == ch || ch < 0) 
     {
      Print("%-*x", len, (reg[cn] >> 10) & 0x3FF );
     }
   }
 }
 
 {
// 29:20 DRAM bank next pointer /////////////////////////////////////////////////
  int cn;
  PrintMBIT("29:20 DRAM bank next pointer ");
  for(cn = 0; cn < cnt; ++cn) 
   {
    if(cn == ch || ch < 0) 
     {
      Print("%-*x", len, (reg[cn] >> 20) & 0x3FF );
     }
   }
 }

 {
// [31:30]  Number of column address bits in DRAM bank # (CAN)
  ABIT can_bt[] = {
    {"00= 8bits", DRAMCON_CAN_8BITS  },
    {"01= 9bits", DRAMCON_CAN_9BITS  },
    {"10=10bits", DRAMCON_CAN_10BITS },
    {"11=11bits", DRAMCON_CAN_11BITS },
    { 0, 0}
  };  

  PrintABIT(ch, "31:30 Number of column bits (CAN)", reg, DRAMCON_CAN, can_bt);
 }
}

// print DRAM Refresh and External I/O Control register (REFEXRCON) /////////////

static void PrintREFEXTCON(void)
{
 U32 reg[1];
 reg[0] = REFEXTCON; 

 Print("\nREFEXTCON=0x%08x", reg[0]);
 PrintMBIT("9:0 External I/O bank 0 base pointer");
 Print("%04x", (REFEXTCON & 0x3FF) );

 PrintUBIT(0,"15 Validity of special reg field(VSF)", reg, REFEXTCON_VSF);
 PrintUBIT(0,"16 Refresh enable (REN)", reg, REFEXTCON_REN);

 {
// [19:17] CAS hold time(tCHR), ROW Cycle Time in SDRAM mode
  ABIT trc_bt[] = {
    {"000=1 cycle",  REFEXTCON_TRC_1CLK},
    {"001=2 cycles", REFEXTCON_TRC_2CLK},
    {"010=3 cycles", REFEXTCON_TRC_3CLK},
    {"011=4 cycles", REFEXTCON_TRC_4CLK},
    {"100=5 cycles", REFEXTCON_TRC_5CLK},
    {"101=6 cycles", REFEXTCON_TRC_6CLK},
    { 0, 0 }
  };

  PrintABIT(0, "19:17 ROW Cycle Time", reg, REFEXTCON_TRC, trc_bt);
 }

// [20] CAS setup time(t CSR) (note2)
 PrintSBIT(0,"20 CAS setup time(t CSR)", reg, REFEXTCON_TCSR,
   "1=2 cycles", "0=1 cycle");

 PrintMBIT("31:21 Refresh count value (duration)");
 Print("%d", ((reg[0] >> 21) * 0x7FF));
}


static void SysRegsView(void)
{
 while(1) {
   U8 it;
   Print("\n[S] SYSCFG - System Configuration Register.");
   Print("\n[C] CLKCON - Clock Control Register.");
   Print("\n[E] EXTACON 0&1 - External I/O access timing register 0,1.");
   Print("\n[W] EXTDBWTH - Data bus width of each bank.");
   Print("\n[R] ROMCON0-5  - ROM/SRAM/FLASH control register 0,1,2,3,4,5 .");
   Print("\n[D] DRAMCON0-3 - DRAM control registers 0,1,2,3.");
   Print("\n[F] REFEXTCON");
   Print("\n[Q] Quit.");

   Print("\nSelect Test Item: ");
   it = get_upper();
   switch(it) {
     case 'S' : PrintCacheConfig(); break;
     case 'C' : PrintCLKCON(); break;
     case 'E' : PrintEXTACON(); break;
     case 'W' : PrintEXTDBWTH(); break;
     case 'R' : PrintROMCON(-6); break;
     case 'D' : PrintDRAMCON(-4); break;
     case 'F' : PrintREFEXTCON(); break;
// 
     case 'Q' : 				// exit from menu 
      return;
     default  : break;
   }
   Print("\nPress any key to continue.\n");
   it = get_upper();
   if(it == 'Q') {
     break;
   } 
 }
}


/************************************************************************/
/*           CACHE & INTERNAL STRAM TEST TOP MODULE                     */
/************************************************************************/

void CacheTest(void)
{
 char it;

 while(1) 
  {
   Print("\nKS32C50100 CACHE & SRAM TEST");
   Print("\n[1] Tag RAM(256words) 10N march test.");	// 
   Print("\n[2] Internal SRAM(4kbytes) 10N march test.");
   Print("\n[3] Internal SRAM(8kbytes) 10N march test.");
   Print("\n[4] Cache Set0(4kbytes) 10N march test.");
   Print("\n[5] Cache Set1(4kbytes) 10N march test.");
   Print("\n[6] Internal SRAM R/W test at 4k or 8k mode.");
   Print("\n    ->SRAM address used as data pattern.");
   Print("\n[7] Cache start up dialog for setup cache mode.");
   Print("\n[8] Cache configuration.");
   Print("\n[9] Change CPU Clock use by clkcon register.");
   Print("\n[C] Cache Memory(SET0[4k]/SET1[4k]) Clear function.");
   Print("\n[B] Pattern search in cache memory region.");
   Print("\n[V] System registers (SYSCFG, CLKCON, ROMCON...).");
   Print("\n[A] All 10N March Test.");
   Print("\n[Q] QUIT - Return main menu.");

   Print("\nSelect Test Item: ");
   it = get_upper();

   switch(it) 
    {
     case '1' : TagMarch10();         break; 	// test tag ram (256 words)
     case '2' : SramMarch10_4k();     break;	// test SRAM in 4k mode
     case '3' : SramMarch10_8k();     break;	// test SRAM in 8k mode	
     case '4' : Set0March10();        break;	// test cache set 0 in SRAM disable mode
     case '5' : Set1March10();        break;	// test cache set 1 in SRAM disable mode
     case '6' : InterSramTest();      break;	// test internal SRAM
     case '7' : CacheConfigure();     break;	// configure cache

     case '8' : 
// show cache configuration ////////////////////////////////////////////////////
      Print("\n\nCache configurations.\n");
      PrintCacheConfig();   
      break;	

     case '9' : CpuClockConfigure();  break;	// Configure CLKCON
     case 'C' : CacheClear();         break;	// Disable Stall En,Cache,Write En, and clear cache RAM
     case 'B' : SearchPattern();      break;	// Search pattern
     case 'A' : March10NAllTest();    break;	//
     case 'V' : SysRegsView();        break;	//

// 
     case 'Q' : return;
     default  : break;
    }

   Print("\nPress any key to continue.\n");
   it = get_upper();
   if(it == 'Q')
    {
     break;
    } 
  } 
}


/*
 *  All 10N March Test
 *  March-10N Test for Tag-RAM, 4K-SRAM, 8K-SRAM, Set0, Set1
 */


int March10NAllTest(void)
{
 unsigned int * tag;		// cache tag RAM base pointer 
 unsigned int * sram;		// SRAM base pointer  
 unsigned int * set1;		// cache SET1 base pointer  
 unsigned int * set0;		// cache SET0 base pointer	

//
 int result = 1;		// return value 
 int tagerr = 0;  		// Tag memory error flag for march test int delay = 0;
 int sramerr = 0; 		// sram flag for march error
 int seterr = 0;  		// set0 flag for march error
 int i;

//////////////////////////////////////////////////////////////////////////////// 
// test cache TAG RAM ////////////////////////////////////////////////////////// 
//////////////////////////////////////////////////////////////////////////////// 

 tag  = (unsigned int *)TagRAM;

/* Cache off,wr buf off,stall off */

 SYSCFG &= 			// 
 ~(
   SYSCFG_WRITE_BUFF	|	// write buffer enable (WE)
   SYSCFG_CACHE		|	// cache enable (CE)
   SYSCFG_STALL			// stall enable (SE)
  );
 Print("\nTag(256words) 10N March Test       ..... ");
 
/* tag(256 words) test by 10N march */
 if(March10N32(tag,256,0x00000000,1) == 0) { tagerr = 1; }
 if(March10N32(tag,256,0x0000ffff,1) == 0) { tagerr = 1; }
 if(March10N32(tag,256,0x00ff00ff,1) == 0) { tagerr = 1; }
 if(March10N32(tag,256,0x0f0f0f0f,1) == 0) { tagerr = 1; }
 if(March10N32(tag,256,0x33333333,1) == 0) { tagerr = 1; }
 if(March10N32(tag,256,0x55555555,1) == 0) { tagerr = 1; }

 if(tagerr) 
  {
   Print("Fail.") ;
   result = 0;
  }
 else
  {
   Print("Ok.") ;
  }


//////////////////////////////////////////////////////////////////////////////// 
// test SRAM in 4Kb mode ///////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////// 

sram = (unsigned int *) (((SYSCFG >> 6) & 0x3FF) << 16);
// [15:6] = Internal SRAM Base Pointer  

Print("\nSRAM(4K bytes) 10N March Test      ..... ");

SYSCFG = 
 (SYSCFG & ~SYSCFG_CACHE_MASK) |	// 
  SYSCFG_CACHE_MODE_4KB_CACHE;		// 4 Kb cache, 4 Kb SRAM 

/* SRAM(2048 words) test by 10N march */

 if(March10N32(sram,1024,0x00000000,1) == 0) { sramerr = 1; }
 if(March10N32(sram,1024,0x0000ffff,1) == 0) { sramerr = 1; }
 if(March10N32(sram,1024,0x00ff00ff,1) == 0) { sramerr = 1; }
 if(March10N32(sram,1024,0x0f0f0f0f,1) == 0) { sramerr = 1; }
 if(March10N32(sram,1024,0x33333333,1) == 0) { sramerr = 1; }
 if(March10N32(sram,1024,0x55555555,1) == 0) { sramerr = 1; }

 if(sramerr) 
  {
   Print("Fail.");
   result = 0;
  }
 else 
  {
   Print("Ok.") ;
  }


//////////////////////////////////////////////////////////////////////////////// 
// test SRAM in 8Kb mode ///////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////// 

 sramerr = 0; 				// clear SRAM error flag 

 sram = (unsigned int *) (((SYSCFG >> 6) & 0x3FF) << 16);

 Print("\nSRAM(8K bytes) 10N March Test      ..... ");

 SYSCFG = 
  (SYSCFG & ~SYSCFG_CACHE_MASK) | 	// clear cache bits
   SYSCFG_CACHE_MODE_8KB_SRAM;		// 8Kb SRAM, 0Kb cache     

/* SRAM(2048 words) test by 10N march */

 if(March10N32(sram,2048,0x00000000,1) == 0) { sramerr = 1; }
 if(March10N32(sram,2048,0x0000ffff,1) == 0) { sramerr = 1; }
 if(March10N32(sram,2048,0x00ff00ff,1) == 0) { sramerr = 1; } 
 if(March10N32(sram,2048,0x0f0f0f0f,1) == 0) { sramerr = 1; } 
 if(March10N32(sram,2048,0x33333333,1) == 0) { sramerr = 1; } 
 if(March10N32(sram,2048,0x55555555,1) == 0) { sramerr = 1; } 

 if(sramerr) 
  {
   Print("Fail.");
   result = 0;
  }
 else 
  {
   Print("Ok.") ;
  }


////////////////////////////////////////////////////////////////////////////////
// test cache SET0/1 in 8kb mode ///////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

 Print("\nCache[SET1]4K bytes 10N March Test ..... ");

 SYSCFG = 
  (SYSCFG & ~SYSCFG_CACHE_MASK) | 	// WE off, SE off, CE off 
   SYSCFG_CACHE_MODE_8KB_CACHE;		// 0Kb SRAM, 8Kb cache 

 seterr = 0; 				// clear march error flag

 set1 = (unsigned int *) Set1CacheRAM;
 for(i=0; i < 1024; i++) 
  {
   *set1 = 0x00000000;   		// Clear set1 RAM 
    set1++; 				// 
  }
	
 set1 = (unsigned int *) Set1CacheRAM;
 for(i=0; i < 1024; i++) 
  {
   if(*set1 != 0x00000000) 
    {
     seterr = 1; 
     result = 0;
     Print("[CACHE:ERROR] SET1[0x%x] = 0x%x!!\n", set1, *set1);
    }
   set1++; 
  } 

 if(seterr == 0)
  {
   set1 = (unsigned int *)Set1CacheRAM;
/* SET1(1024 words) test by 10N march */
   
   if(March10N32(set1,1024,0x00000000,1) == 0) { seterr = 1; } 
   if(March10N32(set1,1024,0x0000ffff,1) == 0) { seterr = 1; }
   if(March10N32(set1,1024,0x00ff00ff,1) == 0) { seterr = 1; } 
   if(March10N32(set1,1024,0x0f0f0f0f,1) == 0) { seterr = 1; } 
   if(March10N32(set1,1024,0x33333333,1) == 0) { seterr = 1; } 
   if(March10N32(set1,1024,0x55555555,1) == 0) { seterr = 1; } 
  }


 if(seterr) 
  {
   Print("Fail.");
   result = 0;
  }
 else 
  {
   Print("Ok.");
  }

////////////////////////////////////////////////////////////////////////////////
// test cache SET0 in 8Kb mode /////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

 Print("\nCache[SET0]4K bytes 10N March Test ..... ");

 SYSCFG = 
  (SYSCFG & ~SYSCFG_CACHE_MASK) |	// WE off, SE off, CE off
   SYSCFG_CACHE_MODE_8KB_CACHE;		// 8 Kb Cache, 0 Kb SRAM

// Clear SET0 Cache SRAM

 set0 = (unsigned int *)Set0CacheRAM;	// cache set 0 base pointer 
 for(i=0; i < 1024; i++) 		// 1K words 
  {
   *set0 = 0x00000000;   		// Clear ram 
    set0++; 
  }

 seterr = 0; 				// clear march error flag
 set0 = (unsigned int *)Set0CacheRAM;
 for(i=0; i < 1024; i++) 
  {
   if(*set0 != 0x00000000) 
    {
     seterr = 1; 			// clear march error flag
     result = 0;
     Print("[CACHE:ERROR] SET0[0x%x] = 0x%x!!\n",set0, *set0);
    }
   set0++; 
  }


 if(seterr == 0)
  {
   set0 = (unsigned int *)Set0CacheRAM;

/* SET0(1024 words) test by 10N march */

   if(March10N32(set0,1024,0x00000000,1) == 0) { seterr = 1; }
   if(March10N32(set0,1024,0x0000ffff,1) == 0) { seterr = 1; }
   if(March10N32(set0,1024,0x00ff00ff,1) == 0) { seterr = 1; }
   if(March10N32(set0,1024,0x0f0f0f0f,1) == 0) { seterr = 1; }
   if(March10N32(set0,1024,0x33333333,1) == 0) { seterr = 1; }
   if(March10N32(set0,1024,0x55555555,1) == 0) { seterr = 1; }
  }

 if(seterr) 
  {
   Print("Fail.");
   result = 0;
  }
 else 
  {
   Print("Ok.");
  }

 return(result);
}


////////////////////////////////////////////////////////////////////////////////
// init cache //////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////


void CacheInit(int cmd)
{
/* Disable cache before cache mode change */  

 SYSCFG &= 
  ~(
    SYSCFG_STALL     |			// stall enable bit 
    SYSCFG_CACHE     |			// cache enable bit 
    SYSCFG_CACHE_MODE|			// cache mode bits
    SYSCFG_WRITE_BUFF			// write buffer enable bit
   ); 

 switch(cmd) 
  {
   case CACHEINIT_4K	: 		/* 4K Cache,4K SRAM */

    SYSCFG |=
     (
      SYSCFG_CACHE_MODE_4KB_CACHE |	// set cache mode 00
      SYSCFG_WRITE_BUFF			// enable write buffer 
     ); 
    CacheFlush();			// cache flush 
    SYSCFG |= SYSCFG_CACHE; 		/* Cache Enable */ 
    break; 

   case CACHEINIT_8K	: 		/* 8K Cache */

    SYSCFG |=
     (
      SYSCFG_CACHE_MODE_8KB_CACHE |	// set cache mode 01 
      SYSCFG_WRITE_BUFF			// enable write buffer 
     ); 
    CacheFlush();			// cache flush
    SYSCFG |= SYSCFG_CACHE; 		/* Cache Enable */ 
    break; 

   case  CACHEINIT_0K 	: 		/* CACHE OFF: 8K SRAM */

    SYSCFG |=
     (
      SYSCFG_CACHE_MODE_8KB_SRAM  | 	// set cache mode 10
      SYSCFG_WRITE_BUFF			// enable write buffer 	
     );  
    CacheFlush();			// cache flush 

   default : break;
  }                
}



